// ==============================================================
//
// f9h_fair_2
// Control module for f9h_fair_2 vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include <cstdio>
#include <cmath>
#include <string>
#include <stdio.h>
#include "OrbiterSoundSDK40.h"

// ==============================================================
// Some vessel parameters
// ==============================================================


const double FAIR_DRYMASS = 875;
const double RCS_ISP = 3000;
const double RCS_TH = 150;
const double CHUTE_SPEED = 0.25;
const double RCS_FUEL = 40;

const double M_E = 2.71828182845904523536;

static const DWORD ntdvtx_geardown = 4;
static TOUCHDOWNVTX tdvtx_geardown[ntdvtx_geardown] = {
	{_V(    0, -6.5, 7), 1e5, 1e5, 2, 2},
	{_V(-2.5,-6.5,-5), 1e5, 1e5, 2, 2},
	{_V( 2.5,-6.5,-5), 1e5, 1e5, 2, 2},
	{_V(   0 ,0 ,   20), 1e5, 1e5, 1}
};

//#define GREEN RGB(125, 255, 125)

void VLiftCoeffHull (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0};


	aoa += PI;
	int idx = max (0, min (15, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 1*(0.1 + oapiGetInducedDrag (*cl, 0.5, 0.2));

	
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeffHull (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 1*(0.1 + oapiGetInducedDrag (*cl, 0.5, 0.2));
	
}

//void VLiftCoeff (VESSEL *v, double aoa, double M, double Re, void *context, double *cl, double *cm, double *cd)
void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 9;
	static const double AOA[nabsc] = {-180*RAD,-60*RAD,-30*RAD, -2*RAD, 10*RAD,15*RAD,50*RAD,90*RAD,180*RAD};
	static const double CL[nabsc]  = {       0,      0,   -0.4,      0,    0.3,     0.6,   0.8,     0,      0};
	static const double CM[nabsc]  = {       0,      0,  0.014, 0.0039, -0.006,-0.008,-0.010,     0,      0};
	for (i = 0; i < nabsc-1 && AOA[i+1] < aoa; i++);
	double f = (aoa-AOA[i]) / (AOA[i+1]-AOA[i]);
	*cl = CL[i] + (CL[i+1]-CL[i]) * f;  // aoa-dependent lift coefficient
	*cm = CM[i] + (CM[i+1]-CM[i]) * f;  // aoa-dependent moment coefficient
	double saoa = sin(aoa);
	double pd = 0.02 + 0.4*saoa*saoa;  // profile drag
	*cd = pd + oapiGetInducedDrag (*cl, 1.5, 0.7) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
}

// 2. horizontal lift component (vertical stabilisers and body)

//void HLiftCoeff (VESSEL *v, double beta, double M, double Re, void *context, double *cl, double *cm, double *cd)
void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = {-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
	static const double CL[nabsc]   = {       0,    +0.3,      0,   -0.4,  +0.4,     0,   -0.3,      0};
	for (i = 0; i < nabsc-1 && BETA[i+1] < beta; i++);
	*cl = CL[i] + (CL[i+1]-CL[i]) * (beta-BETA[i]) / (BETA[i+1]-BETA[i]);
	*cm = 0.0;
	*cd = 0.02 + oapiGetInducedDrag (*cl, 1.5, 0.6) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
}

// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class f9h_fair_2: public VESSEL3 {
public:
	f9h_fair_2 (OBJHANDLE hVessel, int flightmodel);
	~f9h_fair_2 ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostCreation();
	void clbkPreStep(double SimT, double SimDT, double MJD);
	//void clbkPostStep (double simt, double simdt, double mjd);
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	int MyID;
	//bool clbkLoadGenericCockpit ();
	
private:

	void DefineAnimations (void);
	void CreateParafoil(void);
	void EditParafoil(void);
	void GoToWaypoint(void);
	void GoToTarget(void);
	void GetWindData(void);
	void CalcTargetVectorOffset(void);
	void SetBankAngle(void);
	void GoToFlightVector(void);
	void RollToHeadVector(void);
	void CalcFlightVectors(void);
	void KillRCS(void);
	void CalcPitchBank(void);

	THRUSTER_HANDLE th_rcs[15], th_group[3];
	PROPELLANT_HANDLE hProp;
	int chute_status, aero_status, wind_data_sw, auto_status;
	double avg_sink_rate, avg_air_speed, tgt_height_offset;
	double hud_range, hud_tgt_dist;
	double chute_proc, aero_proc;
	double steptime, bank_offset, bank_level;
	VECTOR3 wind_data[61], wind_data_total;
	VECTOR3 target_vector_offset;
	VECTOR3 flight_vector, head_vector;
	double bank_angle, pitch_angle;
	MGROUP_TRANSFORM *Chute_scale;
	UINT anim_chute;
	AIRFOILHANDLE hwing, htail;
	OBJHANDLE hTarget;
};

f9h_fair_2::f9h_fair_2 (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
    

}

f9h_fair_2::~f9h_fair_2 ()
{
	delete Chute_scale;
}

void f9h_fair_2::DefineAnimations (void)
{
	anim_chute = CreateAnimation (0);

	static UINT Chutegrp[1] = {0};
	Chute_scale = new MGROUP_SCALE(1, Chutegrp, 1, _V(0,-1.197,-4.685), _V(0.0001,0.0001,0.0001) );
	AddAnimationComponent (anim_chute, 0, 1, Chute_scale);
}

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void f9h_fair_2::clbkSetClassCaps (FILEHANDLE cfg)
{
	chute_status = 0;
	chute_proc = 1; //folded
	wind_data_sw = 0;
	avg_sink_rate = 3.52;
	avg_air_speed = 17.5;
	tgt_height_offset = 29.6;
	wind_data_total = _V(0,0,0);
	int i;
	for(i = 0; i < 61; i++)
	{
	wind_data[i] = _V(0,0,0);
	}
	target_vector_offset = _V(0,0,0);
	flight_vector = _V(0,0,0);
	head_vector = _V(0,0,0);
	steptime = 0;
	bank_offset = 0;
	bank_level = 0;
	aero_status = 0;
	aero_proc = 0;
	auto_status = 0;
	hud_range = 0;
	hud_tgt_dist = 0;
	bank_angle = 0;
	pitch_angle = 0;

	DefineAnimations();
	// physical vessel parameters
	SetEmptyMass (FAIR_DRYMASS);
	SetSize (7);
	SetPMI(_V(16,16,2));
	SetCrossSections (_V(33,66,11));
	//SetTouchdownPoints ( _V(0,-5, -5), _V(-5, 3, -5), _V( 5, 3,-5));
	SetTouchdownPoints (tdvtx_geardown,ntdvtx_geardown);
	
	SetCW(0.8,0.8,1,1);
	SetRotDrag (_V(0.05,0.05,0.05));


	// camera parameters
	SetCameraOffset (_V(0,3,0));
	SetCameraRotationRange(PI,PI,(0.4*PI),(0.4*PI));


	SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_hull"),MESHVIS_ALWAYS);
	SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_para"),MESHVIS_NEVER);

	////////// thrusters
	hProp = CreatePropellantResource (RCS_FUEL,RCS_FUEL);


	th_rcs[0] = CreateThruster (_V(-2.567,0.99,-4.467), _V(-0.7071,-0.7071,0), RCS_TH, hProp, RCS_ISP, RCS_ISP); //PUP
	th_rcs[1] = CreateThruster (_V( 2.567,0.99,-4.467), _V( 0.7071,-0.7071,0), RCS_TH, hProp, RCS_ISP, RCS_ISP); //PUP
	th_rcs[2] = CreateThruster (_V(-0.057,0.99, 7.774), _V( 0.7071,-0.7071,0), RCS_TH*0.5746, hProp, RCS_ISP, RCS_ISP); //PDOWN
	th_rcs[3] = CreateThruster (_V( 0.057,0.99, 7.774), _V(-0.7071,-0.7071,0), RCS_TH*0.5746, hProp, RCS_ISP, RCS_ISP); //PDOWN

	th_rcs[4] = CreateThruster (_V(-2.567,0.99,-4.467), _V(-0.7071,-0.7071,0), RCS_TH*1, hProp, RCS_ISP, RCS_ISP); //YAWR
	th_rcs[5] = CreateThruster (_V( 2.567,0.99,-4.467), _V( 0.7071,-0.7071,0), RCS_TH*0.713, hProp, RCS_ISP, RCS_ISP); //YAWR
	th_rcs[6] = CreateThruster (_V(-0.057,0.99, 7.774), _V( 0.7071,-0.7071,0), RCS_TH*0.983, hProp, RCS_ISP, RCS_ISP); //YAWR

	th_rcs[7] = CreateThruster (_V(-2.567,0.99,-4.467), _V(-0.7071,-0.7071,0), RCS_TH*0.713, hProp, RCS_ISP, RCS_ISP); //YAWL
	th_rcs[8] = CreateThruster (_V( 2.567,0.99,-4.467), _V( 0.7071,-0.7071,0), RCS_TH*1, hProp, RCS_ISP, RCS_ISP); //YAWL
	th_rcs[9] = CreateThruster (_V( 0.057,0.99, 7.774), _V(-0.7071,-0.7071,0), RCS_TH*0.983, hProp, RCS_ISP, RCS_ISP); //YAWL


	th_rcs[10] = CreateThruster (_V(-0.057,0.99, 7.774), _V( 0.7071,-0.7071,0), RCS_TH*0.575, hProp, RCS_ISP, RCS_ISP); //BANKR
	th_rcs[11] = CreateThruster (_V( 2.567,0.99,-4.467), _V( 0.7071,-0.7071,0), RCS_TH, hProp, RCS_ISP, RCS_ISP); //BANKR
	th_rcs[12] = CreateThruster (_V( 0.057,0.99, 7.774), _V(-0.7071,-0.7071,0), RCS_TH*0.575, hProp, RCS_ISP, RCS_ISP); //BANKL
	th_rcs[13] = CreateThruster (_V(-2.567,0.99,-4.467), _V(-0.7071,-0.7071,0), RCS_TH, hProp, RCS_ISP, RCS_ISP); //BANKL
	


	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[1];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[2];
	th_group[1] = th_rcs[3];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[4];
	th_group[1] = th_rcs[5];
	th_group[2] = th_rcs[6];
	CreateThrusterGroup (th_group, 3, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[7];
	th_group[1] = th_rcs[8];
	th_group[2] = th_rcs[9];
	CreateThrusterGroup (th_group, 3, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[10];
	th_group[1] = th_rcs[11];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[12];
	th_group[1] = th_rcs[13];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);
	
	/*SURFHANDLE exhaust_rcs = oapiRegisterExhaustTexture ("exhaust_atrcs");

	double exlenrcs = 3;
	double exwidrcs = 0.3;

	AddExhaust(th_rcs[0],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs[1],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs[2],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs[3],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs[4],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs[5],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs[6],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs[7],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs[8],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs[9],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs[10],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs[11],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs[12],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs[13],exlenrcs,exwidrcs,exhaust_rcs);*/
	//////////////// pstream

	SURFHANDLE exhaustrcs2 = oapiRegisterParticleTexture ("Contrail1");

	PARTICLESTREAMSPEC exhaust_rcs2 = {
		0, 0.04, 10, 10, 0.05, 0.05, 4, 1, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0.5, 0.5,
		PARTICLESTREAMSPEC::ATM_FLAT, 0.5, 0.5,
		exhaustrcs2
	};

	int j;
	for(j = 0; j < 14; j++)
	{
	AddExhaustStream(th_rcs[j],&exhaust_rcs2);
	}

	/////////////////////////////////////// airfoils

	CreateAirfoil (LIFT_VERTICAL, _V(0,0,0), VLiftCoeffHull, 10, 20, 0.5);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeffHull, 10, 10, 0.5);
	

}

void f9h_fair_2::clbkPostCreation ()
{
	if(chute_status > 0) 
	{
		SetMeshVisibilityMode(1,MESHVIS_ALWAYS);
		CreateParafoil();
		EditParafoil();
	}

	/////////////// sound
	MyID=ConnectToOrbiterSoundDLL(GetHandle());
	SoundOptionOnOff(MyID,PLAYRADIOATC,FALSE);
	//SoundOptionOnOff3(MyID,PLAYWHENATTITUDEMODECHANGE,FALSE);
	SoundOptionOnOff(MyID,PLAYGPWS,FALSE);
	SoundOptionOnOff(MyID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);
	SoundOptionOnOff(MyID,PLAYLANDINGANDGROUNDSOUND,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINAIRCONDITIONING,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINRANDOMAMBIANCE,FALSE);
	SoundOptionOnOff(MyID,DISPLAYTIMER,FALSE);
	SoundOptionOnOff(MyID,PLAYDOCKLANDCLEARANCE,FALSE);
	SoundOptionOnOff(MyID,PLAYRADARBIP,FALSE);
}

int f9h_fair_2::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{
if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) 
  {	 
	switch (key)
	{
	
	} 
  }

  else if (KEYMOD_CONTROL (kstate)) 
  { 
	switch (key)
	{
		
	} 
  }

else 
  { // unmodified keys
    switch (key) 
	{
		case OAPI_KEY_P:
			if (auto_status == 0) // 0 = ON
			{
				auto_status = 1;
				KillRCS();
			}
			else if (auto_status == 1)auto_status = 0;
		return 1;

	}
  }
  return 0;
}


bool f9h_fair_2::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);

	//int chw = 16;

	
	char test[256];
	strcpy(test,"abcdefghijklmnopqrstuvwxyz");
	int it = skp->GetTextWidth(test, 26);
	it = (int)(it/26);
	int chw = it;

	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}



	int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	/*int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	int w3 = (int)(232*d);
	int w4 = (int)(248*d);
	int w5 = (int)(264*d);
	int w6 = (int)(280*d);
	int w7 = (int)(296*d);
	int w8 = (int)(312*d);
	int w9 = (int)(328*d);*/
	//int w10 = (int)(344*d);
	//int w11 = (int)(360*d);
	//int w12 = (int)(376*d);
	//int w13 = (int)(392*d);
	//int w14 = (int)(408*d);
	//int w15 = (int)(424*d);

	if(auto_status == 0)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"P = Autopilot On/Off");
	skp->Text ((10+roxl), (wd+royl), abuf,strlen(abuf));
	}
	else
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"P = Autopilot On/Off");
	skp->Text ((10+roxl), (wd+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Target Dist. %0.3fkm",(hud_tgt_dist*0.001));
	skp->Text ((10+roxl), (wc+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Flight Range. %0.3fkm",(hud_range*0.001));
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}


	return true;
}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new f9h_fair_2 (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (f9h_fair_2*)vessel;
}


void f9h_fair_2::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState(scn);

	sprintf (cbuf, "%d %0.4f %d %0.4f", chute_status, chute_proc, aero_status , aero_proc);
	oapiWriteScenario_string (scn, "CHUTE", cbuf);

	sprintf (cbuf, "%d", auto_status);
	oapiWriteScenario_string (scn, "AUTP", cbuf);



}

void f9h_fair_2::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "CHUTE", 5)) 
		{
			sscanf (line+5, "%d%lf%d%lf", &chute_status, &chute_proc, &aero_status , &aero_proc);
		}
		else if (!strnicmp (line, "AUTP", 4)) 
		{
			sscanf (line+4, "%d", &auto_status);
		}
		else
		{
			ParseScenarioLineEx (line, vs);// unrecognised option - pass to Orbiter's generic parser
        }
   }

	SetAnimation(anim_chute, chute_proc);

}



void f9h_fair_2::clbkPreStep(double SimT, double SimDT, double MJD)
{
	steptime = SimDT;
	double alt = GetAltitude(ALTMODE_GROUND);

	hTarget = oapiGetObjectByName("FRV");
	if(wind_data_sw == 0)GetWindData();

	///////// self destruct if landed or ejected
	if(alt < 7)
	{
		oapiDeleteVessel(GetHandle());
		return;
	}
	else if(alt > 10000000)
	{
		oapiDeleteVessel(GetHandle());
		return;
	}

	/////////////////////////// deploy chutes, update aerodynamics;
	if (alt < 6500 && alt > 1000)
	{
		if(chute_status == 0)
		{
			chute_status = 1;
			aero_status = 1;
			SetMeshVisibilityMode(1,MESHVIS_ALWAYS);
			CreateParafoil();
			KillRCS();
		}
	}

	////////////////////////// autopilot mode
	if(auto_status == 0)
	{
	DeactivateNavmode(NAVMODE_KILLROT);
	DeactivateNavmode(NAVMODE_HLEVEL);
	DeactivateNavmode(NAVMODE_PROGRADE);
	DeactivateNavmode(NAVMODE_RETROGRADE);
	DeactivateNavmode(NAVMODE_NORMAL);
	DeactivateNavmode(NAVMODE_ANTINORMAL);
	DeactivateNavmode(NAVMODE_HOLDALT);

	if (alt < 200) GoToTarget(); // final approach
	else if (alt < 6000) GoToWaypoint(); // aero steering
	else if (alt > 6500)// reentry guidance
		{
			CalcPitchBank();
			CalcFlightVectors();
			GoToFlightVector();
			RollToHeadVector();
		}
	}

	////////////// animations
	if(chute_status == 0) chute_proc = 1;
	else if (chute_status == 1)
	{
		double da = SimDT * 2 ;
		chute_proc = chute_proc - da;
		if (chute_proc < 0)
		{
			chute_proc = 0;
			chute_status = 2;
			KillRCS();
		}
	}
	else if (chute_status == 2)chute_proc = 0;
	SetAnimation(anim_chute, chute_proc);

	/////////// aerodynamics status
	if(aero_status == 0) aero_proc = 0;
	else if (aero_status == 1)
	{
		double da = SimDT * CHUTE_SPEED;
		aero_proc = aero_proc + da;
		if (aero_proc > 1)
		{
			aero_proc = 1;
			aero_status = 2;
		}
		EditParafoil();
	}
	else if (aero_status == 2)aero_proc = 1;
	////////////////////////////

	//sprintf(oapiDebugString(), "chute_status %d chute_proc %0.4f", chute_status, chute_proc);
}

void f9h_fair_2::KillRCS(void)
{
	int i;
	for(i = 0; i < 14; i++)
	{

		SetThrusterLevel(th_rcs[i],0);
	}

}

void f9h_fair_2::CreateParafoil(void)
{
	ClearAirfoilDefinitions ();
	hwing = CreateAirfoil2 (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff,  0.001, 0.001, 1.5); // 0 5 90
	htail = CreateAirfoil2 (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 0.001, 0.001, 1.5); // 0 5 15
	SetSize (20);
	SetPMI(_V(400,40,150));
	SetCrossSections (_V(80,400,40));
	SetCW (2, 0.8, 0.8, 0.8);
	SetRotDrag (_V(0.7,0.35,0.35));
	ShiftCG(_V(0,5,0));

	CreateControlSurface (AIRCTRL_AILERON, 5, 1.5, _V(10, 0, -1), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_AILERON, 5, 1.5, _V(-10, 0, -1), AIRCTRL_AXIS_XNEG);
	CreateControlSurface3 (AIRCTRL_ELEVATOR, 10, 2, _V(0, 0, -20), AIRCTRL_AXIS_XPOS,1.5);

}

void f9h_fair_2::EditParafoil(void)
{
	double pp = aero_proc;
	EditAirfoil(hwing,11101,_V(0,(7*pp),(-0.6*pp)),VLiftCoeff,(10*pp),(270*pp),1.5);
	EditAirfoil(htail,11101,_V(0,(3*pp),(-20*pp)),HLiftCoeff,(10*pp),(60*pp),1.5);
}

void f9h_fair_2::GoToWaypoint(void)
{
	if(hTarget == NULL)return;

	double alt = GetAltitude(ALTMODE_GROUND);
	if(alt < tgt_height_offset)return;

	if(wind_data_sw == 0)GetWindData();

	VECTOR3 tgt_pos_glob;
	oapiGetGlobalPos(hTarget, &tgt_pos_glob);
	VECTOR3 tgt_pos_local;
	Global2Local(tgt_pos_glob, tgt_pos_local);
	VECTOR3 tgt_pos_hrz;
	HorizonRot(tgt_pos_local, tgt_pos_hrz);

	double tgt_headg;
	oapiGetHeading(hTarget,&tgt_headg);
	VECTOR3 att_pnt_offset = _V(0,0,0);
	att_pnt_offset.x = -12.5*(sin(tgt_headg));//22.5
	att_pnt_offset.z = -12.5*(cos(tgt_headg));


	tgt_pos_hrz = tgt_pos_hrz + att_pnt_offset;
	tgt_pos_hrz.y = tgt_pos_hrz.y + tgt_height_offset;

	VECTOR3 ground_spd_vector;
	GetGroundspeedVector(FRAME_HORIZON, ground_spd_vector);

	VECTOR3 air_spd_vector;
	GetAirspeedVector(FRAME_HORIZON, air_spd_vector);

	
	CalcTargetVectorOffset();
	tgt_pos_hrz = tgt_pos_hrz + target_vector_offset;
	

	//sprintf(oapiDebugString(), "tgt_pos_hrz.y %0.1f", tgt_pos_hrz.y);

	double tgt_dist = length(_V(tgt_pos_hrz.x, 0, tgt_pos_hrz.z));

	avg_sink_rate = (3.8 - ground_spd_vector.y)*0.5;
	double flight_time = (alt-tgt_height_offset-40)/avg_sink_rate;
	double air_spd = length(_V(air_spd_vector.x, 0, air_spd_vector.z));
	avg_air_speed = (12 + air_spd)*0.5;
	double range = air_spd*flight_time;

	double tgt_hdg = atan2(tgt_pos_hrz.x,tgt_pos_hrz.z);
	if(tgt_hdg < 0)tgt_hdg = tgt_hdg + PI2;

	double tgt_range_ratio = tgt_dist/range;
	hud_tgt_dist = tgt_dist;
	hud_range = range;
	//sprintf(oapiDebugString(), "tgt_dist %0.2f range %0.2f",tgt_dist, range);
	if(tgt_range_ratio > 1) tgt_range_ratio = 1;
	else if(tgt_range_ratio < 0) tgt_range_ratio = 0;

	double req_hdg_offset = acos(tgt_range_ratio);
	double req_hdg = tgt_hdg + req_hdg_offset;

	double my_hdg;
	oapiGetHeading(GetHandle(), &my_hdg);

	int left_is_neg = -1; // 1 is right

	if(req_hdg < my_hdg)
	{
		if((my_hdg - req_hdg) < PI)left_is_neg = -1;
		else left_is_neg = 1;
	}
	else if(req_hdg > my_hdg)
	{
		if((req_hdg - my_hdg) < PI)left_is_neg = 1;
		else left_is_neg = -1;
	}

	double hdg_error = abs(my_hdg - req_hdg);
	if (hdg_error > PI)hdg_error = PI2 - hdg_error;

	bank_level = hdg_error/(PI*0.5);
	if(bank_level > 1)bank_level = 1;
	bank_level = bank_level * left_is_neg;

	
	//SetControlSurfaceLevel(AIRCTRL_AILERON, bank_level*left_is_neg);
	SetBankAngle();

	//double my_pitch = GetPitch();
	double pitch_control = (-3.8 - ground_spd_vector.y)*0.25;
	pitch_control = pitch_control + (bank_level*0.25);
	if (pitch_control > 1)pitch_control = 1;
	else if (pitch_control < -1)pitch_control = -1;


	SetControlSurfaceLevel(AIRCTRL_ELEVATOR, pitch_control);


	//sprintf(oapiDebugString(), "wind_dir %0.1f req_hdg %0.1f my_hdg %0.1f", wind_dir*DEG, req_hdg*DEG, my_hdg*DEG);
}

void f9h_fair_2::GoToTarget(void)
{
	
	if(hTarget == NULL)return;

	double alt = GetAltitude(ALTMODE_GROUND);
	if(alt < tgt_height_offset)return;

	if(wind_data_sw == 0)GetWindData();

	VECTOR3 tgt_pos_glob;
	oapiGetGlobalPos(hTarget, &tgt_pos_glob);
	VECTOR3 tgt_pos_local;
	Global2Local(tgt_pos_glob, tgt_pos_local);
	VECTOR3 tgt_pos_hrz;
	HorizonRot(tgt_pos_local, tgt_pos_hrz);

	double tgt_headg;
	oapiGetHeading(hTarget,&tgt_headg);
	VECTOR3 att_pnt_offset = _V(0,0,0);
	att_pnt_offset.x = -12.5*(sin(tgt_headg));//22.5
	att_pnt_offset.z = -12.5*(cos(tgt_headg));
	att_pnt_offset.y = tgt_height_offset;


	tgt_pos_hrz = tgt_pos_hrz + att_pnt_offset;
	//tgt_pos_hrz.y = tgt_pos_hrz.y + tgt_height_offset;

	VECTOR3 ground_spd_vector;
	GetGroundspeedVector(FRAME_HORIZON, ground_spd_vector);

	VECTOR3 air_spd_vector;
	GetAirspeedVector(FRAME_HORIZON, air_spd_vector);

	if(alt > tgt_height_offset)
	{
		CalcTargetVectorOffset();
		tgt_pos_hrz = tgt_pos_hrz + target_vector_offset;
	}

	//sprintf(oapiDebugString(), "tgt_pos_hrz.y %0.1f", tgt_pos_hrz.y);

	double tgt_dist = length(_V(tgt_pos_hrz.x, 0, tgt_pos_hrz.z));

	avg_sink_rate = (3.8 - ground_spd_vector.y)*0.5;
	double flight_time = 0.9*(alt-tgt_height_offset)/avg_sink_rate;
	double air_spd = length(_V(air_spd_vector.x, 0, air_spd_vector.z));
	double grd_spd = length(_V(ground_spd_vector.x,0,ground_spd_vector.z));
	avg_air_speed = (12 + air_spd)*0.5;
	double range = air_spd*flight_time;


	double aspda = atan2(-ground_spd_vector.y,air_spd);

	double req_aspda = atan2((alt - tgt_height_offset),tgt_dist);

	double aspda_diff = aspda - req_aspda;

	double tgt_hdg = atan2(tgt_pos_hrz.x,tgt_pos_hrz.z);
	double grnd_spd_hdg = atan2(ground_spd_vector.x, ground_spd_vector.z);
	if(tgt_hdg < 0)tgt_hdg = tgt_hdg + PI2;
	if(grnd_spd_hdg < 0)grnd_spd_hdg = grnd_spd_hdg + PI2;

	//sprintf(oapiDebugString(), "tgt_dist %0.2f range %0.2f sinkangle %0.2f  req_aspda %0.2f aspda_diff %0.2f ",tgt_dist, range, -aspda*DEG, -req_aspda*DEG, -aspda_diff*DEG, GetBank());

	
	double req_grnd_spd_hdg = tgt_hdg;

	double my_hdg;
	oapiGetHeading(GetHandle(), &my_hdg);
	//if(alt < 200)my_hdg = grnd_spd_hdg;

	if(alt > 100)
	{
	double tgt_range_ratio = tgt_dist/range;
	if(tgt_range_ratio > 1) tgt_range_ratio = 1;
	double req_hdg_offset = acos(tgt_range_ratio)*1;
	req_grnd_spd_hdg = tgt_hdg + req_hdg_offset;
	}

	hud_tgt_dist = tgt_dist;
	hud_range = range;

	int left_is_neg = -1; // 1 is right

	if(req_grnd_spd_hdg < my_hdg)
	{
		if((my_hdg - req_grnd_spd_hdg) < PI)left_is_neg = -1;
		else left_is_neg = 1;
	}
	else if(req_grnd_spd_hdg > my_hdg)
	{
		if((req_grnd_spd_hdg - my_hdg) < PI)left_is_neg = 1;
		else left_is_neg = -1;
	}

	double hdg_error = abs(my_hdg - req_grnd_spd_hdg);
	if (hdg_error > PI)hdg_error = PI2 - hdg_error;

	bank_level = hdg_error/(PI*0.15);
	if(bank_level > 1)bank_level = 1;
	bank_level = bank_level*left_is_neg;
	
	//SetControlSurfaceLevel(AIRCTRL_AILERON, bank_level*left_is_neg);
	SetBankAngle();

	//double my_pitch = GetPitch();
	//double pitch_control = 0;// -(aspda_diff*DEG)*0.2;
	double pitch_control = (-3.8 - ground_spd_vector.y)*0.25;
	pitch_control = pitch_control + (bank_level*0.25);
	if (pitch_control > 1)pitch_control = 1;
	else if (pitch_control < -1)pitch_control = -1;

	if(alt < 60)
	{
	pitch_control = (aspda_diff*DEG)*0.02;
	if (pitch_control > 0.2)pitch_control = 0.2;
	else if (pitch_control < -0.2)pitch_control = -0.2;
	//if(tgt_dist < 40)pitch_control = 0.5;
	SetControlSurfaceLevel(AIRCTRL_ELEVATOR, pitch_control,false);
	}
	else SetControlSurfaceLevel(AIRCTRL_ELEVATOR, pitch_control,true);

	//sprintf(oapiDebugString(), "wind_dir %0.1f req_hdg %0.1f my_hdg %0.1f", wind_dir*DEG, req_hdg*DEG, my_hdg*DEG);
}


void f9h_fair_2::SetBankAngle(void)
{
	double current_bank = -GetBank();
	double max_bank = 15*RAD;
	double req_bank = bank_level * max_bank;
	VECTOR3 angv;
	GetAngularVel(angv);
	double bank_diff = req_bank - current_bank;
	double req_bank_vel = bank_diff * 5;
	double bank_vel_diff = DEG*(req_bank_vel - (-angv.z));
	double surf_level = bank_vel_diff/20;
	if(surf_level > 1)surf_level = 1;
	else if(surf_level < -1)surf_level = -1;

	SetControlSurfaceLevel(AIRCTRL_AILERON, surf_level);
	//sprintf(oapiDebugString(), "req_bank %0.2f current_bank %0.2f bank_vel %0.2f  req_bank_vel %0.2f surf_level %0.2f  ",req_bank*DEG, current_bank*DEG, angv.z*DEG, req_bank_vel*DEG, surf_level);
}


void f9h_fair_2::GetWindData(void)
{
	if(hTarget == NULL)return;

	double lon,lat,rad = 0;
	oapiGetEquPos(hTarget, &lon, &lat, &rad);
	double windspeed = 0;
	OBJHANDLE hEarth = GetSurfaceRef();

	//double alt = GetAltitude(ALTMODE_GROUND);

	int i;
	for(i = 1; i < 61; i++)
	{
	wind_data[i] = oapiGetWindVector(hEarth,lon, lat, (i*100), 0, &windspeed );
	if (length(wind_data[i]) == 0) wind_data[i] = _V(-0.00001,0,0);
	}

	wind_data_sw = 1;
}

void f9h_fair_2::CalcTargetVectorOffset(void)
{
	if(hTarget == NULL)return;

	double section_time = 28; //100/avg_sink_rate;// av sr 3.8m/s
	double alt = GetAltitude(ALTMODE_GROUND);

	target_vector_offset = _V(0,0,0);
	wind_data_total = _V(0,0,0);

	if(alt > 200)
	{
	int imax = (int)(alt*0.01);
	if(imax > 59)imax = 59;

	int i;
	for(i = 0; i < (imax+1); i++)
		{
		wind_data_total = wind_data_total + wind_data[i];
		}

	double section_fraction = (alt - (imax*100))*0.01;
	VECTOR3 wind_data_fraction;
	wind_data_fraction.x = wind_data[(imax+1)].x*section_fraction;
	wind_data_fraction.y = wind_data[(imax+1)].y*section_fraction;
	wind_data_fraction.z = wind_data[(imax+1)].z*section_fraction;

	wind_data_total = wind_data_total + wind_data_fraction;
	wind_data_total.x = wind_data_total.x * section_time;
	wind_data_total.y = wind_data_total.y * section_time;
	wind_data_total.z = wind_data_total.z * section_time;

	target_vector_offset = -wind_data_total;

	///////////////////////////////////////////
	
	VECTOR3 ground_spd_vector;
	GetGroundspeedVector(FRAME_HORIZON, ground_spd_vector);

	double final_time = (200 - tgt_height_offset)/(avg_sink_rate+0);
	double grd_speed = length(_V(ground_spd_vector.x,0,ground_spd_vector.z));
	double final_range = final_time*avg_air_speed;

	VECTOR3 wind_data_final = _V(0,0,0);
	for(i = 0; i < 2; i++)
	{
	wind_data_final = wind_data_final + wind_data[i];
	}
	wind_data_final.x = wind_data_final.x;
	wind_data_final.y = wind_data_final.y;
	wind_data_final.z = wind_data_final.z;

	double wind_offset = length(wind_data_final);
	VECTOR3 wind_data_final_unit = wind_data_final;
	normalise(wind_data_final_unit);

	target_vector_offset.x = target_vector_offset.x + (-wind_data_final_unit.x * 50);
	target_vector_offset.y = target_vector_offset.y;
	target_vector_offset.z = target_vector_offset.z + (-wind_data_final_unit.z * 50);
	
	}
	/////////////////////////////////
	else// if(alt < 200)
	{
	OBJHANDLE hMe = GetHandle();
	double lon,lat,rad = 0;
	oapiGetEquPos(hMe, &lon, &lat, &rad);
	double windspeed = 0;
	OBJHANDLE hEarth = GetSurfaceRef();
	VECTOR3 current_wind = oapiGetWindVector(hEarth,lon, lat, (rad-6371010), 0, &windspeed );

	double total_flt_time = (alt-tgt_height_offset)/(avg_sink_rate);

	VECTOR3 target_wind_offset = current_wind;
	target_wind_offset.x = -target_wind_offset.x*total_flt_time;
	target_wind_offset.y = -target_wind_offset.y*total_flt_time;
	target_wind_offset.z = -target_wind_offset.z*total_flt_time;

	target_vector_offset = target_wind_offset;
	}
}


void f9h_fair_2::GoToFlightVector(void)
{
	/*DeactivateNavmode(NAVMODE_KILLROT);
	DeactivateNavmode(NAVMODE_HLEVEL);
	DeactivateNavmode(NAVMODE_PROGRADE);
	DeactivateNavmode(NAVMODE_RETROGRADE);
	DeactivateNavmode(NAVMODE_NORMAL);
	DeactivateNavmode(NAVMODE_ANTINORMAL);
	DeactivateNavmode(NAVMODE_HOLDALT);*/

	double angoa = atan2(-flight_vector.y, flight_vector.z);
	double slipage = atan2(flight_vector.x, flight_vector.z);

	if(angoa > PI)angoa = (PI*2)-angoa;
	else if(angoa < -PI)angoa = (PI*2)+angoa;

	if(slipage > PI)slipage = (PI*2)-slipage;
	else if(slipage < -PI)slipage = (PI*2)+slipage;

	double mass = GetMass();
	VECTOR3 pmi;
	GetPMI(pmi);
	double inrt_x = pmi.x * mass;
	double inrt_y = pmi.y * mass;
	//double inrt_z = pmi.z * mass;

	double max_torq_x = 2 * 4.467 * 0.7071 * RCS_TH;
	double max_torq_y = RCS_TH*0.7071*((4.467 * 1) + (4.467 * 0.713) + (7.774*0.983));
	//double max_torq_z = RCS_TH*0.575*((-0.057 * 0.7071) + (0.99 * 0.7071));
	//max_torq_z = max_torq_z + (2.567 * 0.7071 * RCS_TH) + (0.99 * 0.7071 * RCS_TH);

	VECTOR3 angv;
	GetAngularVel(angv);
	double vrot_lim = 5*RAD;

	double max_acc_x = max_torq_x/inrt_x;
	double max_drot_x = max_acc_x*steptime;

	double max_acc_y = max_torq_y/inrt_y;
	double max_drot_y = max_acc_y*steptime;

	double reqxv = 1*abs(angoa); //1
	if (reqxv > vrot_lim)reqxv = vrot_lim;
	if (angoa > 0)reqxv = -reqxv;
	double req_drotx = reqxv - angv.x;
	double thx_lvl = req_drotx/max_drot_x;
	if((angoa*DEG) < 0.5 && (angoa*DEG) > -0.5) thx_lvl = 0.25*thx_lvl;

	double reqyv = 1*abs(slipage);//1
	if (reqyv > vrot_lim)reqyv = vrot_lim;
	if (slipage > 0)reqyv = -reqyv;
	double req_droty = reqyv - angv.y;
	double thy_lvl = req_droty/max_drot_y;
	if((slipage*DEG) < 0.5 && (slipage*DEG) > -0.5) thy_lvl = 0.25*thy_lvl;

	if(thx_lvl > 1)thx_lvl = 1;
	else if(thx_lvl < -1)thx_lvl = -1;

	if(thy_lvl > 1)thy_lvl = 1;
	else if(thy_lvl < -1)thy_lvl = -1;


	if(thx_lvl > 0)
		{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, thx_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
		}
	else if(thx_lvl < 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, -thx_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
	}
	else 
	{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
	}


	if(thy_lvl > 0)
		{
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, thy_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
		}
	else if(thy_lvl < 0)
		{
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, -thy_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
		}
	else 
	{
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
	}

}

void f9h_fair_2::RollToHeadVector(void)
{
	double roll_angle = atan2(head_vector.x,-head_vector.y);
	if (roll_angle > PI) roll_angle = roll_angle - PI2;
	else if (roll_angle < -PI) roll_angle = roll_angle + PI2;

	double mass = GetMass();
	VECTOR3 pmi;
	GetPMI(pmi);
	//double inrt_x = pmi.x * mass;
	//double inrt_y = pmi.y * mass;
	double inrt_z = pmi.z * mass;

	//double max_torq_x = 2 * 4.467 * 0.7071 * RCS_TH;
	//double max_torq_y = RCS_TH*0.7071*((4.467 * 1) + (4.467 * 0.713) + (7.774*0.983));
	double max_torq_z = RCS_TH*0.575*((-0.057 * 0.7071) + (0.99 * 0.7071));
	max_torq_z = max_torq_z + (2.567 * 0.7071 * RCS_TH) + (0.99 * 0.7071 * RCS_TH);

	VECTOR3 angv;
	GetAngularVel(angv);
	double vrot_lim = 10*RAD;

	double max_acc_z = max_torq_z/inrt_z;
	double max_drot_z = max_acc_z*steptime;


	double reqzv = 1*abs(roll_angle);
	if (reqzv > vrot_lim)reqzv = vrot_lim;
	if (roll_angle > 0)reqzv = -reqzv;
	double req_drot = reqzv - angv.z;
	double thz_lvl = req_drot/max_drot_z;
	if((roll_angle*DEG) < 0.5 && (roll_angle*DEG) > -0.5) thz_lvl = 0.25*thz_lvl;

	if(thz_lvl > 1)thz_lvl = 1;
	else if(thz_lvl < -1)thz_lvl = -1;


	if(thz_lvl > 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT, thz_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT, 0);
	}
	else if(thz_lvl < 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT, -thz_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT, 0);
	}
	else 
	{
		SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT, 0);
		SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT, 0);
	}

}

void f9h_fair_2::CalcFlightVectors(void)
{
	VECTOR3 grnd_spd_vect;
	GetGroundspeedVector(FRAME_HORIZON, grnd_spd_vect);
	normalise(grnd_spd_vect);

	double ba_correct = -bank_angle*RAD;
	double pa_correct = -pitch_angle*RAD;

	VECTOR3 vert_gsv_normal = crossp(_V(0,1,0),grnd_spd_vect);
	normalise(vert_gsv_normal);

	MATRIX3 rot_mat = rotm(vert_gsv_normal, pa_correct);
	VECTOR3 flight_vector_hrz = mul(rot_mat,grnd_spd_vect);
	normalise(flight_vector_hrz);

	VECTOR3 head_vector_hrz = crossp(vert_gsv_normal, flight_vector_hrz);
	normalise(head_vector_hrz);

	rot_mat = rotm(grnd_spd_vect, ba_correct);
	flight_vector_hrz = mul(rot_mat, flight_vector_hrz);
	head_vector_hrz = mul(rot_mat, head_vector_hrz);

	HorizonInvRot(head_vector_hrz,head_vector);
	HorizonInvRot(flight_vector_hrz,flight_vector);

}


void f9h_fair_2::CalcPitchBank(void)
{
	double alt = GetAltitude(ALTMODE_MEANRAD);
	if(hTarget == NULL)
	{
	if (alt > 120000)
		{
		pitch_angle = 40;
		bank_angle = 0;
		}
	else if (alt > 100000)
		{
		pitch_angle = 40;
		bank_angle = 45;
		}
	else if (alt > 80000)
		{
		pitch_angle = 40;
		bank_angle = -45;
		}
	else if (alt > 60000)
		{
		pitch_angle = 40;
		bank_angle = 45;
		}
	else if (alt > 40000)
		{
		pitch_angle = 40;
		bank_angle = -45;
		}
	else if (alt > 25000)
		{
		pitch_angle = 40;
		bank_angle = 45;
		}
	else if (alt > 10000)
		{
		pitch_angle = 40;
		bank_angle = -45;
		}
	else
		{
		pitch_angle = 80;
		bank_angle = 0;
		}
	}
	else // frv exists
	{
	VECTOR3 tgt_glob_pos = _V(0,0,0);
	oapiGetGlobalPos(hTarget, &tgt_glob_pos);

	VECTOR3 tgt_loc_pos = _V(0,0,0);
	Global2Local(tgt_glob_pos,tgt_loc_pos);

	VECTOR3 tgt_hrz_pos = _V(0,0,0);
	HorizonRot(tgt_loc_pos,tgt_hrz_pos);

	VECTOR3 wind_comp = _V(0,0,0);
	int i;
	for(i = 1; i < 61; i++)
	{
	wind_comp = wind_comp + wind_data[i];
	}
	wind_comp.x = wind_comp.x*25;
	wind_comp.y = wind_comp.y*25;
	wind_comp.z = wind_comp.z*25;

	tgt_hrz_pos = tgt_hrz_pos - wind_comp;

	VECTOR3 haspdv = _V(0,0,0);
	GetGroundspeedVector(FRAME_HORIZON,haspdv);

	double gsvx = sqrt((haspdv.x*haspdv.x)+(haspdv.z*haspdv.z));

	double vmass = GetMass();

	OBJHANDLE hearth=GetSurfaceRef();

	VECTOR3 relvelhz = _V(0,0,0);
	GetRelativeVel(hearth, relvelhz);
	double rvh = length(relvelhz);
	rvh = sqrt((rvh*rvh) - (haspdv.y*haspdv.y));
	double acc_cfg = rvh*rvh/(alt+6371010);

	double rds = 6371010+alt;
    double GM = GGRAV*oapiGetMass(hearth);
	double gravt = GM/(rds*rds);
	gravt = gravt - acc_cfg;
	double g = -gravt;
	double aa = g/2;
	double aaa = -g;
	double bb = haspdv.y;
	double cc = alt;
	double timetoimp = (-bb-(sqrt((bb*bb)-(4*aa*cc))))/(2*aa);
	double timetoimp2 = (-bb-(sqrt((bb*bb)-(4*aa*(cc-20000)))))/(2*aa);
	if (alt < 20000)timetoimp2 = 0;

	 double range1 = gsvx*timetoimp2*1.04;  // above 30km
	 
	 double k = 75*9.81/8; // 2
	 double expo = -k*(timetoimp-timetoimp2)/vmass;
	 double range2 = vmass*gsvx/k*(1-(pow(M_E, expo))); //below 25km
	 double range = (range1+range2);


	 double grdspd_az = atan2(haspdv.x, haspdv.z);
	 double tgt_az = atan2(tgt_hrz_pos.x,tgt_hrz_pos.z);
	 double tgt_angle = grdspd_az - tgt_az;
	 double tgt_range = length(_V(tgt_hrz_pos.x,0,tgt_hrz_pos.z));

	 hud_tgt_dist = tgt_range;
	 hud_range = range;

	 if (tgt_angle > PI)tgt_angle = tgt_angle-PI2;
	 if (tgt_angle < -PI)tgt_angle = tgt_angle+PI2;

	 double def_pitch = 60;
	 double def_bank = 0;

	 double bank_cor = (-tgt_angle*DEG)*0.2;
	 if (bank_cor < -1)bank_cor = -1;
	 else if (bank_cor > 1)bank_cor = 1;
	 bank_cor = bank_cor * 45;
	 bank_angle = def_bank + bank_cor;

	 double pitch_cor = 10*((range - tgt_range)/tgt_range);
	 if(pitch_cor > 1)pitch_cor = 1;
	 else if(pitch_cor < -1)pitch_cor = -1;
	 pitch_cor = pitch_cor*20;
	 pitch_angle = def_pitch + pitch_cor;
	 if (alt < 7000) // prepare for chute deploy
	 {
		pitch_angle = 85;
		bank_angle = 0;
	 }
	 //sprintf(oapiDebugString(), "bank_angle %0.2f pitch_angle %0.2f range %0.2f tgt_range %0.2f tgt_angle %0.2f",bank_angle, pitch_angle, range, tgt_range, tgt_angle*DEG);
	}


}