// ==============================================================
//
// f9h_fair_large
// Control module for f9h_fair_large vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include <cstdio>
#include <cmath>
#include <string>
#include <stdio.h>
#include "OrbiterSoundSDK40.h"

// ==============================================================
// Some vessel parameters
// ==============================================================


const double FAIR_DRYMASS = 1250;

static const DWORD ntdvtx_geardown = 4;
static TOUCHDOWNVTX tdvtx_geardown[ntdvtx_geardown] = {
	{_V(    0, -6.5, 7), 1e5, 1e5, 2, 2},
	{_V(-2.5,-6.5,-5), 1e5, 1e5, 2, 2},
	{_V( 2.5,-6.5,-5), 1e5, 1e5, 2, 2},
	{_V(   0 ,0 ,   20), 1e5, 1e5, 1}
};

//#define GREEN RGB(125, 255, 125)

void VLiftCoeffHull (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0};


	aoa += PI;
	int idx = max (0, min (15, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 1*(0.1 + oapiGetInducedDrag (*cl, 0.5, 0.2));

	
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeffHull (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 1*(0.1 + oapiGetInducedDrag (*cl, 0.5, 0.2));
	
}


// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class f9h_fair_large: public VESSEL3 {
public:
	f9h_fair_large (OBJHANDLE hVessel, int flightmodel);
	~f9h_fair_large ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostCreation();
	void clbkPreStep(double SimT, double SimDT, double MJD);
	//void clbkPostStep (double simt, double simdt, double mjd);
	//void clbkSaveState (FILEHANDLE scn);
	//void clbkLoadStateEx (FILEHANDLE scn, void *status);
	//int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	//bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	int MyID;
	//bool clbkLoadGenericCockpit ();
	
private:

};

f9h_fair_large::f9h_fair_large (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
    

}

f9h_fair_large::~f9h_fair_large ()
{
	
}


// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void f9h_fair_large::clbkSetClassCaps (FILEHANDLE cfg)
{
	
	// physical vessel parameters
	SetEmptyMass (FAIR_DRYMASS);
	SetSize (7);
	SetPMI(_V(16,16,2));
	SetCrossSections (_V(50,100,11));
	//SetTouchdownPoints ( _V(0,-5, -5), _V(-5, 3, -5), _V( 5, 3,-5));
	SetTouchdownPoints (tdvtx_geardown,ntdvtx_geardown);
	
	SetCW(0.8,0.8,1,1);
	SetRotDrag (_V(0.05,0.05,0.05));


	// camera parameters
	SetCameraOffset (_V(0,3,0));
	SetCameraRotationRange(PI,PI,(0.4*PI),(0.4*PI));


	SetMeshVisibilityMode(AddMesh("FalconHeavy/f9h_fair_large"),MESHVIS_ALWAYS);

	////////// thrusters
	
	/////////////////////////////////////// airfoils

	CreateAirfoil (LIFT_VERTICAL, _V(0,0,0), VLiftCoeffHull, 10, 40, 0.5);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeffHull, 10, 20, 0.5);
	

}

void f9h_fair_large::clbkPostCreation ()
{
	
	/////////////// sound
	MyID=ConnectToOrbiterSoundDLL(GetHandle());
	SoundOptionOnOff(MyID,PLAYRADIOATC,FALSE);
	//SoundOptionOnOff3(MyID,PLAYWHENATTITUDEMODECHANGE,FALSE);
	SoundOptionOnOff(MyID,PLAYGPWS,FALSE);
	SoundOptionOnOff(MyID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);
	SoundOptionOnOff(MyID,PLAYLANDINGANDGROUNDSOUND,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINAIRCONDITIONING,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINRANDOMAMBIANCE,FALSE);
	SoundOptionOnOff(MyID,DISPLAYTIMER,FALSE);
	SoundOptionOnOff(MyID,PLAYDOCKLANDCLEARANCE,FALSE);
	SoundOptionOnOff(MyID,PLAYRADARBIP,FALSE);
}


// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new f9h_fair_large (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (f9h_fair_large*)vessel;
}

void f9h_fair_large::clbkPreStep(double SimT, double SimDT, double MJD)
{
	double alt = GetAltitude(ALTMODE_GROUND);
	if(alt < 10) oapiDeleteVessel(GetHandle());
	else if (alt > 10000000) oapiDeleteVessel(GetHandle());

}

