// ==============================================================
//
// f9h_stg1_starboard
// Control module for f9h_stg1_starboard vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include <cstdio>
#include <cmath>
#include <string>
#include <stdio.h>

// ==============================================================
// Some vessel parameters
// ==============================================================

const DWORD COMM_SEND_STR = 2300;
const DWORD COMM_SEND_AFS = 2301;
const DWORD COMM_SEND_EDLO = 2304;
const double ISP0 = 3062;//2767;
const double ISPV = 3311;//3051;
const double MAINTH0 = 845222;//756222;
const double MAINTH1 = 914111;//825111;
const double RCSTH = 38000;
const double RCSTH2 = 1000;
const double FUELMASS = 402500;//401500;//396500;//384000
const double DRYMASS = 22100;//19000;
const double LEG_SPEED = 0.25;
const double FIN_SPEED = 0.5;
const VECTOR3 INT_OFF = _V(0,0,0);
const double RCS2MASS = 400;
const double RCS2ISPV = 800;
const double RCS2ISP0 = 800;
const double M_E = 2.71828182845904523536;

const double mu = 398600439968871.2;
const double rt = 6371010;
#define GREEN RGB(125, 255, 125)

OBJHANDLE phGref;

static const DWORD ntdvtx_geardown = 7;
static TOUCHDOWNVTX tdvtx_geardown[ntdvtx_geardown] = {
	{_V(    0, -5, -24), 1e6, 1e6, 2, 2},
	{_V(-4.33, 2.5,-24), 1e6, 1e6, 2, 2},
	{_V( 4.33, 2.5,-24), 1e6, 1e6, 2, 2},
	{_V(   0 ,0 ,   20), 1e6, 1e5, 1},
	{_V(    0,  5, -24), 1e6, 1e5, 1},
	{_V(-4.33,-2.5,-24), 1e6, 1e5, 1},
	{_V( 4.33,-2.5,-24), 1e6, 1e5, 1}
};

static const DWORD ntdvtx_gearup = 4;
static TOUCHDOWNVTX tdvtx_gearup[ntdvtx_gearup] = {
	{_V(0, -5,-20), 1e6, 1e5, 1, 1},
	{_V(-4.33, 2.5,-20), 1e6, 1e5, 1, 1},
	{_V( 4.33, 2.5,-20), 1e6, 1e5, 1, 1},
	{_V(0 ,0 , 20), 1e6, 1e5, 1}
};

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, -0.1, -0.2, -0.1, 0};


	aoa += PI;
	int idx = max (0, min (15, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.55*(0.11 + oapiGetInducedDrag (*cl, 0.16, 0.2));
	
	
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, -0.1, -0.2, -0.1, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 0.55*(0.11 + oapiGetInducedDrag (*cl, 0.16, 0.2));
	
}

// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class f9h_stg1_starboard: public VESSEL3 {
public:
	f9h_stg1_starboard (OBJHANDLE hVessel, int flightmodel);
	~f9h_stg1_starboard ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostCreation();
	void clbkPreStep(double SimT, double SimDT, double MJD);
	void clbkPostStep (double simt, double simdt, double mjd);
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	bool clbkLoadGenericCockpit ();
	
private:

	EXHAUSTSPEC main_exhaust[9];
	EXHAUSTSPEC main_exhaust1[4];
	EXHAUSTSPEC main_exhaust2[2];
	THRUSTER_HANDLE th_main[3],th_rcs[12], th_group[9], th_rcs2[12], th_boom[2];
	THGROUP_HANDLE htgrpmainp, htgrppup,htgrppdown,htgrpbright, htgrpbleft, htgrpyright, htgrpyleft;
	PROPELLANT_HANDLE hProp, hRCSProp;
	OBJHANDLE bbtgtchk;
	OBJHANDLE bbchk;
	ANIMATIONCOMPONENT_HANDLE Section0, Section1, Section2, Section3, Section4;
	PSTREAM_HANDLE at_trail;
	CTRLSURFHANDLE hElevator, hRudder;
	SURFHANDLE bs1, bs2;
	MESHHANDLE hMhull;
	SpotLight *spt_light;
	//PointLight *pnt_light;

	double timetoimp;
	int reentry_correct_sw;

	void ConfigMain(void);
	void ConfigRCS(void);
	void RevertRCS(void);
	void DefineAnimations (void);
	void DoLand(void);
	void DoRAT(void);
	void DoLAT(void);
	void DoTWU(void);
	void GoToFlightVector();
	void SetCamDir();
	void DoBoostBack();
	void DoBoostBack2();
	void KillAllRCS();
	void DoCorrection();
	void DoReentryCorrection();
	void ApplySkin();
	void DoBoom();
	void DoThrustVector();
	void DoLandingRCS();

	char prefname[256];

	double step_time, old_ap_dist, bias, timer, ptimer2, yhdg, skin_sw, boom_sw;//pitch_angle,
	int leg_status, dp_status, eng_statusp, fins_deploy_status, landed_sw;
	int cam_status, boostback_sw, rat_sw, correct_sw, afs_status, rot_sw, rcs_sw, rc_sw;

	UINT anim_legs, anim_pist_rot, anim_pist_trans, anim_fins_deploy, anim_fins_rot_a, anim_fins_rot_b;
	double legs_proc, pist_rot_proc, pist_trans_proc, fins_deploy_proc, fins_rot_a_proc, fins_rot_b_proc;
	double fb_dv_req, pfact , vrot_lim, impact_angle, impact_angle2;

	MGROUP_TRANSFORM *Leg1trans[4], *Leg2trans[4], *Leg3trans[4], *Leg4trans[4];
	MGROUP_TRANSFORM *Leg1rot[2], *Leg2rot[2], *Leg3rot[2], *Leg4rot[2];
	MGROUP_TRANSFORM *FinsDeploy[4], *FinsRotate[4];
	
	VECTOR3 azim_local, vazim1, vazim2, flight_vector, yhrz;

	int swt, dust_sw, rchud_sw, land_sw, edlo_sw, mom_sw, jet_burn_sw;
	double dotprod;
	int afs_status_mem, dp_status_mem, rat_sw_mem, boostback_sw_mem;
	
	VECTOR3 torq_vect;
	double pu_level, pd_level, yr_level, yl_level, mth_level;
	double hud_flight_range, hud_range;
};

f9h_stg1_starboard::f9h_stg1_starboard (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
    

}

f9h_stg1_starboard::~f9h_stg1_starboard ()
{
	int i;
	for (i = 0; i < 4; i++)
	{
	delete Leg1trans[i];
	delete Leg2trans[i];
	delete Leg3trans[i];
	delete Leg4trans[i];
	delete FinsDeploy[i];
	delete FinsRotate[i];
	}

	for (i = 0; i < 2; i++)
	{
	delete Leg1rot[i];
	delete Leg2rot[i];
	delete Leg3rot[i];
	delete Leg4rot[i];
	}


}

void f9h_stg1_starboard::DefineAnimations (void)
{
	anim_legs = CreateAnimation (0);
	anim_pist_rot = CreateAnimation (0);
	anim_pist_trans = CreateAnimation (0);

// 1. leg1x+

	static UINT Leg1grp1[2] = {1,33};
	Leg1rot[0] = new MGROUP_ROTATE(0, Leg1grp1, 2, _V(1.1243, 1.1243, -19.212), _V(-0.7071068,0.7071068,0), (float)(120*RAD));
	AddAnimationComponent (anim_legs, 0, 1, Leg1rot[0]);

	static UINT Leg1grp2[1] = {14};
	Leg1rot[1] = new MGROUP_ROTATE(0, Leg1grp2, 1, _V(1.4142136, 1.4142136, -15.96), _V(-0.7071068,0.7071068,0), (float)(139.5*RAD));
	Section0 = AddAnimationComponent (anim_pist_rot, 0, 1, Leg1rot[1]);

	static UINT Leg1grp3[1] = {13};
	Leg1trans[0] = new MGROUP_TRANSLATE(0, Leg1grp3, 1, _V(0, 0, 1.27));
	Section1 = AddAnimationComponent (anim_pist_trans, 0, 1, Leg1trans[0], Section0);

	static UINT Leg1grp4[1] = {12};
	Leg1trans[1] = new MGROUP_TRANSLATE(0, Leg1grp4, 1, _V(0, 0, 1.27));
	Section2 = AddAnimationComponent (anim_pist_trans, 0, 1, Leg1trans[1], Section1);

	static UINT Leg1grp5[1] = {11};
	Leg1trans[2] = new MGROUP_TRANSLATE(0, Leg1grp5, 1, _V(0, 0, 1.27));
	Section3 = AddAnimationComponent (anim_pist_trans, 0, 1, Leg1trans[2], Section2);

	static UINT Leg1grp6[1] = {10};
	Leg1trans[3] = new MGROUP_TRANSLATE(0, Leg1grp6, 1, _V(0, 0, 1.27));
	Section4 = AddAnimationComponent (anim_pist_trans, 0, 1, Leg1trans[3], Section3);


	// 2. leg1y+

	static UINT Leg2grp1[2] = {2,31};
	Leg2rot[0] = new MGROUP_ROTATE(0, Leg2grp1, 2, _V(-1.1243, 1.1243, -19.212), _V(0.7071068,0.7071068,0), (float)(-120*RAD));
	AddAnimationComponent (anim_legs, 0, 1, Leg2rot[0]);

	static UINT Leg2grp2[1] = {15};
	Leg2rot[1] = new MGROUP_ROTATE(0, Leg2grp2, 1, _V(-1.4142136, 1.4142136, -15.96), _V(0.7071068,0.7071068,0), (float)(-139.5*RAD));
	Section0 = AddAnimationComponent (anim_pist_rot, 0, 1, Leg2rot[1]);

	static UINT Leg2grp3[1] = {16};
	Leg2trans[0] = new MGROUP_TRANSLATE(0, Leg2grp3, 1, _V(0, 0, 1.27));
	Section1 = AddAnimationComponent (anim_pist_trans, 0, 1, Leg2trans[0], Section0);

	static UINT Leg2grp4[1] = {18};
	Leg2trans[1] = new MGROUP_TRANSLATE(0, Leg2grp4, 1, _V(0, 0, 1.27));
	Section2 = AddAnimationComponent (anim_pist_trans, 0, 1, Leg2trans[1], Section1);

	static UINT Leg2grp5[1] = {17};
	Leg2trans[2] = new MGROUP_TRANSLATE(0, Leg2grp5, 1, _V(0, 0, 1.27));
	Section3 = AddAnimationComponent (anim_pist_trans, 0, 1, Leg2trans[2], Section2);

	static UINT Leg2grp6[1] = {19};
	Leg2trans[3] = new MGROUP_TRANSLATE(0, Leg2grp6, 1, _V(0, 0, 1.27));
	Section4 = AddAnimationComponent (anim_pist_trans, 0, 1, Leg2trans[3], Section3);



	// 3. leg3x-

	static UINT Leg3grp1[2] = {3,34};
	Leg3rot[0] = new MGROUP_ROTATE(0, Leg3grp1, 2, _V(-1.1243, -1.1243, -19.212), _V(-0.7071068,0.7071068,0), (float)(-120*RAD));
	AddAnimationComponent (anim_legs, 0, 1, Leg3rot[0]);

	static UINT Leg3grp2[1] = {5};
	Leg3rot[1] = new MGROUP_ROTATE(0, Leg3grp2, 1, _V(-1.4142136, -1.4142136, -15.96), _V(-0.7071068,0.7071068,0), (float)(-139.5*RAD));
	Section0 = AddAnimationComponent (anim_pist_rot, 0, 1, Leg3rot[1]);

	static UINT Leg3grp3[1] = {6};
	Leg3trans[0] = new MGROUP_TRANSLATE(0, Leg3grp3, 1, _V(0, 0, 1.27));
	Section1 = AddAnimationComponent (anim_pist_trans, 0, 1, Leg3trans[0], Section0);

	static UINT Leg3grp4[1] = {8};
	Leg3trans[1] = new MGROUP_TRANSLATE(0, Leg3grp4, 1, _V(0, 0, 1.27));
	Section2 = AddAnimationComponent (anim_pist_trans, 0, 1, Leg3trans[1], Section1);

	static UINT Leg3grp5[1] = {7};
	Leg3trans[2] = new MGROUP_TRANSLATE(0, Leg3grp5, 1, _V(0, 0, 1.27));
	Section3 = AddAnimationComponent (anim_pist_trans, 0, 1, Leg3trans[2], Section2);

	static UINT Leg3grp6[1] = {9};
	Leg3trans[3] = new MGROUP_TRANSLATE(0, Leg3grp6, 1, _V(0, 0, 1.27));
	Section4 = AddAnimationComponent (anim_pist_trans, 0, 1, Leg3trans[3], Section3);


	// 4. leg1y-

	static UINT Leg4grp1[2] = {4,32};
	Leg4rot[0] = new MGROUP_ROTATE(0, Leg4grp1, 2, _V(1.1243, -1.1243, -19.212), _V(0.7071068,0.7071068,0), (float)(120*RAD));
	AddAnimationComponent (anim_legs, 0, 1, Leg4rot[0]);

	static UINT Leg4grp2[1] = {24};
	Leg4rot[1] = new MGROUP_ROTATE(0, Leg4grp2, 1, _V(1.4142136, -1.4142136, -15.96), _V(0.7071068,0.7071068,0), (float)(139.5*RAD));
	Section0 = AddAnimationComponent (anim_pist_rot, 0, 1, Leg4rot[1]);

	static UINT Leg4grp3[1] = {23};
	Leg4trans[0] = new MGROUP_TRANSLATE(0, Leg4grp3, 1, _V(0, 0, 1.27));
	Section1 = AddAnimationComponent (anim_pist_trans, 0, 1, Leg4trans[0], Section0);

	static UINT Leg4grp4[1] = {22};
	Leg4trans[1] = new MGROUP_TRANSLATE(0, Leg4grp4, 1, _V(0, 0, 1.27));
	Section2 = AddAnimationComponent (anim_pist_trans, 0, 1, Leg4trans[1], Section1);

	static UINT Leg4grp5[1] = {21};
	Leg4trans[2] = new MGROUP_TRANSLATE(0, Leg4grp5, 1, _V(0, 0, 1.27));
	Section3 = AddAnimationComponent (anim_pist_trans, 0, 1, Leg4trans[2], Section2);

	static UINT Leg4grp6[1] = {20};
	Leg4trans[3] = new MGROUP_TRANSLATE(0, Leg4grp6, 1, _V(0, 0, 1.27));
	Section4 = AddAnimationComponent (anim_pist_trans, 0, 1, Leg4trans[3], Section3);

	///////////////////////////
	// fin deploy
	anim_fins_deploy = CreateAnimation (0.0);

	static UINT Finsgrp0[1] = {37};
	FinsDeploy[0] = new MGROUP_ROTATE(0, Finsgrp0, 0, _V(-1.355, 1.355, 19.567), _V(0.7071,0.7071,0), PI05);
	Section1 = AddAnimationComponent (anim_fins_deploy, 0, 1, FinsDeploy[0]);

	static UINT Finsgrp1[1] = {36};
	FinsDeploy[1] = new MGROUP_ROTATE(0, Finsgrp1, 0, _V(1.355, -1.355, 19.567), _V(0.7071,0.7071,0), -PI05);
	Section2 = AddAnimationComponent (anim_fins_deploy, 0, 1, FinsDeploy[1]);

	static UINT Finsgrp2[1] = {35};
	FinsDeploy[2] = new MGROUP_ROTATE(0, Finsgrp2, 0, _V(1.355, 1.355, 19.567), _V(-0.7071,0.7071,0), -PI05);
	Section3 = AddAnimationComponent (anim_fins_deploy, 0, 1, FinsDeploy[2]);

	static UINT Finsgrp3[1] = {38};
	FinsDeploy[3] = new MGROUP_ROTATE(0, Finsgrp3, 0, _V(-1.355, -1.355, 19.567), _V(-0.7071,0.7071,0), PI05);
	Section4 = AddAnimationComponent (anim_fins_deploy, 0, 1, FinsDeploy[3]);


	///// fins rotate
	anim_fins_rot_a = CreateAnimation (0.5);

	FinsRotate[0] = new MGROUP_ROTATE(0, Finsgrp0, 1, _V(-1.355, 1.355, 19.567), _V(0,0,1), PI05);
	AddAnimationComponent (anim_fins_rot_a, 0, 1, FinsRotate[0], Section1);

	FinsRotate[1] = new MGROUP_ROTATE(0, Finsgrp1, 1, _V(1.355, -1.355, 19.567), _V(0,0,1), -PI05);
	AddAnimationComponent (anim_fins_rot_a, 0, 1, FinsRotate[1], Section2);

	anim_fins_rot_b = CreateAnimation (0.5);

	FinsRotate[2] = new MGROUP_ROTATE(0, Finsgrp2, 1, _V( 1.355, 1.355, 19.567), _V(0,0,1), PI05);
	AddAnimationComponent (anim_fins_rot_b, 0, 1, FinsRotate[2], Section3);

	FinsRotate[3] = new MGROUP_ROTATE(0, Finsgrp3, 1, _V(-1.355, -1.355, 19.567), _V(0,0,1), -PI05);
	AddAnimationComponent (anim_fins_rot_b, 0, 1, FinsRotate[3], Section4);

}


// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void f9h_stg1_starboard::clbkSetClassCaps (FILEHANDLE cfg)
{
	strcpy(prefname, "NULL");

	fins_deploy_proc = 0;
	fins_rot_a_proc = 0.5;
	fins_rot_b_proc= 0.5;
	fins_deploy_status = 0;

	boom_sw = 0;
	step_time = 0;
	cam_status = 0;
	old_ap_dist = 0;
	bias = 0;
	timer = 0;
	ptimer2 = 0;
	//pitch_angle = 90*RAD;
	dp_status = 0;
	leg_status = 0;
	afs_status = 0;
	legs_proc = 0.0;
	pist_rot_proc = 0.0;
	pist_trans_proc = 0.0;
	//intfair_status = 0;
	eng_statusp = 0;
	//calc_sw = 0;
	rot_sw = 0;
	rcs_sw = 0;
	rc_sw = 0;
	dust_sw = 0;
	rchud_sw = 0;
	jet_burn_sw = 0;
	vrot_lim = 5*RAD;
	azim_local = _V(0,-1,0);
	vazim1 = _V(0,1,0);
	vazim2 = _V(0,1,0);
	flight_vector = _V(0,1,0);
	yhrz = _V(0,0,0);
	yhdg = 0;
	boostback_sw = 0;
	rat_sw = 0;
	correct_sw = 0;
	fb_dv_req = 0;
	pfact = 0;
	timetoimp = 100;
	reentry_correct_sw = 0;
	impact_angle = 70*RAD;
	impact_angle2 = 65*RAD;
	swt = 0;
	dotprod = 0;
	land_sw = 0;
	landed_sw  = 0;
	skin_sw = 0;

	afs_status_mem = 0;
	dp_status_mem = 0;
	rat_sw_mem = 0;
	boostback_sw_mem = 0;

	edlo_sw = 0;
	mom_sw = 0;

	torq_vect = _V(0,0,0);

	pu_level = 0;
	pd_level = 0;
	yr_level = 0;
	yl_level = 0;
	mth_level = 0;

	hud_flight_range = 0;
	hud_range = 0;

	DefineAnimations ();
	// physical vessel parameters
	SetSize (20);
	SetEmptyMass (DRYMASS);
	SetPMI(_V(129,129,1.9));
	SetCrossSections (_V(159,159,16));
	SetCW(0.5,4,1,1);
	SetRotDrag (_V(0.25,0.25,0.1));
	SetTouchdownPoints ( tdvtx_gearup, ntdvtx_gearup);

	// camera parameters
	SetCameraOffset (_V(0,3,7.5));
	SetCameraRotationRange(PI,PI,(0.4*PI),(0.4*PI));

	hMhull = oapiLoadMesh("FalconHeavy/f9h_stg1_starboard");
	SetMeshVisibilityMode(AddMesh(hMhull),MESHVIS_ALWAYS);

	bs2 = oapiLoadTexture ("FalconHeavy/f9_hull_logo_char_stbd.dds", true);

	// propellant resources
	
	hProp = CreatePropellantResource (FUELMASS,FUELMASS);
	hRCSProp = CreatePropellantResource (RCS2MASS,RCS2MASS);

	// ***************** thruster definitions *******************

	th_rcs2[0] = CreateThruster (_V( 1.95, 0, 18.8), _V( -1,0,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[1] = CreateThruster (_V( 0, 1.95, 18.8), _V( 0,-1,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[2] = CreateThruster (_V( -1.95, 0, 18.8), _V( 1,0,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[3] = CreateThruster (_V( 0, -1.95, 18.8), _V( 0,1,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);

	th_rcs2[4] = CreateThruster (_V( 1.95, 0, -18.12), _V( -1,0,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[5] = CreateThruster (_V( 0, 1.95, -18.12), _V( 0,-1,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[6] = CreateThruster (_V( -1.95, 0, -18.12), _V( 1,0,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[7] = CreateThruster (_V( 0, -1.95, -18.12), _V( 0,1,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);

	th_rcs2[8] = CreateThruster (_V( 1.884, 0.103, 18.963), _V( 0,-1,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[9] = CreateThruster (_V( 1.884, -0.103, 18.963), _V( 0,1,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[10] = CreateThruster (_V( -1.884, 0.103, 18.963), _V( 0,-1,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	th_rcs2[11] = CreateThruster (_V( -1.884, -0.103, 18.963), _V( 0,1,0), RCSTH2, hRCSProp, RCS2ISPV, RCS2ISP0);
	//th_rcs2[11] = CreateThruster (_V( -1.884, -0.103, 18.963), _V( 0,1,0), RCSTH2, hProp, ISPV, ISP0);
	
	////////////////////////////

	th_rcs[0] = CreateThruster (_V( 0, 2, -21), _V( 1,0,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[1] = CreateThruster (_V( 0, 2, -21), _V(-1,0,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[2] = CreateThruster (_V( 0,-2, -21), _V( 1,0,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[3] = CreateThruster (_V( 0,-2, -21), _V(-1,0,0), RCSTH, hProp, ISPV, ISP0);

	th_rcs[4] = CreateThruster (_V( 0, 4, -21), _V( 1,0,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[5] = CreateThruster (_V( 0, 4, -21), _V(-1,0,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[6] = CreateThruster (_V( 0,-4, -21), _V( 1,0,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[7] = CreateThruster (_V( 0,-4, -21), _V(-1,0,0), RCSTH, hProp, ISPV, ISP0);

	th_rcs[8] = CreateThruster (_V( 2, 0, -21), _V(0, 1,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[9] = CreateThruster (_V( 2, 0, -21), _V(0,-1,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[10] = CreateThruster (_V(-2,0, -21), _V(0, 1,0), RCSTH, hProp, ISPV, ISP0);
	th_rcs[11] = CreateThruster (_V(-2,0, -21), _V(0,-1,0), RCSTH, hProp, ISPV, ISP0);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[11];
	htgrppup = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[10];
	htgrppdown = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[3];
	htgrpyright = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[2];
	htgrpyleft = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[4];
	th_group[1] = th_rcs[7];
	htgrpbright = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[5];
	th_group[1] = th_rcs[6];
	htgrpbleft = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);


	th_main[0] = CreateThruster (_V( 0, 0, -21), _V( 0,0,1), 9*MAINTH1, hProp, ISPV, ISP0);
	th_main[1] = CreateThruster (_V( 0, 0, -21), _V( 0,0,1), 3*MAINTH1, hProp, ISPV, ISP0);
	th_main[2] = CreateThruster (_V( 0, 0, -21), _V( 0,0,1), MAINTH1, hProp, ISPV, ISP0);

	th_group[0] = th_main[0];
	htgrpmainp = CreateThrusterGroup (th_group, 1, THGROUP_MAIN);

	SURFHANDLE exhaust = oapiRegisterExhaustTexture ("FalconHeavy/KeroExhaust3");
//	SURFHANDLE exhaust = oapiRegisterExhaustTexture ("Exhaust2");
	SURFHANDLE exhaust_rcs = oapiRegisterExhaustTexture ("exhaust_atrcs");

	double exlenrcs = 6;
	double exwidrcs = 0.6;

	AddExhaust(th_rcs2[0],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs2[1],exlenrcs,exwidrcs,_V(1.884,  0.103, 18.963), _V( 0,1,0),exhaust_rcs);
	AddExhaust(th_rcs2[2],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs2[3],exlenrcs,exwidrcs,_V(1.884,-0.103, 18.963), _V( 0,-1,0),exhaust_rcs);
	AddExhaust(th_rcs2[4],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs2[5],exlenrcs,exwidrcs,_V(-1.884,-0.103, 18.963), _V( 0,-1,0),exhaust_rcs);
	AddExhaust(th_rcs2[6],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs2[7],exlenrcs,exwidrcs,_V(-1.884, 0.103, 18.963), _V( 0,1,0),exhaust_rcs);
	AddExhaust(th_rcs2[8],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs2[9],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs2[10],exlenrcs,exwidrcs,exhaust_rcs);
	AddExhaust(th_rcs2[11],exlenrcs,exwidrcs,exhaust_rcs);




	double exlen = 30;
	double exwid = 3.7;

	MATRIX3 rm;
	VECTOR3 exhaust_pos[9];
	VECTOR3 exhaust_pos1[4];
	VECTOR3 exhaust_pos2[2];

	for (int i = 0; i < 8; i++) // core
	{
	rm = rotm(_V(0,0,1),(45*RAD*i));
	exhaust_pos[i] = mul(rm,_V(0,1.32,-20.5));

	main_exhaust[i].th = th_main[0];
	main_exhaust[i].level = NULL;
	main_exhaust[i].lpos = &exhaust_pos[i];
	main_exhaust[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust[i].lsize = exlen;
	main_exhaust[i].wsize = exwid;
	main_exhaust[i].lofs = 0;
	main_exhaust[i].modulate = 0.8;
	main_exhaust[i].tex = exhaust;
	main_exhaust[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust[i].id = 0;
	AddExhaust (&main_exhaust[i]);
	}
	{
	int i = 8;
	exhaust_pos[i] = _V(0,0,-20.5);

	main_exhaust[i].th = th_main[0];
	main_exhaust[i].level = NULL;
	main_exhaust[i].lpos = &exhaust_pos[i];
	main_exhaust[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust[i].lsize = exlen;
	main_exhaust[i].wsize = exwid;
	main_exhaust[i].lofs = 0;
	main_exhaust[i].modulate = 0.8;
	main_exhaust[i].tex = exhaust;
	main_exhaust[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust[i].id = 0;
	AddExhaust (&main_exhaust[i]);
	}
	///////////////////////////// 3  engines
	for (int i = 0; i < 2; i++) // core
	{
	rm = rotm(_V(0,0,1),(180*RAD*i));
	exhaust_pos[i] = mul(rm,_V(1.32, 0,-20.5));

	main_exhaust1[i].th = th_main[1];
	main_exhaust1[i].level = NULL;
	main_exhaust1[i].lpos = &exhaust_pos[i];
	main_exhaust1[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust1[i].lsize = exlen;
	main_exhaust1[i].wsize = exwid;
	main_exhaust1[i].lofs = 0;
	main_exhaust1[i].modulate = 0.8;
	main_exhaust1[i].tex = exhaust;
	main_exhaust1[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust1[i].id = 0;
	AddExhaust (&main_exhaust1[i]);
	}
	for (int i = 2; i < 4; i++) // core
	{
	exhaust_pos1[i] = _V(0, 0,-20.5);

	main_exhaust1[i].th = th_main[1];
	main_exhaust1[i].level = NULL;
	main_exhaust1[i].lpos = &exhaust_pos1[i];
	main_exhaust1[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust1[i].lsize = exlen;
	main_exhaust1[i].wsize = exwid;
	main_exhaust1[i].lofs = 0;
	main_exhaust1[i].modulate = 0.8;
	main_exhaust1[i].tex = exhaust;
	main_exhaust1[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust1[i].id = 0;
	AddExhaust (&main_exhaust1[i]);
	}
	///// 1 engine ////////////////////////
	for (int i = 0; i < 2; i++)
	{
	exhaust_pos2[i] = mul(rm,_V(0, 0,-20.5));

	main_exhaust2[i].th = th_main[2];
	main_exhaust2[i].level = NULL;
	main_exhaust2[i].lpos = &exhaust_pos2[i];
	main_exhaust2[i].ldir = NULL; //&main2_exhaust_dir;
	main_exhaust2[i].lsize = exlen;
	main_exhaust2[i].wsize = exwid;
	main_exhaust2[i].lofs = 0;
	main_exhaust2[i].modulate = 0.8;
	main_exhaust2[i].tex = exhaust;
	main_exhaust2[i].flags = EXHAUST_CONSTANTPOS;
	main_exhaust2[i].id = 0;
	AddExhaust (&main_exhaust2[i]);
	}

	/*AddExhaust(th_main[0],exlen,exwid,_V(0,0,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[0],exlen,exwid,_V(1.32,0,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[0],exlen,exwid,_V(-1.32,0,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[0],exlen,exwid,_V(0,1.32,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[0],exlen,exwid,_V(0,-1.32,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[0],exlen,exwid,_V(0.931,0.931,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[0],exlen,exwid,_V(-0.931,-0.931,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[0],exlen,exwid,_V(-0.931,0.931,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[0],exlen,exwid,_V(0.931,-0.931,-20.5),_V(0,0,-1),exhaust);

	AddExhaust(th_main[1],exlen,exwid,_V(0,0,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[1],exlen,exwid,_V(0,0,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[1],exlen,exwid,_V(1.32,0,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[1],exlen,exwid,_V(-1.32,0,-20.5),_V(0,0,-1),exhaust);

	AddExhaust(th_main[2],exlen,exwid,_V(0,0,-20.5),_V(0,0,-1),exhaust);
	AddExhaust(th_main[2],exlen,exwid,_V(0,0,-20.5),_V(0,0,-1),exhaust);*/


	SURFHANDLE exhaust2 = oapiRegisterParticleTexture ("FalconHeavy/Contrail4x");
	SURFHANDLE exhaust2x = oapiRegisterParticleTexture ("FalconHeavy/Contrail3");
	SURFHANDLE exhaustrcs = oapiRegisterParticleTexture ("Contrail1");

	PARTICLESTREAMSPEC exhaust_main1 = {
		//0, 1, 20, 120, 0.044, 0.11, 2.6, 0.3, PARTICLESTREAMSPEC::EMISSIVE,
		0, 1.0, 20, 120, 0.01, 0.08, 2.6, 1, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 0,
		PARTICLESTREAMSPEC::ATM_FLAT, 1, 1,
		exhaust2
	};

	PARTICLESTREAMSPEC exhaust_main2 = {
		//0, 1, 20, 110, 0.041, 0.09, 2.5, 0.3, PARTICLESTREAMSPEC::EMISSIVE,
		0, 1.0, 20, 120, 0.01, 0.08, 2.5, 1, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 0,
		PARTICLESTREAMSPEC::ATM_FLAT, 1, 1,
		exhaust2x
	};

	PARTICLESTREAMSPEC exhaust_rcs2 = {
		0, 0.1, 10, 100, 0.1, 0.05, 60, 10, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0.5, 1,
		PARTICLESTREAMSPEC::ATM_FLAT, 0.1, 0.5,
		exhaustrcs
	};

	PARTICLESTREAMSPEC atmostrail = {
		0, 3, 20, 2, 3, 1, -0.25, 10, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLIN, 0.5, 1,
		exhaustrcs
	};


	AddExhaustStream(th_main[0],_V(0,0,-21),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(1.32,0,-21),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(-1.32,0,-21),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(0,1.32,-21),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(0,-1.32,-21),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(0.931,0.931,-21),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(-0.931,-0.931,-21),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(-0.931,0.931,-21),&exhaust_main1);
	AddExhaustStream(th_main[0],_V(0.931,-0.931,-21),&exhaust_main1);

	AddExhaustStream(th_main[1],_V(0,0,-21),&exhaust_main1);
	AddExhaustStream(th_main[1],_V(1.32,0,-21),&exhaust_main1);
	AddExhaustStream(th_main[1],_V(-1.32,0,-21),&exhaust_main1);

	AddExhaustStream(th_main[2],_V(0,0,-21),&exhaust_main1);



	AddExhaustStream(th_main[0],_V(0,0,-21),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(1.32,0,-21),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(-1.32,0,-21),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(0,1.32,-21),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(0,-1.32,-21),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(0.931,0.931,-21),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(-0.931,-0.931,-21),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(-0.931,0.931,-21),&exhaust_main2);
	AddExhaustStream(th_main[0],_V(0.931,-0.931,-21),&exhaust_main2);

	AddExhaustStream(th_main[1],_V(0,0,-21),&exhaust_main2);
	AddExhaustStream(th_main[1],_V(1.32,0,-21),&exhaust_main2);
	AddExhaustStream(th_main[1],_V(-1.32,0,-21),&exhaust_main2);

	AddExhaustStream(th_main[2],_V(0,0,-21),&exhaust_main2);
	at_trail = AddExhaustStream(th_main[2],_V(0,0,-19),&atmostrail);

	//// rcs exhaust
	AddExhaustStream(th_rcs2[0], &exhaust_rcs2);
	AddExhaustStream(th_rcs2[1],_V( 1.884, 0.103, 18.963), &exhaust_rcs2);
	AddExhaustStream(th_rcs2[2], &exhaust_rcs2);
	AddExhaustStream(th_rcs2[3],_V( 1.884,-0.103, 18.963),&exhaust_rcs2);
	AddExhaustStream(th_rcs2[4], &exhaust_rcs2);
	AddExhaustStream(th_rcs2[3],_V(-1.884,-0.103, 18.963),&exhaust_rcs2);
	AddExhaustStream(th_rcs2[6], &exhaust_rcs2);
	AddExhaustStream(th_rcs2[1],_V(-1.884, 0.103, 18.963),&exhaust_rcs2);
	AddExhaustStream(th_rcs2[8], &exhaust_rcs2);
	AddExhaustStream(th_rcs2[9], &exhaust_rcs2);
	AddExhaustStream(th_rcs2[10], &exhaust_rcs2);
	AddExhaustStream(th_rcs2[11], &exhaust_rcs2);
	///////////////////////////// boom p streams

	th_boom[0] = CreateThruster (_V( 0, 0, 0), _V( 0,0,1), 0.001, hProp, 2, 1);
	th_boom[1] = CreateThruster (_V( 0, 0, 0), _V( 0,0,1), 0.001, hProp, 2, 1);

	SURFHANDLE boom1 = oapiRegisterParticleTexture ("Contrail4");
	SURFHANDLE boom2 = oapiRegisterParticleTexture ("Contrail3");

	PARTICLESTREAMSPEC boom_pst1 = {
		0, 8, 100, 150, 2, 2, 10, 4, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-7, 1,
		boom1
	};

	PARTICLESTREAMSPEC boom_pst2 = {
		0, 8, 80, 100, 2, 1, 15, 2, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-7, 1,
		boom2
	};

	AddExhaustStream(th_boom[0],_V(0,0,0),&boom_pst1);
	AddExhaustStream(th_boom[1],_V(0,0,0),&boom_pst2);


	///////////////////////////////////////////////////////
	/////AirFoils

	CreateAirfoil (LIFT_VERTICAL, _V(0,0,0.1), VLiftCoeff, 25, 80, 0.16);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,0.1), HLiftCoeff, 25, 80, 0.16); //90
	CreateVariableDragElement (&legs_proc, 12, _V(0, 0, 1)); // landing gear drag + stabiliser
	CreateVariableDragElement (&fins_deploy_proc, 9, _V(0, 0, 1)); // grid fins 3
	////////

	//local light
	COLOUR4 col_d = {0.125,0.1,0.06125,0};
	COLOUR4 col_s = {0,0,0,0};
	COLOUR4 col_a = {0,0,0,0};
	COLOUR4 col_white = {1,1,1,0};
	spt_light = (SpotLight*)AddSpotLight(_V(0,0,-21), _V(0,0,-1),5000, 1e-1, 0, 1e-7, RAD*170, RAD*180, col_d, col_s, col_a);
	//pnt_light = (PointLight*)AddPointLight(_V(0,0,-22),20000, 5e-7, 0, 5e-7, col_d, col_s, col_a);
	//pnt_light->Activate(false);
	spt_light->Activate(false);
	

}

void f9h_stg1_starboard::clbkPostCreation ()
{
 phGref = GetGravityRef();
 OBJHANDLE bbtgt = oapiGetObjectByName(prefname);
 if (bbtgt == NULL)bbtgt = oapiGetBaseByName(phGref, prefname);
 if (bbtgt == NULL)
 {
	boostback_sw = 0;
	strcpy(prefname, "NULL");
	afs_status = 0;
 }

 if (fins_deploy_status == 2)
	{
	hElevator = CreateControlSurface3 (AIRCTRL_ELEVATOR, 2, 3, _V(0,0,20), AIRCTRL_AXIS_XPOS, 0.5);
	hRudder = CreateControlSurface3 (AIRCTRL_RUDDER, 2, 3, _V(0,0,20), AIRCTRL_AXIS_YPOS, 0.5);
	}

  if (eng_statusp > 0) ConfigMain();
 

  if (leg_status == 2 )
  {
	SetTouchdownPoints ( tdvtx_geardown, ntdvtx_geardown);
	/*VESSELSTATUS2 vs;
	memset (&vs, 0, sizeof(vs));
	vs.version = 2;
	GetStatusEx(&vs);
	OBJHANDLE hVessel = GetHandle();
	double longitude, latitude, radius;
	double heading;
	oapiGetEquPos(hVessel,&longitude,&latitude,&radius);
	oapiGetHeading(hVessel,&heading);
	vs.surf_lat = latitude;
	vs.surf_lng = longitude;
	vs.surf_hdg = heading;*/
	//if(vs.status == 1)DefSetStateEx (&vs);
}


if (skin_sw == 2) ApplySkin();

}


// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new f9h_stg1_starboard (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (f9h_stg1_starboard*)vessel;
}


void f9h_stg1_starboard::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	for (int i = 0; i < (int)strlen(prefname); i++)
	{
		if (prefname[i] == ' ')
		{
			prefname[i] = '+';
			break;
		}
	}

	// default vessel parameters
	VESSEL3::clbkSaveState(scn);

	sprintf (cbuf, "%d %0.4f %0.4f %0.4f %0.4f", leg_status, legs_proc, pist_rot_proc, pist_trans_proc, skin_sw);
	oapiWriteScenario_string (scn, "LEGS", cbuf);

	sprintf (cbuf, "%d %0.4f %0.4f %0.4f", fins_deploy_status, fins_deploy_proc, fins_rot_a_proc, fins_rot_b_proc);
	oapiWriteScenario_string (scn, "FINS", cbuf);

	//sprintf (cbuf, "%d %d %d %d %d %d", eng_statusp, intfair_status, dp_status, boostback_sw, rat_sw, afs_status);
	sprintf (cbuf, "%d %d %d %d %d %d %d %0.4f %d", eng_statusp, dp_status, boostback_sw, rat_sw, afs_status, edlo_sw, mom_sw, boom_sw, reentry_correct_sw);
	oapiWriteScenario_string (scn, "APST", cbuf);

	sprintf (cbuf, "%s", prefname);
	oapiWriteScenario_string (scn, "LPRM", cbuf);


}
void f9h_stg1_starboard::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "LEGS", 4)) {
			sscanf (line+4, "%d%lf%lf%lf%lf", &leg_status, &legs_proc, &pist_rot_proc, &pist_trans_proc, &skin_sw);
		} else if (!strnicmp (line, "FINS", 4)) {
		sscanf (line+4, "%d%lf%lf%lf", &fins_deploy_status, &fins_deploy_proc, &fins_rot_a_proc, &fins_rot_b_proc);
		} else if (!strnicmp (line, "APST", 4)) {
		sscanf (line+4, "%d%d%d%d%d%d%d%lf%d", &eng_statusp, &dp_status, &boostback_sw, &rat_sw, &afs_status, &edlo_sw, &mom_sw, &boom_sw, &reentry_correct_sw);
		} else if (!strnicmp (line, "LPRM", 4)) {
			sscanf (line+4, "%s", &prefname);
		}else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }

	SetAnimation(anim_legs, legs_proc);
	SetAnimation(anim_pist_rot, pist_rot_proc);
	SetAnimation(anim_pist_trans, pist_trans_proc);

	SetAnimation(anim_fins_deploy, fins_deploy_proc);
	SetAnimation(anim_fins_rot_a, fins_rot_a_proc);
	SetAnimation(anim_fins_rot_b, fins_rot_b_proc);

	for (int i = 0; i < (int)strlen(prefname); i++)
	{
		if (prefname[i] == '+')
		{
			prefname[i] = ' ';
			break;
		}
	}

}



//bool SetBBTgt(void *id, char *str, void *data)
//{
//	char cbuf[256];
//	sscanf(str, "%s", &cbuf);
//	OBJHANDLE bbtgt = oapiGetObjectByName(cbuf);
//	if (bbtgt == NULL)bbtgt = oapiGetBaseByName(phGref, cbuf);
//	if (bbtgt == NULL)return false;
//	else 
//		{
//		strcpy(prefname,cbuf);
//		return true;
//		}
//}

int f9h_stg1_starboard::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) {
	/////////////////////////////////

	if (key == COMM_SEND_STR) {
			strcpy(prefname,kstate);
			jet_burn_sw = 1;
			return 1;
       }

	if (key == COMM_SEND_AFS) {
			
			bbtgtchk = oapiGetObjectByName(prefname);
			if (bbtgtchk == NULL)bbtgtchk = oapiGetBaseByName(GetGravityRef(), prefname);
			if (bbtgtchk == NULL) afs_status = 0;
			else afs_status = 1;
			return 1;
       }

	if (key == COMM_SEND_EDLO) {
			edlo_sw = 1;
			return 1;
       }

	//////////////////////////////////
if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) 
  {	 
	switch (key)
	{
	

	} 
  }

  else if (KEYMOD_CONTROL (kstate)) 
  { 
	switch (key)
	{
		
	} 
  }

else { // unmodified keys
    switch (key) {

		case OAPI_KEY_G: // legs
		if (leg_status == 0)
			{
			SetTouchdownPoints ( tdvtx_geardown, ntdvtx_geardown);
			leg_status = 1;
			}
		return 1;



		//case OAPI_KEY_M: // descent pilot
		//if (dp_status == 0)
		//	{
		//	rat_sw = 0;
		//	dp_status = 1;
		//	boostback_sw = 0;
		//	KillAllRCS();
		//	}
		//else if (dp_status == 1)
		//	{
		//	dp_status = 0;
		//	KillAllRCS();
		//	}
		//return 1;
		
		

		case OAPI_KEY_J: // deploy grid fins
		if (fins_deploy_status == 1)fins_deploy_status = 3;
		else if (fins_deploy_status == 2)fins_deploy_status = 3;
		else fins_deploy_status = 1;
		return 1;

		case OAPI_KEY_P: // configure main eng
		if (eng_statusp == 0) eng_statusp = 1;
		else if (eng_statusp == 1) eng_statusp = 2;
		else if (eng_statusp == 2) eng_statusp = 0;
		ConfigMain();
		return 1;

		/*case OAPI_KEY_K:
		phGref = GetGravityRef();
		oapiOpenInputBox("Enter Flyback Target Name",SetBBTgt, 0, 50, 0);
		return 1;*/

		case OAPI_KEY_M:
		if (edlo_sw == 0)edlo_sw = 1;
		else edlo_sw = 0;
		return 1;

		case OAPI_KEY_B: //
			bbchk = oapiGetObjectByName(prefname);
			if (bbchk == NULL)bbchk = oapiGetBaseByName(GetSurfaceRef(), prefname);
			if (bbchk == NULL)afs_status = 0;
			
			else if (afs_status_mem == 0)
			{
			if (afs_status == 0)
				{
			afs_status = 1;
			dp_status = 0;
			rat_sw = 0;
			boostback_sw = 0;
			KillAllRCS();
				}
			else
				{
			afs_status_mem = afs_status;
			afs_status = 0;
			dp_status_mem = dp_status;
			dp_status = 0;
			rat_sw_mem = rat_sw;
			rat_sw = 0;
			boostback_sw_mem = boostback_sw;
			boostback_sw = 0;
			KillAllRCS();
				}
			}
			else if (afs_status_mem > 0)
			{
				if (afs_status == 0)
				{
			afs_status = afs_status_mem;
			afs_status_mem = 0;
			dp_status = dp_status_mem;
			dp_status_mem = 0;
			rat_sw = rat_sw_mem;
			rat_sw_mem = 0;
			boostback_sw = boostback_sw_mem;
			boostback_sw_mem = 0;
			KillAllRCS();
				}
				else
				{
			afs_status_mem = afs_status;
			afs_status = 0;
			dp_status_mem = dp_status;
			dp_status = 0;
			rat_sw_mem = rat_sw;
			rat_sw = 0;
			boostback_sw_mem = boostback_sw;
			boostback_sw = 0;
			KillAllRCS();
				}
			}
		return 1;

		case OAPI_KEY_E:
			if (cam_status == 0)cam_status = 1;
			else if (cam_status == 1)cam_status = 0;
			SetCamDir();
		return 1;

		//case OAPI_KEY_V:
		//if (boostback_sw == 0)
		//	{
		//	rat_sw = 0;
		//	dp_status = 0;
		//	boostback_sw = 1;
		//	KillAllRCS();
		//	}
		//else if (boostback_sw > 0)
		//{
		//	boostback_sw = 0;
		//	KillAllRCS();
		//}
		//return 1;

		//case OAPI_KEY_N:
		//if (rat_sw == 0)
		//{
		//	rat_sw = 1;
		//	dp_status = 0;
		//	boostback_sw = 0;
		//	KillAllRCS();
		//}
		//else if (rat_sw > 0)
		//{
		//	rat_sw = 0;
		//	KillAllRCS();
		//}
		//return 1;
	}
  }
  return 0;
}




void f9h_stg1_starboard::clbkPreStep(double SimT, double SimDT, double MJD)
{
SetThrusterDir(th_main[eng_statusp], _V(0,0,1));
double getalt = GetAltitude(ALTMODE_GROUND);
rchud_sw = 0;

if (getalt < 5)
{
	if(GroundContact() == true && boom_sw == 0) boom_sw = 1;
}
if (boom_sw > 0 && boom_sw < 2)DoBoom();

////// char skin
//if (getalt < 100000)
//{
	if (skin_sw < 1)
	{
		if (GetThrusterGroupLevel(THGROUP_MAIN) > 0)
		{
			VECTOR3 aspdloc = _V(0,0,0);
			GetShipAirspeedVector(aspdloc);
			if (aspdloc.z < 0)
			{
				skin_sw = skin_sw+(0.5*SimDT);
				if (skin_sw > 1)
				{
					skin_sw = 2; // finished
					ApplySkin();
				}
			}
		}

	}
//}
//////////// 
///// heat experiment
	//double asp = GetAirspeed ();
	//double atmden = GetAtmDensity();
	//double r_nose = 2;
	//double heatflux2 = atmden*asp*asp*asp*0.5;
	//double heatflux = 0.000183*asp*asp*asp*sqrt(atmden/r_nose);
	
///////////////

if (getalt < 50 && dust_sw == 0)
{
SURFHANDLE exhaustrcs = oapiRegisterParticleTexture ("Contrail1");

PARTICLESTREAMSPEC dust_pstream = {
		0, 5, 20, 200, 0.4, 1.25, 20, 5, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 0.7e-7, 1,
		exhaustrcs
	};
DelExhaustStream(at_trail);
AddExhaustStream(th_main[2],_V(0,0,-24),&dust_pstream);
dust_sw = 1;
}

step_time = SimDT;///////
double simdt = SimDT;

if (jet_burn_sw == 1)
{
ptimer2 = ptimer2+SimDT;
if (ptimer2 < 2)
	{
	eng_statusp = 2;
	ConfigMain();
	SetThrusterLevel(th_main[eng_statusp],0.4);
	}
else
	{
	SetThrusterLevel(th_main[eng_statusp],0);
	eng_statusp = 1;
	ConfigMain();
	jet_burn_sw = 0;
	ptimer2 = 0;
	}
}

double mth_lev = GetThrusterLevel(th_main[eng_statusp]);
if (mth_lev == 0 && rcs_sw == 0) ConfigRCS();
else if (mth_lev > 0 && rcs_sw == 1) RevertRCS();

////////////////engine light
if (mth_lev > 0)
{
	double rndm = 0.15*(oapiRand());
	rndm = (mth_lev*0.9) - rndm;
	if (rndm < 0)rndm = 0;
	else if (rndm > 1)rndm = 1;
	spt_light->Activate(true);
	spt_light->SetIntensity(rndm);

}
else spt_light->Activate(false);

if (afs_status == 1)
	{
	timer = timer+SimDT;
	if (timer > 4) afs_status = 2;
	}
else if(afs_status == 2)
	{
	if (edlo_sw == 1) 
	{
		boostback_sw = 0;		
		afs_status = 3;
	}
	else if (boostback_sw == 0)boostback_sw = 1;
	}
else if(afs_status == 3)
	{
	if (rat_sw == 0 && GetNavmodeState(NAVMODE_KILLROT) == false)rat_sw = 1;
	if (getalt < 100000 && getalt > 15000 && rc_sw == 0) //30000
		{
			VECTOR3 haspv;
			GetAirspeedVector(FRAME_HORIZON, haspv);
			if (haspv.y < 0)
			{
			if (fins_deploy_status == 0)fins_deploy_status = 1;
			DoReentryCorrection();
			}
		}
	if (rc_sw == 0 && getalt < 15000) // 30000
		{
			rc_sw = 1;
			SetThrusterGroupLevel(THGROUP_MAIN, 0);
			mth_level = 0;
		}
	if (getalt < 15000 && rat_sw ==1) //30000
		{
		rat_sw = 0;
		afs_status = 4;
		}

	}
else if(afs_status == 4)
	{
	if (dp_status == 0)dp_status = 1;
	}

//if (calc_sw == 0)
//{
//CalcEmptyMass();
//calc_sw = 1;
//}

if (dp_status == 1)
{
	DoLand();
	if (fins_deploy_status == 0)fins_deploy_status = 1;
}

if (boostback_sw == 1)
{
	DoBoostBack();
	DoBoostBack2();
}
if (boostback_sw == 2)
{
	DoBoostBack();
}

if (rat_sw == 1)DoRAT();


//sprintf(oapiDebugString(), "csw %d st %0.1f", crash_sw, oapiGetSimTime());
//sprintf(oapiDebugString(), "%d %0.4f %0.4f", (sa1_stbd_elb_sts + sa1_stbd_scale_sts + sa1_port_elb_sts + sa1_port_scale_sts),sa1_port_rot_proc,sa1_stbd_rot_proc);

////////////////// ANIMATIONS ///////////////

if (leg_status > 0)
{
double da = SimDT * LEG_SPEED;

if (leg_status == 1)
	{
	legs_proc = legs_proc+da;
if (legs_proc > 1)
	{
	legs_proc = 1;
	leg_status = 2;
	SetTouchdownPoints ( tdvtx_geardown, ntdvtx_geardown);
	}
SetAnimation(anim_legs, legs_proc);
}

else if (leg_status == 2)
{
SetAnimation(anim_legs, 1);
}

else if (leg_status == 3)
	{
	legs_proc = legs_proc-da;
if (legs_proc < 0)
	{
	legs_proc = 0;
	leg_status = 0;
	}
SetAnimation(anim_legs, legs_proc);
}

VECTOR3 leg1_att = _V(1.59, 0, -19.212);
VECTOR3 leg1_tip = _V(2, 0, -10.807);
VECTOR3 pist_att = _V(2, 0, -15.96);

leg1_tip = leg1_tip - leg1_att;
pist_att = pist_att - leg1_att;

VECTOR3 orig_pist = leg1_tip - pist_att;
double orig_pist_length = length(orig_pist);

double theta = (120*RAD)*legs_proc;
VECTOR3 curr_leg1_tip = _V(0,0,0);

double sin_theta = sin(theta);
double cos_theta = cos(theta);

curr_leg1_tip.z = (leg1_tip.z*cos_theta) - (leg1_tip.x*sin_theta);
curr_leg1_tip.x = (leg1_tip.z*sin_theta) + (leg1_tip.x*cos_theta);

VECTOR3 req_pist = curr_leg1_tip - pist_att;
double req_pist_length = length(req_pist);

double excess_length = req_pist_length - orig_pist_length;

pist_trans_proc = excess_length/5.08;
if (pist_trans_proc < 0) pist_trans_proc = 0;
if (pist_trans_proc > 1) pist_trans_proc = 1;


normalise(orig_pist);
normalise(req_pist);

dotprod = dotp(orig_pist,req_pist);
if (dotprod > 1)dotprod = 1;
else if (dotprod < -1)dotprod = -1;

double req_angle = acos(dotprod);
req_angle = DEG*req_angle;

pist_rot_proc = req_angle/139.5;
if (pist_rot_proc < 0) pist_rot_proc = 0;
if (pist_rot_proc > 1) pist_rot_proc = 1;

if (legs_proc == 1) pist_rot_proc = 1;
SetAnimation(anim_pist_rot, pist_rot_proc);

if (legs_proc == 1) pist_trans_proc = 1;
SetAnimation(anim_pist_trans, pist_trans_proc);
}

///////////// fins deploy

if (fins_deploy_status == 1)
{
double da = SimDT * FIN_SPEED;
fins_deploy_proc = fins_deploy_proc+da;
if (fins_deploy_proc > 1)
	{
		fins_deploy_proc = 1;
		fins_deploy_status = 2;
		hElevator = CreateControlSurface3 (AIRCTRL_ELEVATOR, 2, 3, _V(0,0,20), AIRCTRL_AXIS_XPOS, 0.5);
		hRudder = CreateControlSurface3 (AIRCTRL_RUDDER, 2, 3, _V(0,0,20), AIRCTRL_AXIS_YPOS, 0.5);
	}
SetAnimation(anim_fins_deploy, fins_deploy_proc);
}
else if (fins_deploy_status == 2)SetAnimation(anim_fins_deploy, fins_deploy_proc);
else if (fins_deploy_status == 3)
{
	if (fins_deploy_proc == 1)
	{
		ClearControlSurfaceDefinitions ();
	}
double da = SimDT * FIN_SPEED;
fins_deploy_proc = fins_deploy_proc-da;
if (fins_deploy_proc < 0)
	{
		fins_deploy_proc = 0;
		fins_deploy_status = 0;
	}
SetAnimation(anim_fins_deploy, fins_deploy_proc);
}

///////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
//////// fins rotate

if (fins_deploy_status > 1)
{
double pitch_up_level = GetThrusterGroupLevel(THGROUP_ATT_PITCHUP);
double pitch_down_level = GetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN);
double yaw_right_level = GetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT);
double yaw_left_level = GetThrusterGroupLevel(THGROUP_ATT_YAWLEFT);

VECTOR3 torqe_vector = _V((yaw_right_level-yaw_left_level),(pitch_up_level-pitch_down_level),0);
if (fins_deploy_status == 2)
{

double atmfact = ((GetDynPressure()-0000)*0.00005);
if(atmfact > 1)atmfact = 1;
else if(atmfact < 0)atmfact = 0;
atmfact = atmfact*0.8;
atmfact = 1-atmfact;

//sprintf(oapiDebugString(), "hf %0.1fkw hf2 %0.1fkw", heatflux/1000, heatflux2/1000);

if (rcs_sw == 0)
	{
	SetControlSurfaceLevel(AIRCTRL_ELEVATOR, torqe_vector.y*0.3);
	SetControlSurfaceLevel(AIRCTRL_RUDDER, torqe_vector.x*0.3);
	}
else
	{
	double csl_x = 0.6*torqe_vector.x*atmfact;
	double csl_y = 0.6*torqe_vector.y*atmfact;
	//if(abs(csl_x) < 0.05)csl_x = 0;
	//if(abs(csl_y) < 0.05)csl_y = 0;
	SetControlSurfaceLevel(AIRCTRL_ELEVATOR, csl_y);
	SetControlSurfaceLevel(AIRCTRL_RUDDER, csl_x);
	}
}
double torque_factor = 80*length(torqe_vector);
if (rcs_sw == 0)torque_factor = 40*length(torqe_vector);
if (torque_factor > 1)torque_factor = 1;
normalise(torqe_vector);

double torque_angle = atan2(torqe_vector.y, torqe_vector.x);
double ta_adjusted = torque_angle+(PI/4);
if (ta_adjusted > PI2) ta_adjusted = ta_adjusted - PI2;
else if (ta_adjusted < 0) ta_adjusted = ta_adjusted + PI2;

double req_fin_pitch = 0.5 + (sin(ta_adjusted)*torque_factor*0.5);
double req_fin_yaw = 0.5 + (cos(ta_adjusted)*torque_factor*0.5);

if ((pitch_up_level+pitch_down_level+yaw_right_level+yaw_left_level) == 0)
{
	req_fin_pitch = 0.5;
	req_fin_yaw = 0.5;
}

if (fins_deploy_status == 3)
{
	req_fin_pitch = 0.5;
	req_fin_yaw = 0.5;
}

double db = SimDT*FIN_SPEED;

if (abs(req_fin_pitch-fins_rot_a_proc) < db)fins_rot_a_proc = req_fin_pitch;
else if (req_fin_pitch < fins_rot_a_proc)fins_rot_a_proc = fins_rot_a_proc-db;
else if (req_fin_pitch > fins_rot_a_proc)fins_rot_a_proc = fins_rot_a_proc+db;
SetAnimation(anim_fins_rot_a, fins_rot_a_proc);

if (abs(req_fin_yaw-fins_rot_b_proc) < db)fins_rot_b_proc = req_fin_yaw;
else if (req_fin_yaw < fins_rot_b_proc)fins_rot_b_proc = fins_rot_b_proc-db;
else if (req_fin_yaw > fins_rot_b_proc)fins_rot_b_proc = fins_rot_b_proc+db;
SetAnimation(anim_fins_rot_b, fins_rot_b_proc);


}
DoThrustVector ();

if(landed_sw == 1)
	{
	if (GetFlightStatus() == 00) DoLandingRCS();
	else
		{
		SetThrusterLevel(th_rcs2[0], 0);
		SetThrusterLevel(th_rcs2[1], 0);
		SetThrusterLevel(th_rcs2[2], 0);
		SetThrusterLevel(th_rcs2[3], 0);
		}
	}
}

void f9h_stg1_starboard::clbkPostStep (double simt, double simdt, double mjd)
{
	if (GroundContact() == true)landed_sw = 1;

}




bool f9h_stg1_starboard::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);

	//int chw = 16;

	
	char test[256];
	strcpy(test,"abcdefghijklmnopqrstuvwxyz");
	int it = skp->GetTextWidth(test, 26);
	it = (int)(it/26);
	int chw = it;

	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}



	int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	int w3 = (int)(232*d);
	int w4 = (int)(248*d);
	int w5 = (int)(264*d);
	int w6 = (int)(280*d);
	int w7 = (int)(296*d);
	int w8 = (int)(312*d);
	int w9 = (int)(328*d);
	int w10 = (int)(344*d);
	int w11 = (int)(360*d);
	int w12 = (int)(376*d);
	int w13 = (int)(392*d);
	int w14 = (int)(408*d);
	int w15 = (int)(424*d);



	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	double m1 = GetEmptyMass()+GetPropellantMass(hRCSProp);
	double m2 = GetPropellantMass(hProp);
	double test = ((m1+m2)/m1);
	double isp_curr = GetThrusterIsp(th_main[2]);
	double zdv = (isp_curr*(log(test)));

	sprintf(abuf,"Remaining dV %0.1fm/s : RCS(N)Prop. %0.1fkg", zdv, GetPropellantMass(hRCSProp) );
	skp->Text ((10+roxl), (wc+royl), abuf,strlen(abuf));
	}


	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	sprintf(abuf,"Flyback Target: %s", prefname);
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x00FFFFFF);
	char abuf[256];
	if(edlo_sw == 1)sprintf(abuf,"M = Set Flyback Mode: EDL ONLY");
	else sprintf(abuf,"M = Set Flyback Mode: FLYBACK-EDL");
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}

	if (afs_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	if (edlo_sw == 1) sprintf(abuf,"B = Enable Auto EDL ONLY");
	else sprintf(abuf,"B = Enable Auto FLYBACK-EDL");
	skp->Text ((10+roxl), (w2+royl), abuf,strlen(abuf));
	}
	else
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	if (edlo_sw == 1) sprintf(abuf,"B = Auto EDL ONLY Active");
	else sprintf(abuf,"B = Auto FLYBACK-EDL Active");
	skp->Text ((10+roxl), (w2+royl), abuf,strlen(abuf));
	}

	
	if (rchud_sw == 1)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"FlyBack Burn Enabled  : Required dV %0.1f m/s", fb_dv_req);
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}
	else if (rchud_sw == 2)
	{
	skp->SetTextColor(0x006666FF);
	char abuf[256];
	sprintf(abuf,"FlyBack Burn Active : Required dV %0.1f m/s", fb_dv_req);
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}
	else if (rchud_sw == 3)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"Re-Entry Attitude Enabled");
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}
	else if (rchud_sw == 4)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"Re-Entry Burn Enabled");
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}
	else if (rchud_sw == 5)
	{
	skp->SetTextColor(0x006666FF);
	char abuf[256];
	sprintf(abuf,"Re-Entry Burn Active");
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}
	else if (rchud_sw == 6)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"Aero.Steering/Landing Burn Enabled");
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}
	else if (rchud_sw == 7)
	{
	skp->SetTextColor(0x006666FF);
	char abuf[256];
	sprintf(abuf,"Landing Burn Active");
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}
	
	if (rchud_sw == 4)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"Flt.Rng. %0.1fkm : Tgt.Dist. %0.1fkm", hud_flight_range*0.001, hud_range*0.001);
	skp->Text ((10+roxl), (w4+royl), abuf,strlen(abuf));
	}
	else if (rchud_sw == 5)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"Flt.Rng. %0.1fkm : Tgt.Dist. %0.1fkm", hud_flight_range*0.001, hud_range*0.001);
	skp->Text ((10+roxl), (w4+royl), abuf,strlen(abuf));
	}
	

	if (eng_statusp == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"P = Engine Selection : 9");
	skp->Text ((10+roxl), (w6+royl), abuf,strlen(abuf));
	}

	else if (eng_statusp == 1)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"P = Engine Selection : 3");
	skp->Text ((10+roxl), (w6+royl), abuf,strlen(abuf));
	}

	else if (eng_statusp == 2)
	{
	skp->SetTextColor(0x006666FF);
	char abuf[256];
	sprintf(abuf,"P = Engine Selection : 1");
	skp->Text ((10+roxl), (w6+royl), abuf,strlen(abuf));
	}

	if (fins_deploy_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"J = Deploy Grid Fins");
	skp->Text ((10+roxl), (w7+royl), abuf,strlen(abuf));
	}
	else if (fins_deploy_status == 1)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"J = Stow Grid Fins");
	skp->Text ((10+roxl), (w7+royl), abuf,strlen(abuf));
	}
	else if (fins_deploy_status == 2)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"J = Stow Grid Fins");
	skp->Text ((10+roxl), (w7+royl), abuf,strlen(abuf));
	}
	else if (fins_deploy_status == 3)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"J = Deploy Grid Fins");
	skp->Text ((10+roxl), (w7+royl), abuf,strlen(abuf));
	}

	if (leg_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"G = Deploy Landing Gear");
	skp->Text ((10+roxl), (w8+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"E = Set View +Z/-Z");
	skp->Text ((10+roxl), (w9+royl), abuf,strlen(abuf));
	}

	return true;
}



void f9h_stg1_starboard::RevertRCS (void)
{
	SetThrusterGroupLevel(htgrppup,0);
	SetThrusterGroupLevel(htgrppdown,0);
	SetThrusterGroupLevel(htgrpbright,0);
	SetThrusterGroupLevel(htgrpbleft,0);
	SetThrusterGroupLevel(htgrpyright,0);
	SetThrusterGroupLevel(htgrpyleft,0);

	DelThrusterGroup(htgrppup,THGROUP_ATT_PITCHUP, false);
	DelThrusterGroup(htgrppdown,THGROUP_ATT_PITCHDOWN, false);
	DelThrusterGroup(htgrpbright,THGROUP_ATT_BANKRIGHT, false);
	DelThrusterGroup(htgrpbleft,THGROUP_ATT_BANKLEFT, false);
	DelThrusterGroup(htgrpyright,THGROUP_ATT_YAWRIGHT, false);
	DelThrusterGroup(htgrpyleft,THGROUP_ATT_YAWLEFT, false);

	th_group[0] = th_rcs[9];
	th_group[1] = th_rcs[11];
	htgrppup = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[10];
	htgrppdown = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[3];
	htgrpyright = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[2];
	htgrpyleft = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[4];
	th_group[1] = th_rcs[7];
	htgrpbright = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[5];
	th_group[1] = th_rcs[6];
	htgrpbleft = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);

	rcs_sw = 0;
	vrot_lim = 5*RAD;
}

void f9h_stg1_starboard::ConfigRCS (void)
{
	SetThrusterGroupLevel(htgrppup,0);
	SetThrusterGroupLevel(htgrppdown,0);
	SetThrusterGroupLevel(htgrpbright,0);
	SetThrusterGroupLevel(htgrpbleft,0);
	SetThrusterGroupLevel(htgrpyright,0);
	SetThrusterGroupLevel(htgrpyleft,0);

	DelThrusterGroup(htgrppup,THGROUP_ATT_PITCHUP, false);
	DelThrusterGroup(htgrppdown,THGROUP_ATT_PITCHDOWN, false);
	DelThrusterGroup(htgrpbright,THGROUP_ATT_BANKRIGHT, false);
	DelThrusterGroup(htgrpbleft,THGROUP_ATT_BANKLEFT, false);
	DelThrusterGroup(htgrpyright,THGROUP_ATT_YAWRIGHT, false);
	DelThrusterGroup(htgrpyleft,THGROUP_ATT_YAWLEFT, false);

	th_group[0] = th_rcs2[3];
	th_group[1] = th_rcs2[5];
	htgrppup = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs2[1];
	th_group[1] = th_rcs2[7];
	htgrppdown = CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs2[8];
	th_group[1] = th_rcs2[11];
	htgrpbright = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs2[9];
	th_group[1] = th_rcs2[10];
	htgrpbleft = CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);

	th_group[0] = th_rcs2[2];
	th_group[1] = th_rcs2[4];
	htgrpyright = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs2[0];
	th_group[1] = th_rcs2[6];
	htgrpyleft = CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	rcs_sw = 1;
	vrot_lim = 3*RAD;
}

void f9h_stg1_starboard::ConfigMain (void)
{
	double old_tg_lev = GetThrusterGroupLevel(htgrpmainp);
	SetThrusterGroupLevel(htgrpmainp,0);



	DelThrusterGroup(htgrpmainp,false);
	DelThrusterGroup(THGROUP_MAIN, false);

	if (eng_statusp == 0) 
	{
		//th_group[0] = th_main[0];
		SetThrusterLevel(th_main[1],0);
		SetThrusterLevel(th_main[2],0);
		int i;
		for (i = 0; i < 12; i++)
		{
		SetThrusterMax0(th_rcs[i],RCSTH*1);
		}
	}
	else if (eng_statusp == 1)
	{
		//th_group[0] = th_main[1];
		SetThrusterLevel(th_main[0],0);
		SetThrusterLevel(th_main[2],0);
		int i;
		for (i = 0; i < 12; i++)
		{
		SetThrusterMax0(th_rcs[i],RCSTH*0.25);
		}
	}
	else if (eng_statusp == 2)
	{
		//th_group[0] = th_main[2];
		SetThrusterLevel(th_main[0],0);
		SetThrusterLevel(th_main[1],0);
		int i;
		for (i = 0; i < 12; i++)
		{
		SetThrusterMax0(th_rcs[i],RCSTH*0.15);
		}
	}
	htgrpmainp = CreateThrusterGroup (&th_main[eng_statusp], 1, THGROUP_MAIN);

	SetThrusterGroupLevel(htgrpmainp,old_tg_lev);
}

void f9h_stg1_starboard::KillAllRCS()
{
SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
SetThrusterGroupLevel(THGROUP_ATT_BANKRIGHT, 0);
SetThrusterGroupLevel(THGROUP_ATT_BANKLEFT, 0);
}


void f9h_stg1_starboard::DoLand(void)
{
double m2 = GetPropellantMass(hProp);
if (m2 > 20000 && eng_statusp == 2)
	{
	eng_statusp = 1;
	ConfigMain();
	}
else if (m2 > 20000 && eng_statusp == 0)
	{
	eng_statusp = 1;
	ConfigMain();
	}
else if (m2 < 20000 && eng_statusp == 1)
	{
	eng_statusp = 2;
	ConfigMain();
	}
else if (m2 < 20000 && eng_statusp == 0)
	{
	eng_statusp = 2;
	ConfigMain();
	}

VECTOR3 weightv = _V(0,0,0);
double weight = 0;
GetWeightVector(weightv);
weight = length(weightv);

VECTOR3 haspdv = _V(0,0,0);
GetGroundspeedVector(FRAME_HORIZON, haspdv);

VECTOR3 aspdv = _V(0,0,0);
GetGroundspeedVector(FRAME_LOCAL, aspdv);
double air_spd = length(haspdv);

	VECTOR3 z_axis_ves = _V(0,0,1);
	VECTOR3 z_axis_hrz = _V(0,0,1);
	HorizonRot(z_axis_ves, z_axis_hrz);
	normalise(z_axis_hrz);
	double ves_pitch = PI05 - acos(dotp(z_axis_hrz, _V(0,1,0)));

double pitch = ves_pitch;
double cosp = (cos(pitch));
double sinp = (sin(pitch));
double pf = (1/(sin(pitch)));

//double bank = GetBank();
//double cosb = (cos(bank));
//double bf = (1/(cos(bank)));

double alt = GetAltitude(ALTMODE_GROUND);
double mass = 0;
VECTOR3 drag = _V(0,0,0);
GetDragVector(drag);
VECTOR3 drag_hrz = _V(0,0,0);
HorizonRot(drag,drag_hrz);
double th_max = GetThrusterMax(th_main[eng_statusp]);

	double m1 = GetEmptyMass()+GetPropellantMass(hRCSProp);
	mass = m1+m2;

	double test = ((m1+m2)/m1);
	double isp_curr = GetThrusterIsp(th_main[eng_statusp]);
	double dv = (isp_curr*(log(test)));

	/*if (alt < 15000 && alt > 4000)
	{
		if (m2 > 4000 && correct_sw == 0) 
		{
			DoCorrection();
		}
		else
		{	
		SetThrusterLevel(th_main[eng_status],0);
		KillAllRCS();
		correct_sw = 1;
		rot_sw = 0;
		
		}
	}
	else if (alt < 4000)
	{
	correct_sw = 1;
	}*/

	if (alt < 15000)// && correct_sw == 1)
	{
		if (GroundContact() == true)landed_sw = 1; // ensure shutdown on landing
		if (landed_sw == 1)
		{
		double englvl = GetThrusterGroupLevel(THGROUP_MAIN);
		if (englvl > 0.2)englvl = 0.2; // ensure bloody SHUTDOWN, I said!
		englvl = englvl - (step_time*1.0); //0.5
		SetThrusterGroupLevel(THGROUP_MAIN, englvl);
		mth_level = englvl;
		if(englvl < 0)
			{
			SetThrusterGroupLevel(THGROUP_MAIN, 0);
			mth_level = 0;
			KillAllRCS();
			dp_status = 0;
			afs_status = 0;
			fins_deploy_status = 3;
			}
			return;
		}

	if (alt < 300 && leg_status == 0) 
		{
		leg_status = 1;
		SetTouchdownPoints ( tdvtx_geardown, ntdvtx_geardown);
		}
	if (alt > 35) DoLAT();//40
	else if (alt > 24.5)DoTWU();

	double alt_factor = m2/2625;//3000;
	if (alt_factor < 1.0)alt_factor = 1.0;
	else if (alt_factor > 2.0)alt_factor = 2.0;//

	double accfact = 1.0;

	double start_alt = (100+(750*alt_factor));//-250
	if(alt > start_alt)
	{
	double time = (-2*(alt-start_alt)/(haspdv.y+40));

	double acc = accfact*((-(haspdv.y+40))/time);

	double max_acc = 0.5*(((sinp*th_max)-weight+drag_hrz.y)/mass); //0.75

	double force = ((mass*acc)+ weight - drag_hrz.y);

	double req_englvl = min(0.7, pf*force/th_max);

	req_englvl = max(0,req_englvl);

	double englvl = GetThrusterGroupLevel(THGROUP_MAIN);

	if(englvl < req_englvl)
	{
	englvl = englvl + step_time;
	if(englvl > req_englvl)englvl = req_englvl;
	}
	else if(englvl > req_englvl)
	{
	englvl = englvl - step_time;
	if(englvl < req_englvl)englvl = req_englvl;
	}

	if (GetThrusterGroupLevel(THGROUP_MAIN) > 0)max_acc = 0;
	if (max_acc < acc)
	{
	SetThrusterGroupLevel(THGROUP_MAIN, englvl);
	mth_level = englvl;
	}
	else
	{
		SetThrusterGroupLevel(THGROUP_MAIN, 0);
		mth_level = 0;
	}
	if(haspdv.y > 0)
	{
		SetThrusterGroupLevel(THGROUP_MAIN, 0);
		mth_level = 0;
	}
	}

	else if (alt > ((alt_factor*200)-100))
	{
	double time = (-2*(alt-((alt_factor*200)-100))/(haspdv.y+25));


	double acc = accfact*((-(haspdv.y+25))/time);

	double force = ((mass*acc)+ weight - drag_hrz.y);
	
	double englvl = min(0.7, pf*force/th_max);

	englvl = max(0,englvl);

	//sprintf(oapiDebugString(), "f %0.4f t %0.4f",force, (th_max*englvl*sinp));

	SetThrusterGroupLevel(THGROUP_MAIN, englvl);
	mth_level = englvl;
	if(haspdv.y > 0)
		{
		SetThrusterGroupLevel(THGROUP_MAIN, 0);
		mth_level = 0;
		}
	}
	else if (alt > (1*35))
	{
	double time = (-2*(alt-(1*35))/(haspdv.y+4));


	double acc = accfact*((-(haspdv.y+4))/time);

	double force = ((mass*acc)+ weight - drag_hrz.y);
	
	double req_englvl = min(0.7, pf*force/th_max);

	req_englvl = max(0,req_englvl);

	double englvl = GetThrusterGroupLevel(THGROUP_MAIN);

	if(englvl < req_englvl)
	{
	englvl = englvl + step_time;
	if(englvl > req_englvl)englvl = req_englvl;
	}
	else if(englvl > req_englvl)
	{
	englvl = englvl - step_time;
	if(englvl < req_englvl)englvl = req_englvl;
	}

	if (drag_hrz.y < 0)englvl = 0;
	SetThrusterGroupLevel(THGROUP_MAIN, englvl);
	mth_level = englvl;
	
	}
	else if (alt > 26)
	{
	double time = (-2*(alt-26)/(haspdv.y+2.0));

	if (time < 0)landed_sw = 1;

	double acc = ((-(haspdv.y+2.0))/time);

	double force = ((mass*acc)+ weight - drag_hrz.y);
	
	double req_englvl = min(0.7, pf*force/th_max);

	req_englvl = max(0,req_englvl);

	double englvl = GetThrusterGroupLevel(THGROUP_MAIN);

	if(englvl < req_englvl)
	{
	englvl = englvl + step_time;
	if(englvl > req_englvl)englvl = req_englvl;
	}
	else if(englvl > req_englvl)
	{
	englvl = englvl - step_time;
	if(englvl < req_englvl)englvl = req_englvl;
	}

	if (drag_hrz.y < 0)englvl = 0;
	SetThrusterGroupLevel(THGROUP_MAIN, englvl);
	mth_level = englvl;
	}
	else if (alt > 25)
	{
	double time = (-2*(alt-25)/(haspdv.y+2.0));

	double acc = ((-(haspdv.y+2.0))/time);

	if (haspdv.y > -1)acc = 0;

	double force = ((mass*acc)+ weight - drag_hrz.y);
	
	double req_englvl = min(0.7, pf*force/th_max);

	req_englvl = max(0,req_englvl);

	double englvl = GetThrusterGroupLevel(THGROUP_MAIN);

	if(englvl < req_englvl)
	{
	englvl = englvl + step_time;
	if(englvl > req_englvl)englvl = req_englvl;
	}
	else if(englvl > req_englvl)
	{
	englvl = englvl - step_time;
	if(englvl < req_englvl)englvl = req_englvl;
	}

	if (drag_hrz.y < 0)englvl = 0;

	SetThrusterGroupLevel(THGROUP_MAIN, englvl);
	mth_level = englvl;
	}
	else
		{
	landed_sw = 1;
	double englvl = GetThrusterGroupLevel(THGROUP_MAIN);
	englvl = englvl - (step_time*0.5);
	SetThrusterGroupLevel(THGROUP_MAIN, englvl);
	mth_level = englvl;
	if(englvl < 0)
		{
		SetThrusterGroupLevel(THGROUP_MAIN, 0);
		mth_level = 0;
		KillAllRCS();
		dp_status = 0;
		afs_status = 0;
		fins_deploy_status = 3;
		}
		}
	}
}


void f9h_stg1_starboard::DoRAT(void)
{
	
	double angoa = GetAOA();
	double slipage = GetSlipAngle();

	if(angoa > 0)angoa = angoa-PI;
	else angoa = angoa+PI;

	if(slipage > 0)slipage = slipage-PI;
	else slipage = slipage+PI;
	slipage = -slipage;

	double mass = GetMass();

	double max_rcs = GetThrusterMax(th_rcs[0]);
	double max_torque = 2*max_rcs*21;

	if (rcs_sw == 1)
	{
		max_rcs = GetThrusterMax(th_rcs2[0]);
		max_torque = max_rcs*(18.12+18.8);
	}

	double inertia = 129*mass;

	double finfact = 1;//(GetDynPressure()/40000);
	if (fins_deploy_status > 0 && rcs_sw == 1) finfact = 0.05;

	VECTOR3 avel = _V(0,0,0);
	GetAngularVel(avel);
	double max_acc = max_torque/inertia;
	double max_drot = max_acc*step_time;

	double reqxv = 0.8*(sqrt(abs(angoa*max_acc*2)));
	if (reqxv > vrot_lim)reqxv = vrot_lim;
	if (angoa > 0)reqxv = -reqxv;
	double req_drotx = reqxv - avel.x;
	double thx_lvl = req_drotx/max_drot;
	if((angoa*DEG) < 1 && (angoa*DEG) > -1)
	{
	reqxv = 1*-angoa;
	double acc_reqx = (reqxv-avel.x)/0.1;
	thx_lvl = acc_reqx/max_acc;
	}
	thx_lvl = thx_lvl*finfact;

	double reqyv = 0.8*(sqrt(abs(slipage*max_acc*2)));
	if (reqyv > vrot_lim)reqyv = vrot_lim;
	if (slipage > 0)reqyv = -reqyv;
	double req_droty = reqyv - avel.y;
	double thy_lvl = req_droty/max_drot;
	if((slipage*DEG) < 1 && (slipage*DEG) > -1)
	{
	reqyv = 1*-slipage;
	double acc_reqy = (reqyv-avel.y)/0.1;
	thy_lvl = acc_reqy/max_acc;
	}
	thy_lvl = thy_lvl*finfact;

	//sprintf(oapiDebugString(), "slip %0.1f avely %0.1f", slipage*DEG, avel.y*DEG);
	//sprintf(oapiDebugString(), "angoa %0.6f thy_lvl %0.1f thx_lvl %0.1f",angoa*DEG, thy_lvl, thx_lvl);

	if(thx_lvl > 1)thx_lvl = 1;
	else if(thx_lvl < -1)thx_lvl = -1;

	if(thy_lvl > 1)thy_lvl = 1;
	else if(thy_lvl < -1)thy_lvl = -1;

	double tacc = oapiGetTimeAcceleration();
	if(oapiGetTimeAcceleration() > 2)
	{
		thx_lvl = thx_lvl/tacc;//*0.5;
		thy_lvl = thy_lvl/tacc;//*0.5;
	}


	if(thx_lvl > 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, thx_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
		pu_level = thx_lvl;
		pd_level = 0;
	}
	else if(thx_lvl < 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, -thx_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
		pd_level = -thx_lvl;
		pu_level = 0;
	}
	else 
	{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
		pu_level = 0;
		pd_level = 0;
	}

	if(thy_lvl > 0)
	{		
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, thy_lvl);
		yr_level = 0;
		yl_level = thy_lvl;
	}
	else if(thy_lvl < 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, -thy_lvl);
		yl_level = 0;
		yr_level = -thy_lvl;
	}
	else 
	{
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
		yr_level = 0;
		yl_level = 0;
	}

	rchud_sw = 3;
}


void f9h_stg1_starboard::DoTWU(void)
{
	VECTOR3 wv = _V(0,0,0);
	GetWeightVector(wv);
	wv = -wv;

	
	double angoa = atan2(-wv.y, wv.z);
	double slipage = atan2(-wv.x, wv.z);
	slipage = -slipage;

	if(angoa > PI)angoa = (PI*2)-angoa;
	else if(angoa < -PI)angoa = (PI*2)+angoa;

	if(slipage > PI)slipage = (PI*2)-slipage;
	else if(slipage < -PI)slipage = (PI*2)+slipage;

	double mass = GetMass();

	double max_rcs = GetThrusterMax(th_rcs[0]);
	double max_torque = 2*max_rcs*21;

	if (rcs_sw == 1)
	{
		max_rcs = GetThrusterMax(th_rcs2[0]);
		max_torque = max_rcs*(18.12+18.8);
	}

	double inertia = 129*mass;

	VECTOR3 avel = _V(0,0,0);
	GetAngularVel(avel);
	double max_acc = max_torque/inertia;
	double max_drot = max_acc*step_time;

	double reqxv = 0.9*(sqrt(abs(angoa*max_acc*2)));
	if (reqxv > vrot_lim)reqxv = vrot_lim;
	if (angoa > 0)reqxv = -reqxv;
	double req_drotx = reqxv - avel.x;
	double thx_lvl = req_drotx/max_drot;
	if((angoa*DEG) < 1 && (angoa*DEG) > -1)
	{
	reqxv = 1*-angoa;
	double acc_reqx = (reqxv-avel.x)/0.1;
	thx_lvl = acc_reqx/max_acc;
	}

	double reqyv = 0.9*(sqrt(abs(slipage*max_acc*2)));
	if (reqyv > vrot_lim)reqyv = vrot_lim;
	if (slipage > 0)reqyv = -reqyv;
	double req_droty = reqyv - avel.y;
	double thy_lvl = req_droty/max_drot;
	if((slipage*DEG) < 1 && (slipage*DEG) > -1)
	{
	reqyv = 1*-slipage;
	double acc_reqy = (reqyv-avel.y)/0.1;
	thy_lvl = acc_reqy/max_acc;
	}

	if(thx_lvl > 1)thx_lvl = 1;
	else if(thx_lvl < -1)thx_lvl = -1;

	if(thy_lvl > 1)thy_lvl = 1;
	else if(thy_lvl < -1)thy_lvl = -1;


	if(thx_lvl > 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, thx_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
		pu_level = thx_lvl;
		pd_level = 0;
	}
	else if(thx_lvl < 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, -thx_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
		pd_level = -thx_lvl;
		pu_level = 0;
	}
	else 
	{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
		pu_level = 0;
		pd_level = 0;
	}

	if(thy_lvl > 0)
	{		
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, thy_lvl);
		yr_level = 0;
		yl_level = thy_lvl;
	}
	else if(thy_lvl < 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, -thy_lvl);
		yl_level = 0;
		yr_level = -thy_lvl;
	}
	else 
	{
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
		yr_level = 0;
		yl_level = 0;
	}


}


void f9h_stg1_starboard::GoToFlightVector()
{

	if(GetAltitude() < 300) vrot_lim = 5*RAD;
	
	DeactivateNavmode(NAVMODE_KILLROT);

	double angoa = atan2(-flight_vector.y, flight_vector.z);
	double slipage = atan2(flight_vector.x, flight_vector.z);

	if(angoa > PI)angoa = (PI*2)-angoa;
	else if(angoa < -PI)angoa = (PI*2)+angoa;

	if(slipage > PI)slipage = (PI*2)-slipage;
	else if(slipage < -PI)slipage = (PI*2)+slipage;

	double mass = GetMass();

	double max_rcs = GetThrusterMax(th_rcs[0]);
	double max_torque = 2*max_rcs*21;

	if (rcs_sw == 1)
	{
		max_rcs = GetThrusterMax(th_rcs2[0]);
		max_torque = max_rcs*(18.12+18.8);
	}

	double inertia = 129*mass;

	double finfact = 1;
	if (fins_deploy_status > 0 && rcs_sw == 1) finfact = 0.05; // 0.5

	
	VECTOR3 avel = _V(0,0,0);
	GetAngularVel(avel);
	double max_acc = max_torque/inertia;
	double max_drot = max_acc*step_time;

	double reqxv = 0.8*(sqrt(abs(angoa*max_acc*2)));
	if (reqxv > vrot_lim)reqxv = vrot_lim;
	if (angoa > 0)reqxv = -reqxv;
	double req_drotx = reqxv - avel.x;
	double thx_lvl = req_drotx/max_drot;
	if((angoa*DEG) < 1 && (angoa*DEG) > -1)
	{
	reqxv = 1*-angoa;
	double acc_reqx = (reqxv-avel.x)/0.2;
	thx_lvl = acc_reqx/max_acc;
	}
	thx_lvl = thx_lvl*finfact;


	double reqyv = 0.8*(sqrt(abs(slipage*max_acc*2)));
	if (reqyv > vrot_lim)reqyv = vrot_lim;
	if (slipage > 0)reqyv = -reqyv;
	double req_droty = reqyv - avel.y;
	double thy_lvl = req_droty/max_drot;
	if((slipage*DEG) < 1 && (slipage*DEG) > -1)
	{
	reqyv = 1*-slipage;
	double acc_reqy = (reqyv-avel.y)/0.2;
	thy_lvl = acc_reqy/max_acc;
	}
	thy_lvl = thy_lvl*finfact;

	if(thx_lvl > 1)thx_lvl = 1;
	else if(thx_lvl < -1)thx_lvl = -1;

	if(thy_lvl > 1)thy_lvl = 1;
	else if(thy_lvl < -1)thy_lvl = -1;

	double tacc = oapiGetTimeAcceleration();
	thx_lvl = thx_lvl/tacc;
	thy_lvl = thy_lvl/tacc;

	if(thx_lvl > 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, thx_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
		pu_level = thx_lvl;
		pd_level = 0;
	}
	else if(thx_lvl < 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, -thx_lvl);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
		pd_level = -thx_lvl;
		pu_level = 0;
	}
	else 
	{
		SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
		SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
		pu_level = 0;
		pd_level = 0;
	}

	if(thy_lvl > 0)
	{		
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, thy_lvl);
		yr_level = 0;
		yl_level = thy_lvl;
	}
	else if(thy_lvl < 0)
	{
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, -thy_lvl);
		yl_level = 0;
		yr_level = -thy_lvl;
	}
	else 
	{
		SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
		SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);
		yr_level = 0;
		yl_level = 0;
	}


}


bool f9h_stg1_starboard::clbkLoadGenericCockpit ()
{
	SetCamDir();
return true;
}

void f9h_stg1_starboard::SetCamDir()
{
	VECTOR3 camdefdir = _V(0,0,1);
	if(cam_status == 1)camdefdir = _V(0,0,-1);
	SetCameraDefaultDirection(camdefdir, 0);
	oapiCameraAttach(GetHandle(), 2);
}


/////////////////////// BOOSTBACKS ///////////////////////

void f9h_stg1_starboard::DoBoostBack (void)
{
if (eng_statusp != 1)
{
eng_statusp = 1;
ConfigMain();
}

	VECTOR3 haspdv = _V(0,0,0);
	GetGroundspeedVector(FRAME_HORIZON,haspdv);

	

	double emp_mass = GetEmptyMass()+GetPropellantMass(hRCSProp);
	double vmass = GetMass();
	double mass_fct = (emp_mass*emp_mass)/(vmass*vmass);
	if (mass_fct < 0.4 )mass_fct = 0.4;
	double tgt_height_offset = 5550+(4020*mass_fct);//-(1.59-sin(2*((90*RAD)-impact_angle))));//9000//5750

	double fpang = atan2(haspdv.x,haspdv.z);
	double sinfpang = sin(fpang);
	double fpfact = -2000*sinfpang;
	tgt_height_offset = tgt_height_offset + fpfact;

	double aoi_fct = 15000*(sin((65*RAD)-impact_angle));
	tgt_height_offset = tgt_height_offset + aoi_fct ;/////////////

	double aoi_fct2 = 70000*(sin(impact_angle - (70*RAD)));
	if(aoi_fct2 < 0)aoi_fct2 = 0;
	tgt_height_offset = tgt_height_offset + aoi_fct2;/////////////
	
	double ref_fct = ((vmass - emp_mass)/10000) + 0.8;
	if (ref_fct > 3)ref_fct = 3;
	else if (ref_fct < 1)ref_fct = 1;
	tgt_height_offset = tgt_height_offset*ref_fct;////*2.9
	
	//sprintf(oapiDebugString(), "tho %0.1f impa %0.1f", tgt_height_offset, impact_angle*DEG);



	OBJHANDLE hearth=GetSurfaceRef();
	OBJHANDLE htgt_vessel = oapiGetObjectByName(prefname);
	if (htgt_vessel == NULL)htgt_vessel = oapiGetBaseByName(hearth,prefname);
	if (htgt_vessel == NULL)
	{
		boostback_sw = 0;
		return;
	}
	DeactivateNavmode(NAVMODE_KILLROT);
	

		VECTOR3 relvel = _V(0,0,0);
		GetRelativeVel(hearth,relvel);
		

	double ves_lng = 0;
	double ves_lat = 0;
	double ves_rad = 0;
	if (oapiGetObjectType(htgt_vessel) == OBJTP_VESSEL)
	{
	oapiGetEquPos(htgt_vessel, &ves_lng, &ves_lat, &ves_rad);
	
	}

	else
	{
		oapiGetBaseEquPos(htgt_vessel, &ves_lng, &ves_lat, &ves_rad);
		
	}
	double lng_off = PI2*(timetoimp+0.0)/86164;


	double tgt_lng = ves_lng+lng_off;
	double tgt_lat = ves_lat;
	if (tgt_lng > PI2) tgt_lng = tgt_lng - PI2;
	double tgt_rad = ves_rad + tgt_height_offset;

	

	VECTOR3 tgt_pos_global = _V(0,0,0);
	oapiEquToGlobal(hearth, tgt_lng, tgt_lat, tgt_rad, &tgt_pos_global);

	

	VECTOR3 earth_pos = _V(0,0,0);
	oapiGetGlobalPos(hearth, &earth_pos);

	VECTOR3 tgt_vector_rele = tgt_pos_global - earth_pos;
	
	VECTOR3 my_vector_rele = _V(0,0,0);
	GetRelativePos(hearth, my_vector_rele);

	VECTOR3 tgt_dist = my_vector_rele - tgt_vector_rele;
	double r1 = length(my_vector_rele);
	double r2 = length(tgt_vector_rele);
	double d = length(tgt_dist);
	double s = 0.5*(r1 + r2 + d);
	double GM = GGRAV*oapiGetMass(hearth);
	double h = (r1-r2);
	

	double v2 = (2*GM*(s-r1))/(r1*s);
	double v = sqrt(v2);


	double tantheta = sqrt(((s-d)*(d-h))/(s*(d+h)));
	double theta = atan(tantheta);


	double vx = v*cos(theta);
	double vy = v*sin(theta);

		double grdspd = sqrt((length(relvel)*length(relvel))-(haspdv.y*haspdv.y));
		double rds = (rt + GetAltitude());
		double ang_vel = grdspd/(1*rds);//(orb_grnd_spd+vx)/(2*rds);
		double acc_cent_fact = 1;
		double acc_cent = ang_vel*ang_vel*rds*acc_cent_fact;
		double gravt = GM/(rds*rds);
		double g = acc_cent - gravt;


	double tti = (sqrt((s*d*(s-d))/(2*GM)))*(1 + ((s/(sqrt(d*(s-d)))) *(atan(sqrt(d/(s-d))))));
	timetoimp = tti;


	normalise(tgt_vector_rele);
	normalise(my_vector_rele);

	dotprod = dotp(tgt_vector_rele,my_vector_rele);
	if (dotprod > 1)dotprod = 1;
	else if (dotprod < -1)dotprod = -1;

	double tgt_angle = acos(dotprod);

	VECTOR3 normal_me_tgt_vects = _V(0,0,0);
	normal_me_tgt_vects = crossp(tgt_vector_rele,my_vector_rele);
	normalise(normal_me_tgt_vects);

	VECTOR3 tangential_vector = _V(0,0,0);
	tangential_vector = crossp(my_vector_rele, normal_me_tgt_vects);
	normalise(tangential_vector);

	double tgt_range = 500 + (tgt_angle * (rt+((GetAltitude()+tgt_height_offset)/2)));

	VECTOR3 vertvel_glob = _V((vy*my_vector_rele.x),(vy*my_vector_rele.y),(vy*my_vector_rele.z));
	
	


	double final_vel_xz = tgt_range/timetoimp;
	VECTOR3 final_vel_xz_glob = _V(0,0,0);
	final_vel_xz_glob.x = tangential_vector.x*vx;
	final_vel_xz_glob.y = tangential_vector.y*vx;
	final_vel_xz_glob.z = tangential_vector.z*vx;

	

	VECTOR3 final_vel_vect = final_vel_xz_glob+vertvel_glob;

	double fvvh = GetAltitude();//-tgt_height_offset;
	double finalvv = length(vertvel_glob)+sqrt(2*-g*fvvh);
	double finalvh = length(final_vel_xz_glob);
	impact_angle = atan2(finalvv,finalvh);
	//impact_angle = PI05-theta;
	
	

	VECTOR3 req_dv_vector = _V(0,0,0);
	req_dv_vector.x = final_vel_vect.x - relvel.x;
	req_dv_vector.y = final_vel_vect.y - relvel.y;
	req_dv_vector.z = final_vel_vect.z - relvel.z;

	double dvmain = length(req_dv_vector);
	double fraction = exp(dvmain/GetThrusterIsp(th_main[eng_statusp]));
	double fuelmass = vmass - (vmass/fraction);
	double englevel = 1;//GetThrusterLevel(th_main[eng_statusp]);
	double fuelrate = GetThrusterMax(th_main[eng_statusp])*englevel/GetThrusterIsp(th_main[eng_statusp]);
	double dvtime = fuelmass/fuelrate;
	double vvel_off = 0.75*dvtime*-g;
	if (vvel_off < 0) vvel_off = 0;

	vertvel_glob = _V((vvel_off*my_vector_rele.x),(vvel_off*my_vector_rele.y),(vvel_off*my_vector_rele.z));

	req_dv_vector.x = req_dv_vector.x + vertvel_glob.x;
	req_dv_vector.y = req_dv_vector.y + vertvel_glob.y;
	req_dv_vector.z = req_dv_vector.z + vertvel_glob.z;


	VECTOR3 mypos_glob = _V(0,0,0);
	GetGlobalPos(mypos_glob);

	
	req_dv_vector = req_dv_vector + mypos_glob;

	VECTOR3 req_dv_vector_loc = _V(0,0,0);
	Global2Local(req_dv_vector,req_dv_vector_loc);

	/////////////

	fb_dv_req = length(req_dv_vector_loc);

	flight_vector = req_dv_vector_loc;
	GoToFlightVector();

	VECTOR3 req_dv_vector_norm = req_dv_vector_loc;
	normalise(req_dv_vector_norm);
	VECTOR3 curr_att_loc = _V(0,0,1);

	dotprod = dotp(curr_att_loc,req_dv_vector_norm);
	if (dotprod > 1)dotprod = 1;
	else if (dotprod < -1)dotprod = -1;

	double pointing_error = acos(dotprod);


	double max_dv = cos(pointing_error)*step_time*(GetThrusterMax(th_main[eng_statusp])/vmass);
	double eng_lvl = flight_vector.z/max_dv;
	if (eng_lvl > 1)eng_lvl = 1;
	else if (eng_lvl < 0)eng_lvl = 0;

	double pe_fact = abs(fb_dv_req*tan(pointing_error));


	if ((pe_fact) < 25 && flight_vector.z > 4)
	{
		SetThrusterGroupLevel(THGROUP_MAIN, eng_lvl);
		mth_level = eng_lvl;
		rchud_sw = 2; // flyback burn active
	}
	else if (abs(pointing_error*DEG) < 90)
	{
	eng_statusp = 2;
	ConfigMain();
	SetThrusterGroupLevel(THGROUP_MAIN, 0.4);
	mth_level = 0.4;
	rchud_sw = 2; // flyback burn active
	}
	else 
	{	
	SetThrusterGroupLevel(THGROUP_MAIN, 0);
	mth_level = 0;
	rchud_sw = 1; // flyback burn enabled
	}

	if (abs(flight_vector.z) < 4 && length(flight_vector) < 8)
	{
	KillAllRCS();
	SetThrusterGroupLevel(THGROUP_MAIN,0);
	mth_level = 0;
	if (afs_status == 2)afs_status = 3;
	//ActivateNavmode(NAVMODE_KILLROT);
	boostback_sw = 0;
	}
	

}

void f9h_stg1_starboard::DoBoostBack2 (void)
{
if (boostback_sw == 0) return;

if (eng_statusp != 1)
{
eng_statusp = 1;
ConfigMain();
}

	
	OBJHANDLE hearth=GetSurfaceRef();
	OBJHANDLE htgt_vessel = oapiGetObjectByName(prefname);
	if (htgt_vessel == NULL)htgt_vessel = oapiGetBaseByName(hearth,prefname);
	if (htgt_vessel == NULL)
	{
		boostback_sw = 0;
		return;
	}
	DeactivateNavmode(NAVMODE_KILLROT);
	

	VECTOR3 haspdv = _V(0,0,0);
	GetGroundspeedVector(FRAME_HORIZON,haspdv);
	double GM = GGRAV*oapiGetMass(hearth);

	if (haspdv.y > 0)
		{
		boostback_sw = 2;
		return;
		}

	double emp_mass = GetEmptyMass()+GetPropellantMass(hRCSProp);
	double vmass = GetMass();
	double mass_fct = (emp_mass*emp_mass)/(vmass*vmass);
	if (mass_fct < 0.4 )mass_fct = 0.4;
	double tgt_height_offset = 5550+(4020*mass_fct);//-(1.59-sin(2*((90*RAD)-impact_angle))));//9000 //5750

	double fpang = atan2(haspdv.x,haspdv.z);
	double sinfpang = sin(fpang);
	double fpfact = -2000*sinfpang;
	tgt_height_offset = tgt_height_offset + fpfact;

	double aoi_fct = 15000*(sin((65*RAD)-impact_angle));
	tgt_height_offset = tgt_height_offset + aoi_fct ;/////////////
	
	double ref_fct = ((vmass - emp_mass)/7500) + 0.8;//10000
	if (ref_fct > 3)ref_fct = 3;
	else if (ref_fct < 1)ref_fct = 1;
	tgt_height_offset = tgt_height_offset*ref_fct;////*2.9

	//centrifugal acc
		VECTOR3 relvel = _V(0,0,0);
		GetRelativeVel(hearth,relvel);
		double orb_vel = length(relvel);
		double orb_grnd_spd = sqrt((orb_vel*orb_vel)-(haspdv.y*haspdv.y));
		double rds = (rt + GetAltitude());
		double ang_vel = orb_grnd_spd/rds;
		double acc_cent_fact = 1;
		double acc_cent = ang_vel*ang_vel*rds*acc_cent_fact;
		double gravt = GM/(rds*rds);
		double g = acc_cent - gravt;

	double aa = g/2;
	double bb = haspdv.y;
	double cc = GetAltitude()-tgt_height_offset;

	double timetoimp = (-bb-(sqrt((bb*bb)-(4*aa*cc))))/(2*aa);

	double ves_lng = 0;
	double ves_lat = 0;
	double ves_rad = 0;
	if (oapiGetObjectType(htgt_vessel) == OBJTP_VESSEL)
	{
		oapiGetEquPos(htgt_vessel, &ves_lng, &ves_lat, &ves_rad);
	}
	
	else
	{
		oapiGetBaseEquPos(htgt_vessel, &ves_lng, &ves_lat, &ves_rad);
	}
	double lng_off = PI2*timetoimp/86164;

	double tgt_lng = ves_lng+lng_off;
	double tgt_lat = ves_lat;
	if (tgt_lng > PI2) tgt_lng = tgt_lng - PI2;
	double tgt_rad = ves_rad + tgt_height_offset;

	VECTOR3 tgt_pos_global = _V(0,0,0);
	oapiEquToGlobal(hearth, tgt_lng, tgt_lat, tgt_rad, &tgt_pos_global);

	VECTOR3 earth_pos = _V(0,0,0);
	oapiGetGlobalPos(hearth, &earth_pos);


	VECTOR3 tgt_vector_rele = tgt_pos_global - earth_pos;
	normalise(tgt_vector_rele);

	VECTOR3 my_vector_rele = _V(0,0,0);
	GetRelativePos(hearth, my_vector_rele);
	normalise(my_vector_rele);

	dotprod = dotp(tgt_vector_rele,my_vector_rele);
	if (dotprod > 1)dotprod = 1;
	else if (dotprod < -1)dotprod = -1;
	
	double tgt_angle = acos(dotprod);

	VECTOR3 normal_me_tgt_vects = _V(0,0,0);
	normal_me_tgt_vects = crossp(tgt_vector_rele,my_vector_rele);
	normalise(normal_me_tgt_vects);

	VECTOR3 tangential_vector = _V(0,0,0);
	tangential_vector = crossp(my_vector_rele, normal_me_tgt_vects);
	normalise(tangential_vector);

	double tgt_range = tgt_angle * (rt+((GetAltitude()+tgt_height_offset)/2));

	VECTOR3 vertvel_glob = _V((bb*my_vector_rele.x),(bb*my_vector_rele.y),(bb*my_vector_rele.z));
	

	double final_vel_xz = tgt_range/timetoimp;
	VECTOR3 final_vel_xz_glob = _V(0,0,0);
	final_vel_xz_glob.x = tangential_vector.x*final_vel_xz;
	final_vel_xz_glob.y = tangential_vector.y*final_vel_xz;
	final_vel_xz_glob.z = tangential_vector.z*final_vel_xz;

	

	VECTOR3 final_vel_vect = final_vel_xz_glob+vertvel_glob;

	double fvvh = GetAltitude()-tgt_height_offset;
	double finalvv = length(vertvel_glob)+sqrt(2*-g*fvvh);
	double finalvh = length(final_vel_xz_glob);
	impact_angle2 = atan2(finalvv,finalvh);
	//sprintf(oapiDebugString(), "impang2 %0.1f", impact_angle2*DEG);


	VECTOR3 req_dv_vector = _V(0,0,0);
	req_dv_vector.x = final_vel_vect.x - relvel.x;
	req_dv_vector.y = final_vel_vect.y - relvel.y;
	req_dv_vector.z = final_vel_vect.z - relvel.z;

	VECTOR3 mypos_glob = _V(0,0,0);
	GetGlobalPos(mypos_glob);

	req_dv_vector = req_dv_vector + mypos_glob;

	VECTOR3 req_dv_vector_loc = _V(0,0,0);
	Global2Local(req_dv_vector,req_dv_vector_loc);


	if (length(req_dv_vector_loc) > fb_dv_req)
	{	
		boostback_sw = 2;
		return;
	}
	else fb_dv_req = length(req_dv_vector_loc);

	flight_vector = req_dv_vector_loc;
	GoToFlightVector();

	VECTOR3 req_dv_vector_norm = req_dv_vector_loc;
	normalise(req_dv_vector_norm);
	VECTOR3 curr_att_loc = _V(0,0,1);

	double pointing_error = acos(dotp(curr_att_loc,req_dv_vector_norm));


	double max_dv = step_time*(GetThrusterMax(th_main[eng_statusp])/vmass);
	double eng_lvl = flight_vector.z/max_dv;
	if (eng_lvl > 1)eng_lvl = 1;
	else if (eng_lvl < 0)eng_lvl = 0;

	double pe_fact = abs(fb_dv_req*tan(pointing_error));

	if (pe_fact < 25 && flight_vector.z > 4)
	{
		SetThrusterGroupLevel(THGROUP_MAIN, eng_lvl);
		mth_level = eng_lvl;
		rchud_sw = 2;
	}
	else 
	{
	SetThrusterGroupLevel(THGROUP_MAIN, 0);
	mth_level = 0;
	rchud_sw = 1;
	}

	if (abs(flight_vector.z) < 4 && length(flight_vector) < 8)
	{
	KillAllRCS();
	SetThrusterGroupLevel(THGROUP_MAIN,0);
	mth_level = 0;
	if (afs_status == 2)afs_status = 3;
	//ActivateNavmode(NAVMODE_KILLROT);
	boostback_sw = 0;
	}
}

void f9h_stg1_starboard::DoCorrection (void)
{
if (mom_sw == 1)/// 3 eng
{
if (eng_statusp != 1)
	{
	eng_statusp = 1;
	ConfigMain();
	}
}
else if (edlo_sw == 0)
{
if (eng_statusp != 2)
	{
	eng_statusp = 2;
	ConfigMain();
	}
}
else
{
if (eng_statusp != 1)
	{
	eng_statusp = 1;
	ConfigMain();
	}
}

OBJHANDLE htgt_vessel = oapiGetObjectByName(prefname);
if (htgt_vessel == NULL)htgt_vessel = oapiGetBaseByName(GetGravityRef(),prefname);
if (htgt_vessel == NULL) 
{
	//sprintf(oapiDebugString(), "here3  cs %d", correct_sw);
	correct_sw = 1;
	return;
}

double emp_mass = GetEmptyMass()+GetPropellantMass(hRCSProp);
double vmass = GetMass();
double tgt_height_offset = 100;//250

VECTOR3 tgt_glob_pos = _V(0,0,0);
oapiGetGlobalPos(htgt_vessel, &tgt_glob_pos);

VECTOR3 tgt_loc_pos = _V(0,0,0);
Global2Local(tgt_glob_pos,tgt_loc_pos);

VECTOR3 tgt_hrz_pos = _V(0,0,0);
HorizonRot(tgt_loc_pos,tgt_hrz_pos);

VECTOR3 haspdv = _V(0,0,0);
GetGroundspeedVector (FRAME_HORIZON,haspdv);

double alti = GetAltitude();


	OBJHANDLE hearth=GetSurfaceRef();
	double rds = rt+alti;
    double GM = GGRAV*oapiGetMass(hearth);
	double gravt = GM/(rds*rds);
	double g = -gravt;
	double aa = g/2;
	double bb = haspdv.y;
	double cc = alti-tgt_height_offset;
	double timetoimp = (-bb-(sqrt((bb*bb)-(4*aa*cc))))/(2*aa);


	double k_factor = (30000 - alti)/20000; // 0 -1
	if (k_factor > 1)k_factor = 1;
	k_factor = 1.85 - (1.0*k_factor);
	double k = 75*9.81/k_factor;//120
	//double k = 75*9.81/1.3;
	 double gsvx = sqrt((haspdv.x*haspdv.x)+(haspdv.z*haspdv.z));
	 double expo = -k*timetoimp/vmass;

	 double  range = vmass*gsvx/k*(1-(pow(M_E, expo)));

	 double grdspd_az = atan2(haspdv.x, haspdv.z);
	 double tgt_az = atan2(tgt_hrz_pos.x,tgt_hrz_pos.z);
	 double tgt_angle = grdspd_az - tgt_az;

	 if (tgt_angle > PI)tgt_angle = tgt_angle-PI2;
	 if (tgt_angle < -PI)tgt_angle = tgt_angle+PI2;

	double tgt_range = sqrt((tgt_hrz_pos.x*tgt_hrz_pos.x)+(tgt_hrz_pos.z*tgt_hrz_pos.z));
	tgt_range = tgt_range*cos(tgt_angle);


	//sprintf(oapiDebugString(), "rng %0.1f tgtrng %0.1f tgtang %0.1f", range, tgt_range, tgt_angle*DEG);
	if (range < tgt_range)
	{
		correct_sw = 1;
		rot_sw = 0;
		return;
	}
	//sprintf(oapiDebugString(), "rng %0.1f tgtrng %0.1f correct_sw %0.1f", range, tgt_range, correct_sw);


	VECTOR3 req_fl_vec_hrz =  haspdv;

	if(abs(tgt_angle) < (1*RAD)) rot_sw = 1;
	else if(abs(tgt_angle) > (80*RAD)) rot_sw = 1;

	if (rot_sw == 0)
	{
	VECTOR3 norm_flt_pln_hrz = crossp(_V(0,1,0),haspdv);
	VECTOR3 rot_vec = crossp(norm_flt_pln_hrz,haspdv);
	normalise(rot_vec);

	/////// more efficient if no correction!
	double rot_angle = 0 * -tgt_angle;

	double ct = cos(rot_angle);
	double st = sin(rot_angle);

	double ux = rot_vec.x;
	double uy = rot_vec.y;
	double uz = rot_vec.z;
	double ux2 = ux*ux;
	double uy2 = uy*uy;
	double uz2 = uz*uz;

	MATRIX3 RG;

	RG.m11 = ux2+(1-ux2)*ct;
	RG.m12 = ux*uy*(1-ct)-uz*st;
	RG.m13 = ux*uz*(1-ct)+uy*st;
	RG.m21 = ux*uy*(1-ct)+uz*st;
	RG.m22 = uy2+(1-uy2)*ct;
	RG.m23 = uy*uz*(1-ct)-ux*st;
	RG.m31 = ux*uz*(1-ct)-uy*st;
	RG.m32 = uy*uz*(1-ct)+ux*st;
	RG.m33 = uz2+(1-uz2)*ct;

	req_fl_vec_hrz = mul(RG, haspdv);
	}


	VECTOR3 req_fv_loc = _V(0,0,0);
	HorizonInvRot(-req_fl_vec_hrz,req_fv_loc);
	flight_vector = req_fv_loc;
	GoToFlightVector();

	VECTOR3 req_fv_loc_norm = req_fv_loc;
	normalise(req_fv_loc_norm);
	VECTOR3 curr_att_loc = _V(0,0,1);

	dotprod = dotp(curr_att_loc,req_fv_loc_norm);
	if (dotprod > 1)dotprod = 1;
	else if (dotprod < -1)dotprod = -1;

	double pointing_error = acos(dotprod);

	//sprintf(oapiDebugString(), "pt err %0.1f rng %0.1f tgtrng %0.1f rot_sw %d", pointing_error*DEG, range, tgt_range, rot_sw);

	if (pointing_error < (10*RAD) && range > tgt_range)
	{
	SetThrusterLevel(th_main[eng_statusp],1);
	if ((edlo_sw+mom_sw) > 0)SetThrusterLevel(th_main[eng_statusp],0.7);
	}
	else 
	{
	SetThrusterLevel(th_main[eng_statusp],0);
	}

	//// hud info
	rchud_sw = 4;
	if (swt == 1)rchud_sw = 5;
	
}

void f9h_stg1_starboard::DoReentryCorrection (void)
{

OBJHANDLE htgt_vessel = oapiGetObjectByName(prefname);
if (htgt_vessel == NULL)htgt_vessel = oapiGetBaseByName(GetGravityRef(),prefname);
if (htgt_vessel == NULL) return;

if (mom_sw == 1)/// 3 eng
{
if (eng_statusp != 1)
	{
	eng_statusp = 1;
	ConfigMain();
	}
}

else if (edlo_sw == 0)/// 1 eng
{
if (eng_statusp != 2)
	{
	eng_statusp = 2;
	ConfigMain();
	}
}
else/// 3 eng
{
if (eng_statusp != 1)
	{
	eng_statusp = 1;
	ConfigMain();
	}
}

double alti = GetAltitude();
double emp_mass = GetEmptyMass()+GetPropellantMass(hRCSProp);
double vmass = GetMass();
double current_fuel_mass = vmass - emp_mass;
double tgt_height_offset = 700 + (edlo_sw * 800);

VECTOR3 tgt_glob_pos = _V(0,0,0);
oapiGetGlobalPos(htgt_vessel, &tgt_glob_pos);

VECTOR3 tgt_loc_pos = _V(0,0,0);
Global2Local(tgt_glob_pos,tgt_loc_pos);

VECTOR3 tgt_hrz_pos = _V(0,0,0);
HorizonRot(tgt_loc_pos,tgt_hrz_pos);

VECTOR3 haspdv = _V(0,0,0);
GetGroundspeedVector(FRAME_HORIZON,haspdv);

double momentum = length(haspdv)*(vmass-emp_mass);
if (mom_sw == 0)
{
if (momentum > (1000*10000)) //1500*20000
	{
	mom_sw = 1;
	if (eng_statusp != 1)
		{
		eng_statusp = 1;
		ConfigMain();
		}
	}
}

double gsvx = sqrt((haspdv.x*haspdv.x)+(haspdv.z*haspdv.z));

//double alti = GetAltitude();
OBJHANDLE hearth=GetSurfaceRef();

VECTOR3 relvelhz = _V(0,0,0);
GetRelativeVel(hearth, relvelhz);

double rvh = length(relvelhz);
rvh = sqrt((rvh*rvh) - (haspdv.y*haspdv.y));
double acc_cfg = rvh*rvh/(alti+rt); // centrifugal acc

	double rds = rt+alti;
    double GM = GGRAV*oapiGetMass(hearth);
	double gravt = GM/(rds*rds);
	gravt = gravt - acc_cfg;
	double g = -gravt;
	double aa = g/2;
	//double aaa = -g;
	double bb = haspdv.y;
	double cc = alti-tgt_height_offset;
	double timetoimp = (-bb-(sqrt((bb*bb)-(4*aa*cc))))/(2*aa);
	double timetoimp2 = (-bb-(sqrt((bb*bb)-(4*aa*(cc-25000)))))/(2*aa);
	if (timetoimp2 < 0)timetoimp2 = 0;
	else if (((bb*bb)-(4*aa*(cc-25000))) < 0)timetoimp2 = 0;

	 double range1 = gsvx*timetoimp2;  // above 25km
	 
	 double k = 75*9.81/2; // 1.3
	 double expo = -k*(timetoimp-timetoimp2)/vmass;
	 double range2 = vmass*gsvx/k*(1-(pow(M_E, expo))); //below 25km
	 double range = (range1+range2);


	 double grdspd_az = atan2(haspdv.x, haspdv.z);
	 double tgt_az = atan2(tgt_hrz_pos.x,tgt_hrz_pos.z);
	 double tgt_angle = grdspd_az - tgt_az;

	 if (tgt_angle > PI)tgt_angle = tgt_angle-PI2;
	 if (tgt_angle < -PI)tgt_angle = tgt_angle+PI2;

	double tgt_range = 250 + sqrt((tgt_hrz_pos.x*tgt_hrz_pos.x)+(tgt_hrz_pos.z*tgt_hrz_pos.z));

	// get pitch
	VECTOR3 z_axis_ves = _V(0,0,1);
	VECTOR3 z_axis_hrz = _V(0,0,1);
	HorizonRot(z_axis_ves, z_axis_hrz);
	normalise(z_axis_hrz);
	double ves_pitch = PI05 - acos(dotp(z_axis_hrz, _V(0,1,0)));

	double cosp = cos(ves_pitch);//cos(GetPitch());
	double sinp = sin(ves_pitch);//sin(GetPitch());
	//sprintf(oapiDebugString(), "req_rdv %0.4f cosp %0.4f", req_rdv, cosp);

	double th_isp = GetThrusterIsp(th_main[eng_statusp]);
	double th_max = GetThrusterMax(th_main[eng_statusp]);
	if ((edlo_sw+mom_sw) > 0)th_max = th_max*0.7; // limit 3 eng to 70%

	double renglvl = 0;

	//if(swt == 0)// ready to start burn
	{
	double fuelmass = current_fuel_mass-3500;//3800; //4000
	double avail_dv = th_isp*log(vmass/(vmass-fuelmass));
	//double gspeed = length(haspdv);
	//if(avail_dv > gspeed) avail_dv = gspeed;
	double fuelrate = th_max/th_isp;
	double burntime = fuelmass/fuelrate;
	

	////////// dv check
	VECTOR3 finv = haspdv + _V(0,(burntime * g),0);
	if(avail_dv > length(finv))
	{
	double dvmain = length(finv);
	double fraction = exp(dvmain/GetThrusterIsp(th_main[eng_statusp]));
	fuelmass = vmass - (vmass/fraction);
	burntime = fuelmass/fuelrate;
	finv = haspdv + _V(0,(burntime * g),0); 
	
	// 2nd iteration
	dvmain = length(finv);
	fraction = exp(dvmain/GetThrusterIsp(th_main[eng_statusp]));
	fuelmass = vmass - (vmass/fraction);
	burntime = fuelmass/fuelrate;
	finv = haspdv + _V(0,(burntime * g),0);
	avail_dv = length(finv);
	dvmain = length(finv);
	fraction = exp(dvmain/GetThrusterIsp(th_main[eng_statusp]));
	fuelmass = vmass - (vmass/fraction);
	burntime = fuelmass/fuelrate;
	}

	double burntime_frac = burntime*0.05;
	double fuelmass_frac = fuelmass*0.05;
	double avail_dv_frac = th_isp*log(vmass/(vmass-fuelmass_frac));
	double av_gravt = 9.81;//0.5*(9.81 + gravt);
	double cos_hdg_err = abs(cos(1.8*tgt_angle));//2
	double gsvx_current = gsvx;
	double v_vel_current = haspdv.y;
	double cosp_current = cosp;
	double sinp_current = sinp;
	double final_vel_h = gsvx_current - (cosp_current*avail_dv_frac*cos_hdg_err);
	double final_vel_v = v_vel_current + (sinp_current*avail_dv_frac)-(av_gravt*burntime_frac);
	double final_ves_pitch = atan2(-final_vel_v,final_vel_h);
	

	//double av_vel_h = (gsvx + final_vel_h)*0.5;
	//double av_vel_v = (haspdv.y + final_vel_v)*0.5;

	double acc_h = (final_vel_h - gsvx_current)/burntime_frac;
	double acc_v = (final_vel_v - v_vel_current)/burntime_frac;

	double final_tgt_range = tgt_range;// - ((burntime*gsvx) + (acc_h*burntime*burntime*0.5));
	double final_ves_alt = alti;// + ((burntime*haspdv.y) + (acc_v*burntime*burntime*0.5));
	
	int i;
	for (i = 0; i < 20; i++)
	{
		avail_dv_frac = th_isp*log((vmass-(fuelmass_frac*i))/(vmass-(fuelmass_frac*(i+1))));
		final_vel_h = gsvx_current - (cosp_current*avail_dv_frac*cos_hdg_err);
		final_vel_v = v_vel_current + (sinp_current*avail_dv_frac)-(av_gravt*burntime_frac);
		final_ves_pitch = atan2(-final_vel_v,final_vel_h);

		acc_h = (final_vel_h - gsvx_current)/burntime_frac;
		acc_v = (final_vel_v - v_vel_current)/burntime_frac;
		final_tgt_range = final_tgt_range - ((burntime_frac*gsvx_current) + (acc_h*burntime_frac*burntime_frac*0.5));
		final_ves_alt = final_ves_alt + ((burntime_frac*v_vel_current) + (acc_v*burntime_frac*burntime_frac*0.5));

		gsvx_current = final_vel_h;
		v_vel_current = final_vel_v;
		cosp_current = cos(final_ves_pitch);
		sinp_current = sin(final_ves_pitch);
	}

	bb = final_vel_v;
	cc = final_ves_alt - tgt_height_offset;
	timetoimp = (-bb-(sqrt((bb*bb)-(4*aa*cc))))/(2*aa);
	timetoimp2 = (-bb-(sqrt((bb*bb)-(4*aa*(cc-25000)))))/(2*aa);
	if (timetoimp2 < 0)timetoimp2 = 0;
	else if (((bb*bb)-(4*aa*(cc-25000))) < 0)timetoimp2 = 0;

	range1 = final_vel_h*timetoimp2;  // above 25km
	 
	k = 75*9.81/2; // 1.3
	expo = -k*(timetoimp-timetoimp2)/(emp_mass+3700); //4000
	range2 = (emp_mass+3700)*final_vel_h/k*(1-(pow(M_E, expo)));//4000 //below 25km
	double final_range = (range1+range2);

	VECTOR3 gspd_vect_loc = _V(0,0,1);
	GetGroundspeedVector(FRAME_LOCAL,gspd_vect_loc);
	normalise(gspd_vect_loc);
	double ptng_err = acos(dotp(gspd_vect_loc, _V(0,0,-1)));

	if(ptng_err < 5*RAD)
		{
		if(final_range < final_tgt_range)reentry_correct_sw = 1;
		if(final_range > (final_tgt_range) && reentry_correct_sw == 1)swt = 1;
		if(edlo_sw == 0)
			{
			if(final_ves_alt < (20000))swt = 1;
			if(final_tgt_range < (2000)) swt = 1;
			}
		}

	////sprintf(oapiDebugString(), "avail_dv %0.1f burntime %0.1f av_ves_pitch %0.1f final_tgt_range %0.1f final_range %0.1f range %0.1f tgt_range %0.1f g %0.1f final_ves_alt %0.1f cos_hdg_err %0.1f swt %d ptng_err %0.1f", avail_dv, burntime,av_ves_pitch*DEG, final_tgt_range, final_range,range, tgt_range, g, final_ves_alt, cos_hdg_err, swt, DEG*ptng_err);
	//sprintf(oapiDebugString(), "avail_dv %0.1f burntime %0.1f final_tgt_range %0.1f final_range %0.1f range %0.1f tgt_range %0.1f g %0.1f final_ves_alt %0.1f", avail_dv, burntime, final_tgt_range, final_range,range, tgt_range, g, final_ves_alt);
	}


	renglvl = swt;
	if((edlo_sw+swt) == 2)renglvl = 0.7; // 3 eng throttle70%
	else if((mom_sw+swt) == 2)renglvl = 0.7; // 3 eng throttle70%

	hud_flight_range = range;
	hud_range = tgt_range;

	if (range < tgt_range)// burn end
	{
		SetThrusterLevel(th_main[eng_statusp],0);
		mth_level = 0;
		rc_sw = 1;
		correct_sw = 1;
		return;
	}
	else if (GetPropellantMass(hProp) < 3500)// burn end
	{
		SetThrusterLevel(th_main[eng_statusp],0);
		mth_level = 0;
		rc_sw = 1;
		correct_sw = 1;
		return;
	}
	

	VECTOR3 req_fl_vec_hrz =  haspdv;
	int rot_sw2 = 0;
	if(abs(tgt_angle) < (0.05*RAD)) rot_sw2 = 1;
	else if(abs(tgt_angle) > (90*RAD)) rot_sw2 = 1;
	if (alti < 35000)rot_sw2 = 1;
	if(swt == 0)rot_sw2 = 1;

	if (rot_sw2 == 0)
	{
	VECTOR3 norm_flt_pln_hrz = crossp(_V(0,1,0),haspdv);
	normalise(norm_flt_pln_hrz);
	VECTOR3 rot_vec = crossp(norm_flt_pln_hrz,haspdv);
	normalise(rot_vec);
	//////////
	//rot_vec = _V(0,-1,0);

	double rot_angle = -5 * tgt_angle; //10

	double ct = cos(rot_angle);
	double st = sin(rot_angle);

	double ux = rot_vec.x;
	double uy = rot_vec.y;
	double uz = rot_vec.z;
	double ux2 = ux*ux;
	double uy2 = uy*uy;
	double uz2 = uz*uz;

	MATRIX3 RG;

	RG.m11 = ux2+(1-ux2)*ct;
	RG.m12 = ux*uy*(1-ct)-uz*st;
	RG.m13 = ux*uz*(1-ct)+uy*st;
	RG.m21 = ux*uy*(1-ct)+uz*st;
	RG.m22 = uy2+(1-uy2)*ct;
	RG.m23 = uy*uz*(1-ct)-ux*st;
	RG.m31 = ux*uz*(1-ct)-uy*st;
	RG.m32 = uy*uz*(1-ct)+ux*st;
	RG.m33 = uz2+(1-uz2)*ct;

	req_fl_vec_hrz = mul(RG, haspdv);
	}


	VECTOR3 req_fv_loc = _V(0,0,0);
	HorizonInvRot(-req_fl_vec_hrz,req_fv_loc);
	flight_vector = req_fv_loc;
	GoToFlightVector();

	VECTOR3 req_fv_loc_norm = req_fv_loc;
	normalise(req_fv_loc_norm);
	VECTOR3 curr_att_loc = _V(0,0,1);

	dotprod = dotp(curr_att_loc,req_fv_loc_norm);
	if (dotprod > 1)dotprod = 1;
	else if (dotprod < -1)dotprod = -1;

	double pointing_error = acos(dotprod);

	if (pointing_error < (80*RAD) && range > tgt_range)
	{
	SetThrusterLevel(th_main[eng_statusp],renglvl);
	mth_level = renglvl;
	}
	else 
	{
	SetThrusterLevel(th_main[eng_statusp],0);
	mth_level = 0;
	}
	rat_sw = 0;

	//// hud info
	rchud_sw = 4;
	if (swt == 1)rchud_sw = 5;
}


void f9h_stg1_starboard::DoLAT (void)
{
	double machnum = GetMachNumber();
OBJHANDLE htgt_vessel = oapiGetObjectByName(prefname);
if (htgt_vessel == NULL)htgt_vessel = oapiGetBaseByName(GetGravityRef(),prefname);
if (htgt_vessel == NULL) 
{
	DoRAT();
	return;
}
if (machnum > 2)
{
	DoRAT();
	return;
}

double lon,lat,rad = 0;
oapiGetEquPos(GetHandle(), &lon, &lat, &rad);
double windspeed = 0;
VECTOR3 windvector = oapiGetWindVector(GetSurfaceRef(),lon, lat, (rad-6371010), 0, &windspeed );
if (length(windvector) == 0) windvector = _V(0,0.00001,0);

double alti = GetAltitude(ALTMODE_GROUND);

double vmass = GetMass();
double tgt_height_offset = 50+(2*alti/3);

VECTOR3 tgt_glob_pos = _V(0,0,0);
oapiGetGlobalPos(htgt_vessel, &tgt_glob_pos);

VECTOR3 tgt_loc_pos = _V(0,0,0);
Global2Local(tgt_glob_pos,tgt_loc_pos);

VECTOR3 tgt_hrz_pos = _V(0,0,0);
HorizonRot(tgt_loc_pos,tgt_hrz_pos);

VECTOR3 haspdv = _V(0,0,0);
GetGroundspeedVector (FRAME_HORIZON,haspdv);


	 double gsvx = sqrt((haspdv.x*haspdv.x)+(haspdv.z*haspdv.z));
	

	VECTOR3 haspdv_nm = haspdv;
	normalise(haspdv_nm);
	VECTOR3 tgt_vec_nm = _V(tgt_hrz_pos.x,-(alti-tgt_height_offset), tgt_hrz_pos.z);
	normalise(tgt_vec_nm);

	double tgt_vec_length = sqrt((tgt_hrz_pos.x*tgt_hrz_pos.x)+(tgt_hrz_pos.z*tgt_hrz_pos.z));
	tgt_vec_length = tgt_vec_length + 125;
	VECTOR3 tgt_hrz_pos_adj = tgt_hrz_pos;
	tgt_hrz_pos_adj.y = 0;
	normalise(tgt_hrz_pos_adj);
	tgt_hrz_pos_adj.x = tgt_hrz_pos_adj.x*tgt_vec_length;
	tgt_hrz_pos_adj.y = 0;
	tgt_hrz_pos_adj.z = tgt_hrz_pos_adj.z*tgt_vec_length;

	VECTOR3 tgt_vec_nm2 = _V(tgt_hrz_pos_adj.x,-(alti*0.66), tgt_hrz_pos_adj.z);//0.55
	//VECTOR3 tgt_vec_nm2 = _V(tgt_hrz_pos.x,-(alti*0.66), tgt_hrz_pos.z);//0.55
	normalise(tgt_vec_nm2);

	double dotprod = dotp(tgt_vec_nm,_V(0,-1,0));
	if (dotprod > 1)dotprod = 1;
	else if (dotprod < -1)dotprod = -1;
	 double tgt_angle = acos(dotprod);

	dotprod = dotp(haspdv_nm,_V(0,-1,0));
	if (dotprod > 1)dotprod = 1;
	else if (dotprod < -1)dotprod = -1;
	 double aspd_angle = acos(dotprod);

	 if (tgt_angle > PI)tgt_angle = tgt_angle-PI2;
	 if (tgt_angle < -PI)tgt_angle = tgt_angle+PI2;

	double tgt_range = sqrt((tgt_hrz_pos.x*tgt_hrz_pos.x)+(tgt_hrz_pos.z*tgt_hrz_pos.z));

	VECTOR3 req_fl_vec_hrz =  _V(0,-1,0);

	if(GetThrusterGroupLevel(THGROUP_MAIN) == 0)
	{
	req_fl_vec_hrz = haspdv_nm;
	
	dotprod = dotp(tgt_vec_nm2,haspdv_nm);
	if (dotprod > 1)dotprod = 1;
	else if (dotprod < -1)dotprod = -1;
	double tgt_angle_az = acos(dotprod);
	VECTOR3 rot_vec = crossp(tgt_vec_nm2,haspdv_nm);
	normalise(rot_vec);
	

	

	double mach_rot_factor = 1*(2 - machnum);
	if(mach_rot_factor > 1)mach_rot_factor = 1;
	else if(mach_rot_factor < 0)mach_rot_factor = 0;

	double rot_angle = -2.0 * tgt_angle_az*(1.0+windspeed/40);
	

	if(rot_angle > (10*RAD))rot_angle = 10*RAD;
	else if(rot_angle < (-10*RAD))rot_angle = -10*RAD;

	rot_angle = rot_angle*mach_rot_factor;

	double ct = cos(rot_angle);
	double st = sin(rot_angle);

	double ux = rot_vec.x;
	double uy = rot_vec.y;
	double uz = rot_vec.z;
	double ux2 = ux*ux;
	double uy2 = uy*uy;
	double uz2 = uz*uz;

	MATRIX3 RG;

	RG.m11 = ux2+(1-ux2)*ct;
	RG.m12 = ux*uy*(1-ct)-uz*st;
	RG.m13 = ux*uz*(1-ct)+uy*st;
	RG.m21 = ux*uy*(1-ct)+uz*st;
	RG.m22 = uy2+(1-uy2)*ct;
	RG.m23 = uy*uz*(1-ct)-ux*st;
	RG.m31 = ux*uz*(1-ct)-uy*st;
	RG.m32 = uy*uz*(1-ct)+ux*st;
	RG.m33 = uz2+(1-uz2)*ct;

	req_fl_vec_hrz = mul(RG, haspdv);

	if(alti < 2000)req_fl_vec_hrz =  _V(0,-1,0);

	rchud_sw = 6;
	}

	else
	{
	VECTOR3 rot_vec = crossp(tgt_vec_nm,haspdv_nm);
	normalise(rot_vec);

	dotprod = dotp(tgt_vec_nm,haspdv_nm);
	if (dotprod > 1)dotprod = 1;
	else if (dotprod < -1)dotprod = -1;
	tgt_angle = 0.75*acos(dotprod); //75
	double rot_angle = tgt_angle;


	int windrot_sw = 0;

	if (alti < 350)	//300
	{
	rot_angle = -0.02 * gsvx * aspd_angle * 0.8;// 1.6
	rot_vec = crossp(haspdv_nm,_V(0,-1,0));
	normalise(rot_vec);
	windrot_sw = 1;
	}


	double ct = cos(rot_angle);
	double st = sin(rot_angle);

	double ux = rot_vec.x;
	double uy = rot_vec.y;
	double uz = rot_vec.z;
	double ux2 = ux*ux;
	double uy2 = uy*uy;
	double uz2 = uz*uz;

	MATRIX3 RG;

	RG.m11 = ux2+(1-ux2)*ct;
	RG.m12 = ux*uy*(1-ct)-uz*st;
	RG.m13 = ux*uz*(1-ct)+uy*st;
	RG.m21 = ux*uy*(1-ct)+uz*st;
	RG.m22 = uy2+(1-uy2)*ct;
	RG.m23 = uy*uz*(1-ct)-ux*st;
	RG.m31 = ux*uz*(1-ct)-uy*st;
	RG.m32 = uy*uz*(1-ct)+ux*st;
	RG.m33 = uz2+(1-uz2)*ct;

	req_fl_vec_hrz = mul(RG, haspdv);

	
		
		
	rot_vec = crossp(req_fl_vec_hrz,windvector);
	normalise(rot_vec);
	
	rot_angle = 30*RAD*windspeed*windspeed/1000;
	if (windrot_sw == 1)rot_angle = 20*RAD*windspeed*windspeed/1000;
	if (rot_angle > (5*RAD))rot_angle = 5*RAD;

	ct = cos(rot_angle);
	st = sin(rot_angle);

	ux = rot_vec.x;
	uy = rot_vec.y;
	uz = rot_vec.z;
	ux2 = ux*ux;
	uy2 = uy*uy;
	uz2 = uz*uz;

	RG.m11 = ux2+(1-ux2)*ct;
	RG.m12 = ux*uy*(1-ct)-uz*st;
	RG.m13 = ux*uz*(1-ct)+uy*st;
	RG.m21 = ux*uy*(1-ct)+uz*st;
	RG.m22 = uy2+(1-uy2)*ct;
	RG.m23 = uy*uz*(1-ct)-ux*st;
	RG.m31 = ux*uz*(1-ct)-uy*st;
	RG.m32 = uy*uz*(1-ct)+ux*st;
	RG.m33 = uz2+(1-uz2)*ct;

	req_fl_vec_hrz = mul(RG, req_fl_vec_hrz);
	rchud_sw = 7;
	}

	
	VECTOR3 req_fv_loc = _V(0,0,0);
	HorizonInvRot(-req_fl_vec_hrz,req_fv_loc);
	flight_vector = req_fv_loc;
	GoToFlightVector();

}

void f9h_stg1_starboard::ApplySkin ()
{	
	bs1 = oapiGetTextureHandle (hMhull, 1);
	oapiBlt(bs1, bs2, 0, 0, 0, 0, 512, 1024);

}

void f9h_stg1_starboard::DoBoom ()
{
	VESSELSTATUS2 vs;
	memset (&vs, 0, sizeof(vs));
	vs.version = 2;
	GetStatusEx(&vs);
	OBJHANDLE hVessel = GetHandle();
	double longitude, latitude, radius;
	double heading;
	oapiGetEquPos(hVessel,&longitude,&latitude,&radius);
	oapiGetHeading(hVessel,&heading);
	vs.surf_lat = latitude;
	vs.surf_lng = longitude;
	vs.surf_hdg = heading;
	vs.status = 1;
	DefSetStateEx (&vs);

	//SetMeshVisibilityMode(0,MESHVIS_NEVER);
	//SetMeshVisibilityMode(1,MESHVIS_NEVER);

	boom_sw = boom_sw + step_time;
	if (boom_sw < 2)
	{
		SetThrusterLevel(th_boom[0],1);
		SetThrusterLevel(th_boom[1],1);
	}
	else
	{
		boom_sw = 2;
		SetThrusterLevel(th_boom[0],0);
		SetThrusterLevel(th_boom[1],0);
	}

}

void f9h_stg1_starboard::DoThrustVector ()
{
if (mth_level == 0) return;

double abs_pitch_level = pu_level - pd_level;
double abs_yaw_level = yr_level - yl_level;

//sprintf(oapiDebugString(), "apl %0.5f ayl %0.5f", abs_pitch_level, abs_yaw_level);

double main_max_th = GetThrusterMax(th_main[eng_statusp]);
double rcs_max_th = GetThrusterMax(th_rcs[0]) * 2;
double main_level = mth_level;

double abs_main_th = main_max_th * main_level;
double abs_pitch_th = rcs_max_th * abs_pitch_level;
double abs_yaw_th = rcs_max_th * abs_yaw_level;

VECTOR3 main_vector = _V(-abs_yaw_th,-abs_pitch_th,abs_main_th);
double ratio_main = length(main_vector) / length(_V(0,0,abs_main_th));
normalise(main_vector);

double new_th_main_level = ratio_main * main_level;
if (new_th_main_level > 1)new_th_main_level = 1;
else if (new_th_main_level < 0)new_th_main_level = 0;

SetThrusterDir(th_main[eng_statusp], main_vector);

double ga = acos(dotp(main_vector, _V(0,0,1)));
//sprintf(oapiDebugString(), "gimbal angle %0.6f", ga*DEG);
SetThrusterLevel(th_main[eng_statusp], new_th_main_level);

SetThrusterGroupLevel(THGROUP_ATT_PITCHUP, 0);
SetThrusterGroupLevel(THGROUP_ATT_PITCHDOWN, 0);
SetThrusterGroupLevel(THGROUP_ATT_YAWRIGHT, 0);
SetThrusterGroupLevel(THGROUP_ATT_YAWLEFT, 0);


}

void f9h_stg1_starboard::DoLandingRCS()
{
	VECTOR3 surf_norm_hrz;
	VECTOR3 surf_norm_ves;
	surf_norm_hrz = GetSurfaceNormal();
	HorizonInvRot(surf_norm_hrz,surf_norm_ves);
	normalise(surf_norm_ves);
	double y_angle = atan2(surf_norm_ves.x,surf_norm_ves.z);
	double x_angle = atan2(surf_norm_ves.y,surf_norm_ves.z);

	//sprintf(oapiDebugString(), "y_angle %0.3f x_angle %0.3f", DEG*y_angle, DEG*x_angle);
	//sprintf(oapiDebugString(), "surf_norm_hrz.x %0.3f surf_norm_hrz.y %0.3f surf_norm_hrz.z %0.3f", surf_norm_hrz.x, surf_norm_hrz.y, surf_norm_hrz.z);

	VECTOR3 ang_vel = _V(0,0,0);
	GetAngularVel(ang_vel);

	if (DEG*(abs(y_angle) + abs(x_angle)) < 1)
	{
	SetThrusterLevel(th_rcs2[0], 0);
	SetThrusterLevel(th_rcs2[1], 0);
	SetThrusterLevel(th_rcs2[2], 0);
	SetThrusterLevel(th_rcs2[3], 0);
	return;	
	}

	else if (DEG*(abs(ang_vel.y) + abs(ang_vel.x)) < 0.2)
	{
	SetThrusterLevel(th_rcs2[0], 0);
	SetThrusterLevel(th_rcs2[1], 0);
	SetThrusterLevel(th_rcs2[2], 0);
	SetThrusterLevel(th_rcs2[3], 0);
	return;	
	}

	double y_level_req = DEG*(-y_angle + (2*ang_vel.y));
	if(y_level_req > 1) y_level_req = 1;
	else if(y_level_req < -1) y_level_req = -1;

	double x_level_req = DEG*(-x_angle + (2*ang_vel.x));
	if(x_level_req > 1) x_level_req = 1;
	else if(x_level_req < -1) x_level_req = -1;

	if(y_level_req > 0)
	{
		SetThrusterLevel(th_rcs2[2], y_level_req);
		SetThrusterLevel(th_rcs2[0], 0);
	}
	else if(y_level_req < 0)
	{
		SetThrusterLevel(th_rcs2[2], 0);
		SetThrusterLevel(th_rcs2[0], -y_level_req);
	}

	if(x_level_req > 0)
	{
		SetThrusterLevel(th_rcs2[1], x_level_req);
		SetThrusterLevel(th_rcs2[3], 0);
	}
	else if(x_level_req < 0)
	{
		SetThrusterLevel(th_rcs2[1], 0);
		SetThrusterLevel(th_rcs2[3], -x_level_req);
	}

}