// ==============================================================
//                 ORBITER MODULE: frv
//                  
//
// frv.cpp
// Control module for frv vessel class
//
// 
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include "OrbiterSoundSDK40.h"
#include <stdio.h>
#include <math.h>

// ==============================================================
// Some vessel parameters
// ==============================================================

const double RCS_ISP = 300000;
const double RCSTH = 4000;
const double MAINTH = 80000;
const double SOLP_SPEED = 0.25;
const double FUELMASS = 5000;
const double EMPTY_MASS = 20000;
const double M_E = 2.71828182845904523536;

VECTOR3 net_offset = _V(0,29.6,-12.5);

VECTOR3 bspos1 = _V(2.216, 14.80, 11.373);
VECTOR3 bspos2 = _V(2.216, 13.00, 11.373);
VECTOR3 bspos3 = _V( 1.99, 8.211, 7.121);
VECTOR3 bspos4 = _V(-1.99, 8.211, 7.121);
VECTOR3 bspos5 = _V( 1.4,15.271,12);
VECTOR3 bspos6 = _V(-1.4,15.271,12);
VECTOR3 bscol = _V(1,0,0);
VECTOR3 bscol2 = _V(1,1,1);

#define YELLOW RGB(255, 255, 80)
#define GREEN RGB(0, 225, 0)

static const DWORD ntdvtx = 3;
static TOUCHDOWNVTX tdvtx_geardown[ntdvtx] = {
	{_V(    0, -0.1, 10), 1e6, 1e5, 0.1, 0.1},
	{_V(-8.66, -0.1,-5), 1e6, 1e5, 0.1, 0.1},
	{_V( 8.66, -0.1,-5), 1e6, 1e5, 0.1, 0.1}
};


// Calculate lift coefficient [Cl] as a function of aoa (angle of attack) over -Pi ... Pi
// Implemented here as a piecewise linear function
void VLiftCoeff (VESSEL *v, double aoa, double M, double Re, void *context, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 9;
	static const double AOA[nabsc] = {-180*RAD,-60*RAD,-30*RAD, -2*RAD, 10*RAD,15*RAD,50*RAD,90*RAD,180*RAD};
	static const double CL[nabsc]  = {0,0,0,0,0,0,0,0,0};
	static const double CM[nabsc]  = {0,0,0,0,0,0,0,0,0};
	for (i = 0; i < nabsc-1 && AOA[i+1] < aoa; i++);
	double f = (aoa-AOA[i]) / (AOA[i+1]-AOA[i]);
	*cl = 0;//CL[i] + (CL[i+1]-CL[i]) * f;  // aoa-dependent lift coefficient
	*cm = 0;//CM[i] + (CM[i+1]-CM[i]) * f;  // aoa-dependent moment coefficient
	double saoa = sin(aoa);
	double pd = 0.03 + 0.4*saoa*saoa;  // profile drag
	*cd = 0;// pd + oapiGetInducedDrag (*cl, 1.5, 0.7) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	// profile drag + (lift-)induced drag + transonic/supersonic wave (compressibility) drag
}

void HLiftCoeff (VESSEL *v, double beta, double M, double Re, void *context, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = {-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
	static const double CL[nabsc]   = {0,0,0,0,0,0,0,0};
	for (i = 0; i < nabsc-1 && BETA[i+1] < beta; i++);
	*cl = 0;//CL[i] + (CL[i+1]-CL[i]) * (beta-BETA[i]) / (BETA[i+1]-BETA[i]);
	*cm = 0.0;
	*cd = 0; //0.03 + oapiGetInducedDrag (*cl, 1.5, 0.7) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
}


class frv: public VESSEL3 {
public:
	
	frv (OBJHANDLE hVessel, int flightmodel);
	~frv ();

	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	void clbkPostStep (double simt, double simdt, double mjd);
	void clbkPostCreation();
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	bool clbkLoadGenericCockpit ();
	int MyID;
	

private:
	void DefineAnimations (void);
	void Grapple(void);
	void SetCamDir(void);
	void CalcFair1(void);
	void CalcFair2(void);
	void SpawnBalloon(void);
	void TrackBalloon(void);

	THRUSTER_HANDLE th_rcs[6], th_group[2];
	PROPELLANT_HANDLE hProp;
	ATTACHMENTHANDLE hAtt;
	MGROUP_TRANSFORM *Netscale, *Fairtrans, *Fair1_scale, *Fair2_scale, *Fair3_scale;
	UINT anim_net, anim_fair1, anim_fair2, anim_fair3, anim_fair3_trans;
	double net_proc;
	int net_status, fair_status, night_mesh_status, cam_status;
	BEACONLIGHTSPEC beac1, beac2, beac3, beac4, beac5, beac6;
	SpotLight *spot_light;
	int tgt_fair1_status, tgt_fair2_status, balloon_status;
	OBJHANDLE hFair1, hFair2;
	double fair1_alt, fair1_dist, fair1_brg, fair1_range;
	double fair2_alt, fair2_dist, fair2_brg, fair2_range; 
	double wind_data_alt, wind_spd, wind_brg, steptime, timer;
	int next_alt;

};

frv::frv (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
}

frv::~frv ()
{
	delete Netscale;
	delete Fairtrans;
	delete Fair1_scale;
	delete Fair2_scale;
	delete Fair3_scale;
}

// Define animation sequences for moving parts
// --------------------------------------------------------------
void frv::DefineAnimations (void)
{
	// 1. net
	anim_net = CreateAnimation (0);

	static UINT NetGrp0[1] = {24};
	Netscale = new MGROUP_SCALE(0, NetGrp0, 1, _V(0, 25.991,-12.56), _V(1,10,1));
	AddAnimationComponent (anim_net, 0, 1, Netscale);

	// 2. Fair_Net
	anim_fair3 = CreateAnimation (0);

	static UINT FairGrp1[1] = {2};
	Fair3_scale = new MGROUP_SCALE(0, FairGrp1, 1, _V(0,0,0), _V(0.000001, 0.000001, 0.000001));
	AddAnimationComponent (anim_fair3, 0, 1, Fair3_scale);

	anim_fair3_trans = CreateAnimation (0);

	static UINT FairGrp2[1] = {2};
	Fairtrans = new MGROUP_TRANSLATE(0, FairGrp2, 1, _V(0, -15, 0));
	AddAnimationComponent (anim_fair3_trans, 0, 1, Fairtrans);

	// 3 Fair 1
	anim_fair1 = CreateAnimation (0);

	static UINT FairGrp3[1] = {0};
	Fair1_scale = new MGROUP_SCALE(0, FairGrp3, 1, _V(0,0,0), _V(0.000001, 0.000001, 0.000001));
	AddAnimationComponent (anim_fair1, 0, 1, Fair1_scale);

	// 3 Fair 2
	anim_fair2 = CreateAnimation (0);

	static UINT FairGrp4[1] = {1};
	Fair2_scale = new MGROUP_SCALE(0, FairGrp4, 1, _V(0,0,0), _V(0.000001, 0.000001, 0.000001));
	AddAnimationComponent (anim_fair2, 0, 1, Fair2_scale);


}



// ==============================================================
// frv class interface
// ==============================================================

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void frv::clbkSetClassCaps (FILEHANDLE cfg)
{
	net_proc = 0;
	net_status = 0;
	fair_status = 0;
	night_mesh_status = 0;
	cam_status = 0;
	tgt_fair1_status = 0;
	tgt_fair2_status = 0;
	fair1_alt = 0;
	fair1_dist = 0;
	fair1_brg = 0;
	fair1_range = 0;
	fair2_alt = 0;
	fair2_dist = 0;
	fair2_brg = 0;
	fair2_range = 0;
	balloon_status = 0;
	wind_data_alt = 0;
	wind_spd = 0;
	wind_brg = 0;
	steptime = 0;
	timer = 0;
	next_alt = 100;
	DefineAnimations ();
	
	// physical specs
	SetCW (0.1, 0.1, 0.1, 0.1);
	SetRotDrag (_V(0.1,0.1,0.1));
	SetPMI (_V(20,20,20));
	SetCrossSections (_V(1,1,1));
	SetSize (30);
	SetEmptyMass (EMPTY_MASS);
	SetMeshVisibilityMode ((AddMesh ("FalconHeavy/frv")), MESHVIS_ALWAYS);
	SetMeshVisibilityMode ((AddMesh ("FalconHeavy/frv_cabin")), MESHVIS_COCKPIT);
	SetMeshVisibilityMode ((AddMesh ("FalconHeavy/frv_cabin_night")), MESHVIS_NEVER);
	//SetTouchdownPoints (_V(0, -0.0001, 8), _V(-4,-0.0001,-8), _V(4,-0.0001,-8));
	SetTouchdownPoints(tdvtx_geardown, ntdvtx);

	// propellant resources

	hProp = CreatePropellantResource (FUELMASS, FUELMASS);

	
	// ***************** thruster definitions *******************
	
	th_rcs[ 0] = CreateThruster (_V(0, 0,20), _V( 1, 0, 0), RCSTH, hProp, RCS_ISP);
	th_rcs[ 1] = CreateThruster (_V(0, 0, 20), _V(-1, 0, 0), RCSTH, hProp, RCS_ISP);
	th_rcs[ 2] = CreateThruster (_V(0, 0,-20), _V( 1, 0, 0), RCSTH, hProp, RCS_ISP);
	th_rcs[ 3] = CreateThruster (_V(0, 0,-20), _V(-1, 0, 0), RCSTH, hProp, RCS_ISP);

	th_rcs[ 4] = CreateThruster (_V(0,0, 0), _V( 0, 0, 1), MAINTH, hProp, RCS_ISP);
	th_rcs[ 5] = CreateThruster (_V(0,0, 0), _V( 0, 0,-1), MAINTH, hProp, RCS_ISP);
	

	th_group[0] = th_rcs[ 0];
	th_group[1] = th_rcs[ 3];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[ 1];
	th_group[1] = th_rcs[ 2];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	


	th_group[0] = th_rcs[ 4];
	CreateThrusterGroup (th_group, 1, THGROUP_MAIN);

	th_group[0] = th_rcs[ 5];
	CreateThrusterGroup (th_group, 1, THGROUP_RETRO);



	///////////////////////////////////////////////////////
	/////AirFoils


	CreateAirfoil3 (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 0, 0.1, 0.2, 0.1);
	CreateAirfoil3 (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 0, 0.1, 0.2, 0.1);

	///////////////////////////////////////////////////////
	////// atts

	hAtt = CreateAttachment(false, _V(0,4.871,-8.494), _V(0,-1, 0),_V(0, 0, 1), "XS");

	/////////////////////////
	SURFHANDLE smoke = oapiRegisterParticleTexture ("Contrail1");

	PARTICLESTREAMSPEC smoketrail = {
		0, 1.5, 6, 8, 0.08, 0.5, -0.2, 0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLIN, 0.5, 1,
		smoke
	};

	AddExhaustStream(th_rcs[ 4], _V(-3,0.76,-32), &smoketrail);
	AddExhaustStream(th_rcs[ 4], _V(-2,0.76,-32), &smoketrail);
	AddExhaustStream(th_rcs[ 4], _V(-1,0.76,-32), &smoketrail);
	AddExhaustStream(th_rcs[ 4], _V(0,0.76,-32), &smoketrail);
	AddExhaustStream(th_rcs[ 4], _V(1,0.76,-32), &smoketrail);
	AddExhaustStream(th_rcs[ 4], _V(2,0.76,-32), &smoketrail);
	AddExhaustStream(th_rcs[ 4], _V(3,0.76,-32), &smoketrail);
	AddExhaustStream(th_rcs[ 5], _V(-5.4,0.15,-30), &smoketrail);

	///////////////////////////// LIGHTS

	beac1.shape = BEACONSHAPE_STAR;
	beac1.pos = &bspos1;
	beac1.col = &bscol;
	beac1.size = 0.75;
	beac1.falloff = 0.5;
	beac1.period = 0;
	beac1.duration = 0;
	beac1.tofs = 0;
	beac1.active = false;
	AddBeacon (&beac1);

	beac2.shape = BEACONSHAPE_STAR;
	beac2.pos = &bspos2;
	beac2.col = &bscol;
	beac2.size = 0.75;
	beac2.falloff = 0.5;
	beac2.period = 0;
	beac2.duration = 0;
	beac2.tofs = 0;
	beac2.active = false;
	AddBeacon (&beac2);

	beac3.shape = BEACONSHAPE_STAR;
	beac3.pos = &bspos3;
	beac3.col = &bscol2;
	beac3.size = 1.5;
	beac3.falloff = 0.5;
	beac3.period = 0;
	beac3.duration = 0;
	beac3.tofs = 0;
	beac3.active = false;
	AddBeacon (&beac3);

	beac4.shape = BEACONSHAPE_STAR;
	beac4.pos = &bspos4;
	beac4.col = &bscol2;
	beac4.size = 1.5;
	beac4.falloff = 0.5;
	beac4.period = 0;
	beac4.duration = 0;
	beac4.tofs = 0;
	beac4.active = false;
	AddBeacon (&beac4);

	beac5.shape = BEACONSHAPE_STAR;
	beac5.pos = &bspos5;
	beac5.col = &bscol2;
	beac5.size = 1.5;
	beac5.falloff = 0.5;
	beac5.period = 0;
	beac5.duration = 0;
	beac5.tofs = 0;
	beac5.active = false;
	AddBeacon (&beac5);

	beac6.shape = BEACONSHAPE_STAR;
	beac6.pos = &bspos6;
	beac6.col = &bscol2;
	beac6.size = 1.5;
	beac6.falloff = 0.5;
	beac6.period = 0;
	beac6.duration = 0;
	beac6.tofs = 0;
	beac6.active = false;
	AddBeacon (&beac6);

	COLOUR4 col_d = {0.9,0.9,0.8,0};
	COLOUR4 col_s = {0.9,0.9,0.8,0};
	COLOUR4 col_a = {0,0,0,0};
	COLOUR4 col_white = {1,1,1,0};
	spot_light = (SpotLight*)AddSpotLight(_V(0,12.37,10.37), _V(0,0,-1), 2000, 1.8e-4, 0, 1.8e-4, RAD*90, RAD*120, col_d, col_s, col_a);
	spot_light->Activate(false);
}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new frv (hvessel, flightmodel);
}




// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (frv*)vessel;
}
// ==============================================================
// Overloaded clbkPostCreation callback functions
// ==============================================================

void frv::clbkPostCreation ()
{
	MyID=ConnectToOrbiterSoundDLL(GetHandle());
	ReplaceStockSound(MyID,"Sound\\Vessel\\attsustain.wav",	REPLACE_MAIN_THRUST);
	SoundOptionOnOff(MyID,PLAYRADIOATC,FALSE);
	//SoundOptionOnOff3(MyID,PLAYWHENATTITUDEMODECHANGE,FALSE);
	SoundOptionOnOff(MyID,PLAYGPWS,FALSE);
	SoundOptionOnOff(MyID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);
	SoundOptionOnOff(MyID,PLAYLANDINGANDGROUNDSOUND,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINAIRCONDITIONING,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINRANDOMAMBIANCE,FALSE);
	SoundOptionOnOff(MyID,DISPLAYTIMER,FALSE);
	SoundOptionOnOff(MyID,PLAYDOCKLANDCLEARANCE,FALSE);
	SoundOptionOnOff(MyID,PLAYRADARBIP,FALSE);
	
	if (GetPropellantMass(hProp)< 0.001 && oapiGetSimTime() > 0.1)
	{
		SetPropellantMass(hProp,FUELMASS);
	}

	if (fair_status == 1)
	{
		SetAnimation (anim_fair1, 0);
		SetAnimation (anim_fair2, 1);
	}
	else if (fair_status == 2)
	{
		SetAnimation (anim_fair1, 0);
		SetAnimation (anim_fair2, 0);
	}
	else
	{
		SetAnimation (anim_fair1, 1);
		SetAnimation (anim_fair2, 1);
	}

	if(net_status == 1)
	{
	SetAnimation (anim_fair3, 0);
	SetAnimation (anim_fair3_trans, net_proc);
	}
	else
	{
	SetAnimation (anim_fair3, 1);
	SetAnimation (anim_fair3_trans, 0);
	}

}
void frv::clbkSaveState (FILEHANDLE scn)
{



	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState (scn);

	// custom parameters
	sprintf (cbuf, "%d %d %0.4f", fair_status, net_status, net_proc);
	oapiWriteScenario_string (scn, "NET", cbuf);

	sprintf (cbuf, "%d %0.4f %d %0.1f %0.1f %0.1f", balloon_status, timer, next_alt, wind_data_alt, wind_brg, wind_spd);
	oapiWriteScenario_string (scn, "BLN", cbuf);
	
}
void frv::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "NET", 3))
		{
			sscanf (line+3, "%d%d%lf", &fair_status, &net_status, &net_proc);
		}
		else if (!strnicmp (line, "BLN", 3))
		{
			sscanf (line+3, "%d%lf%d%lf%lf%lf", &balloon_status, &timer, &next_alt, &wind_data_alt, &wind_brg, &wind_spd);
		}
		else
		{
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }
	
	SetAnimation (anim_net, net_proc);
	
}






int frv::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) {

  if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) 
  {
	  switch (key) 
	  {
		
	  }
	  return true;
  } 
  else { // unmodified keys
    switch (key) {
	
	//case OAPI_KEY_G: // deploy/stow hoist
	//	
	//	return 1;

	case OAPI_KEY_E: //  cam_status
		if (cam_status == 0)cam_status = 1;
		else if (cam_status == 1)cam_status = 0;
		SetCamDir();
		return 1;

	case OAPI_KEY_J: //  cam_status
		if(balloon_status == 0)SpawnBalloon();
		return 1;
	
		return true;
	}
}
  
return 0;
}




void frv::clbkPreStep(double SimT, double SimDT, double MJD)
{
	steptime = SimDT;

	hFair1 = oapiGetObjectByName("Fair1");
	if (hFair1 != NULL) tgt_fair1_status = 1;
	else tgt_fair1_status = 0;
	if(tgt_fair1_status == 1) CalcFair1();

	hFair2 = oapiGetObjectByName("Fair2");
	if (hFair2 != NULL) tgt_fair2_status = 1;
	else tgt_fair2_status = 0;
	if(tgt_fair2_status == 1) CalcFair2();

//sprintf(oapiDebugString(), "%d", ibsw);

	VECTOR3 surf_vel = _V(0,0,0);
	GetGroundspeedVector(FRAME_LOCAL,surf_vel);
	double speed = length(surf_vel);
	if(speed > 1)
	{
	if(speed > 20)speed = 20;
	VECTOR3 resist_force = -surf_vel;
	normalise(resist_force);
	resist_force.x = resist_force.x*speed*60000/16;
	resist_force.y = resist_force.y*speed*60000/16;
	resist_force.z = resist_force.z*speed*60000/16;
	AddForce(resist_force, _V(0,0,0));
	}
	////////////// night check
	VECTOR3 rsunloc;
	VECTOR3 rsun = _V(0,0,0);
	Global2Local(rsun,rsunloc);

	if(rsunloc.y < 0)
	{
	if(night_mesh_status == 0)
		{
		SetMeshVisibilityMode(1, MESHVIS_NEVER);
		SetMeshVisibilityMode(2, MESHVIS_COCKPIT);
		night_mesh_status = 1;
		beac1.active = true;
		beac2.active = true;
		beac3.active = true;
		beac4.active = true;
		beac5.active = true;
		beac6.active = true;
		spot_light->Activate(true);
		}
	}
	else if(rsunloc.y > 0)
	{
	if(night_mesh_status == 1)
		{
		SetMeshVisibilityMode(1, MESHVIS_COCKPIT);
		SetMeshVisibilityMode(2, MESHVIS_NEVER);
		night_mesh_status = 0;
		beac1.active = false;
		beac2.active = false;
		beac3.active = false;
		beac4.active = false;
		beac5.active = false;
		beac6.active = false;
		spot_light->Activate(false);
		}
	}

	if(fair_status < 2)Grapple();

	/// animations
	if (net_status == 0) net_proc = 0;

	else if (net_status == 1) 
	{
		double da = SimDT * SOLP_SPEED;
		net_proc = net_proc + da;
		if (net_proc > 1) 
		{
			net_proc = 1;
			net_status = 2;
		}
		SetAnimation (anim_fair3_trans, net_proc);
	}

	else if (net_status == 2)
	{
		net_proc = 1;
		SetAnimation (anim_fair3, 1);
		if (fair_status == 0)
		{
			SetAnimation (anim_fair1, 0);
			fair_status = 1;
		}
		else if (fair_status == 1)
		{
			SetAnimation (anim_fair2, 0);
			fair_status = 2;
		}
		net_status = 3;
	}

	else if (net_status == 3) 
	{
		double da = SimDT * SOLP_SPEED;
		net_proc = net_proc - da;
		if (net_proc < 0) 
		{
			net_proc = 0;
			net_status = 0;
		}
	}
	SetAnimation (anim_net, net_proc);

	if(net_status == 4)net_status = 1;

	if(balloon_status == 1)TrackBalloon();
}


void frv::clbkPostStep (double simt, double simdt, double mjd)
{

}

bool frv::clbkLoadGenericCockpit ()
{
	SetCamDir();
	return true;
}

void frv::SetCamDir()
{
	VECTOR3 camdefdir = _V(0,0,1);
	VECTOR3 camdefpos = _V(0,9.9,15.35);
	if(cam_status == 1)
	{
		camdefdir = _V(0,0,-1);
		camdefpos = _V(0,9.9,9);
	}
	SetCameraDefaultDirection(camdefdir, 0);
	SetCameraOffset(camdefpos);
	oapiCameraAttach(GetHandle(), 2);
}


void frv::Grapple ()
{

VECTOR3 att_pos = _V(0,29.6,-12.5);
double grapple_dist_max = 21;

VECTOR3 att_pos_global = _V(0,0,0);
Local2Global(att_pos, att_pos_global);

int num_ves = oapiGetVesselCount();
for (int i = 0; i < num_ves; i++) 
	{

	OBJHANDLE hVessel = oapiGetVesselByIndex(i);
	if (hVessel == GetHandle()) continue;

	VECTOR3 ves_pos_glob = _V(0,0,0);
	oapiGetGlobalPos(hVessel, &ves_pos_glob);
	if (dist(ves_pos_glob, att_pos_global) > grapple_dist_max)continue;

	double ves_alt = 0;
	oapiGetAltitude(hVessel, ALTMODE_GROUND, &ves_alt);
	if (ves_alt < 34.5 && ves_alt > 24.5)
		{
		oapiDeleteVessel(hVessel, GetHandle());
		net_status = 4;
		SetAnimation (anim_fair3, 0);
		break;
		}
	}
}


bool frv::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);

	//int chw = 16;

	
	char test[256];
	strcpy(test,"abcdefghijklmnopqrstuvwxyz");
	int it = skp->GetTextWidth(test, 26);
	it = (int)(it/26);
	int chw = it;

	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}



	int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	int w3 = (int)(232*d);
	int w4 = (int)(248*d);
	int w5 = (int)(264*d);
	int w6 = (int)(280*d);
	int w7 = (int)(296*d);
	int w8 = (int)(312*d);
	int w9 = (int)(328*d);
	//int w10 = (int)(344*d);
	//int w11 = (int)(360*d);
	//int w12 = (int)(376*d);
	//int w13 = (int)(392*d);
	//int w14 = (int)(408*d);
	//int w15 = (int)(424*d);

	if(tgt_fair1_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Fair1: No Target");
	skp->Text ((10+roxl), (wd+royl), abuf,strlen(abuf));
	}
	else
	{
		{
		skp->SetTextColor(0x0066FFFF);
		char abuf[256];
		sprintf(abuf,"Fair1: Tracking");
		skp->Text ((10+roxl), (wd+royl), abuf,strlen(abuf));
		}

		{
		//skp->SetTextColor(0x0066FF66);
		char abuf[256];
		sprintf(abuf,"Fair1: Brg.%0.1f Alt.%0.1fkm Dist.%0.1fkm Flight Range %0.1fkm",(fair1_brg*DEG),(fair1_alt*0.001), (fair1_dist*0.001), (fair1_range*0.001));
		skp->Text ((10+roxl), (wc+royl), abuf,strlen(abuf));
		}
	}

	if(tgt_fair2_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Fair2: No Target");
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}
	else
	{
		{
		skp->SetTextColor(0x0066FFFF);
		char abuf[256];
		sprintf(abuf,"Fair2: Tracking");
		skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
		}

		{
		//skp->SetTextColor(0x0066FF66);
		char abuf[256];
		sprintf(abuf,"Fair2: Brg.%0.1f Alt.%0.1fkm Dist.%0.1fkm Flight Range %0.1fkm",(fair2_brg*DEG),(fair2_alt*0.001), (fair2_dist*0.001), (fair2_range*0.001));
		skp->Text ((10+roxl), (w2+royl), abuf,strlen(abuf));
		}
	}

	if(balloon_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"J = Acquire Wind Data");
	skp->Text ((10+roxl), (w4+royl), abuf,strlen(abuf));
	}
	else
	{
		{
		skp->SetTextColor(0x00FFFFFF);
		char abuf[256];
		sprintf(abuf,"Acquiring Wind Data");
		skp->Text ((10+roxl), (w4+royl), abuf,strlen(abuf));
		}
	}

	if((wind_data_alt + balloon_status) > 0)
	{
		//skp->SetTextColor(0x006688FF);
		char abuf[256];
		sprintf(abuf,"Average Wind to %dm : Brg.%0.1f Spd.%0.1fm/s ", (int)(wind_data_alt), wind_brg*DEG, wind_spd);
		skp->Text ((10+roxl), (w5+royl), abuf,strlen(abuf));
	}

	

	return true;
}

void frv::CalcFair1()
{
	VECTOR3 fair1_pos_glob = _V(0,0,0);
	oapiGetGlobalPos(hFair1, &fair1_pos_glob);
	VECTOR3 fair1_pos_loc = _V(0,0,0);
	Global2Local(fair1_pos_glob,fair1_pos_loc);
	VECTOR3 fair1_pos_hrz = _V(0,0,0);
	HorizonRot(fair1_pos_loc, fair1_pos_hrz);
	fair1_pos_hrz = fair1_pos_hrz - net_offset;

	fair1_alt = fair1_pos_hrz.y;
	fair1_dist = length(_V(fair1_pos_hrz.x, 0, fair1_pos_hrz.z));
	fair1_brg = atan2(fair1_pos_hrz.x, fair1_pos_hrz.z);
	if (fair1_brg < 0)fair1_brg = fair1_brg + PI2;

	///////////////////
	double alt = fair1_alt;
	if(alt > 5500)
	{
	VECTOR3 haspdv = _V(0,0,0);
	oapiGetGroundspeedVector(hFair1,FRAME_HORIZON,&haspdv);

	double gsvx = sqrt((haspdv.x*haspdv.x)+(haspdv.z*haspdv.z));

	double vmass = 860;

	OBJHANDLE hearth=GetSurfaceRef();

	VECTOR3 relvelhz = _V(0,0,0);
	oapiGetRelativeVel(hFair1, hearth, &relvelhz);
	double rvh = length(relvelhz);
	rvh = sqrt(abs((rvh*rvh) - (haspdv.y*haspdv.y)));
	double acc_cfg = rvh*rvh/(alt+6371010);

	double rds = 6371010+alt;
    double GM = GGRAV*oapiGetMass(hearth);
	double gravt = GM/(rds*rds);
	gravt = gravt - acc_cfg;
	double g = -gravt;
	double aa = g/2;
	double aaa = -g;
	double bb = haspdv.y;
	double cc = alt;
	double timetoimp = (-bb-(sqrt((bb*bb)-(4*aa*cc))))/(2*aa);
	double timetoimp2 = (-bb-(sqrt((bb*bb)-(4*aa*(cc-20000)))))/(2*aa);
	if (alt < 20000)timetoimp2 = 0;

	double range1 = gsvx*timetoimp2*1.04;  // above 30km
	
	double k = 75*9.81/8; // 2
	double expo = -k*(timetoimp-timetoimp2)/vmass;
	double range2 = vmass*gsvx/k*(1-(pow(M_E, expo))); //below 25km
	fair1_range = (range1+range2);
	}
	else
	{
	VECTOR3 haspdv = _V(0,0,0);
	oapiGetAirspeedVector(hFair1,FRAME_HORIZON,&haspdv);

	double avg_sink_rate = (3.8 - haspdv.y)*0.5;
	double flight_time = 0.9*alt/avg_sink_rate;
	double air_spd = length(_V(haspdv.x, 0, haspdv.z));
	double avg_air_speed = (16 + air_spd)*0.5;
	fair1_range = avg_air_speed*flight_time;
	}
}

void frv::CalcFair2()
{
	VECTOR3 fair2_pos_glob = _V(0,0,0);
	oapiGetGlobalPos(hFair2, &fair2_pos_glob);
	VECTOR3 fair2_pos_loc = _V(0,0,0);
	Global2Local(fair2_pos_glob,fair2_pos_loc);
	VECTOR3 fair2_pos_hrz = _V(0,0,0);
	HorizonRot(fair2_pos_loc, fair2_pos_hrz);
	fair2_pos_hrz = fair2_pos_hrz - net_offset;

	fair2_alt = fair2_pos_hrz.y;
	fair2_dist = length(_V(fair2_pos_hrz.x, 0, fair2_pos_hrz.z));
	fair2_brg = atan2(fair2_pos_hrz.x, fair2_pos_hrz.z);
	if (fair2_brg < 0)fair2_brg = fair2_brg + PI2;

	///////////////////
	double alt2 = fair2_alt;
	if(alt2 > 6500)
	{
	VECTOR3 haspdv = _V(0,0,0);
	oapiGetGroundspeedVector(hFair2,FRAME_HORIZON,&haspdv);

	double gsvx = sqrt((haspdv.x*haspdv.x)+(haspdv.z*haspdv.z));

	double vmass = 860;

	OBJHANDLE hearth=GetSurfaceRef();

	VECTOR3 relvelhz = _V(0,0,0);
	oapiGetRelativeVel(hFair2, hearth, &relvelhz);
	double rvh = length(relvelhz);
	rvh = sqrt(abs((rvh*rvh) - (haspdv.y*haspdv.y)));
	double acc_cfg = rvh*rvh/(alt2+6371010);

	double rds = 6371010+alt2;
    double GM = GGRAV*oapiGetMass(hearth);
	double gravt = GM/(rds*rds);
	gravt = gravt - acc_cfg;
	double g = -gravt;
	double aa = g/2;
	double aaa = -g;
	double bb = haspdv.y;
	double cc = alt2;
	double timetoimp = (-bb-(sqrt((bb*bb)-(4*aa*cc))))/(2*aa);
	double timetoimp2 = (-bb-(sqrt((bb*bb)-(4*aa*(cc-20000)))))/(2*aa);
	if (alt2 < 20000)timetoimp2 = 0;

	double range1 = gsvx*timetoimp2*1.04;  // above 30km
	
	double k = 75*9.81/8; // 2
	double expo = -k*(timetoimp-timetoimp2)/vmass;
	double range2 = vmass*gsvx/k*(1-(pow(M_E, expo))); //below 25km
	fair2_range = (range1+range2);
	}
	else
	{
	VECTOR3 haspdv = _V(0,0,0);
	oapiGetAirspeedVector(hFair2,FRAME_HORIZON,&haspdv);

	double avg_sink_rate = (3.8 - haspdv.y)*0.5;
	double flight_time = 0.9*alt2/avg_sink_rate;
	double air_spd = length(_V(haspdv.x, 0, haspdv.z));
	double avg_air_speed = (16 + air_spd)*0.5;
	fair2_range = avg_air_speed*flight_time;
	}
}

void frv::SpawnBalloon()
{
	VESSELSTATUS vs;
	GetStatus(vs);
	Local2Rel (_V(0,9,23), vs.rpos);
	OBJHANDLE hEarth = GetSurfaceRef();
	GetRelativeVel(hEarth, vs.rvel);
	vs.status = 0;
	GetGlobalOrientation(vs.arot);
	oapiCreateVessel("Weather_Balloon", "FalconHeavy/weather_balloon", vs);
	balloon_status = 1;
}

void frv::TrackBalloon()
{


	OBJHANDLE hBalloon = oapiGetObjectByName("Weather_Balloon");
	if (hBalloon == NULL)
	{
		balloon_status = 0;
		timer = 0;
		next_alt = 0;
		return;
	}

	timer = timer + steptime;

	VECTOR3 balloon_pos_glob;
	oapiGetGlobalPos(hBalloon, &balloon_pos_glob);
	VECTOR3 balloon_pos_loc;
	VECTOR3 offset = _V(0,0,22.999);
	Global2Local(balloon_pos_glob, balloon_pos_loc);
	balloon_pos_loc = balloon_pos_loc - offset;

	VECTOR3 balloon_pos_hrz;
	HorizonRot(balloon_pos_loc, balloon_pos_hrz);

	if (balloon_pos_hrz.y > next_alt)
	{
	wind_spd = length(_V(balloon_pos_hrz.x,0,balloon_pos_hrz.z))/timer;
	wind_brg = atan2(-balloon_pos_hrz.x,-balloon_pos_hrz.z);
	if(wind_brg < 0)wind_brg = wind_brg + PI2;
	wind_data_alt = next_alt;
	next_alt = 100*((int)(balloon_pos_hrz.y * 0.01) + 1);
	}




}