// ==============================================================
//                 ORBITER MODULE: landpad
//
// landpad.cpp
// Control module for landpad vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"

// ==============================================================
// Some vessel parameters
// ==============================================================

VECTOR3 bspos = _V(59.3, 0.7, 38.51);
VECTOR3 bspos1 = _V(-59.3, 0.7, -38.51);
VECTOR3 bspos2 = _V(-38.51, 0.7, 59.3);
VECTOR3 bspos3 = _V(38.51, 0.7, -59.3);
VECTOR3 bscol = _V(1,0,0);

const double PB_FUELMASS = 750.0;
const double PB_ISP = 5e4;
const double PB_MAXHOVERTH = 1000;


static const DWORD ntdvtx_geardown = 3;
static TOUCHDOWNVTX tdvtx_geardown[ntdvtx_geardown] = {
	{_V(    0,-0.5,100), 1e6, 1e5, 1, 1},
	{_V(-86.6,-0.5,-50), 1e6, 1e5, 1, 1},
	{_V( 86.6,-0.5,-50), 1e6, 1e5, 1, 1}
};

class landpad: public VESSEL3 {
public:
	landpad (OBJHANDLE hVessel, int flightmodel)
		: VESSEL3 (hVessel, flightmodel) {}

	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	//void clbkPostStep (double simt, double simdt, double mjd);
	//void clbkPostCreation();
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	//int clbkConsumeDirectKey(char *kstate);
	//bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);

	THRUSTER_HANDLE th_hover;

private:
	//int light_status;
	SpotLight *spotlight1;
	BEACONLIGHTSPEC beac, beac1, beac2, beac3;
	int pad_vis;

};
// Define animation sequences for moving parts
// --------------------------------------------------------------



// ==============================================================
// landpad class interface
// ==============================================================

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void landpad::clbkSetClassCaps (FILEHANDLE cfg)
{
	pad_vis = 0;

	SetEmptyMass (25000);
	SetCW (1, 1, 1, 1);
	SetSize (150);
	SetCrossSections (_V(100,100,100));
	SetPMI (_V(200,100,200));
	//SetTouchdownPoints (_V(0,-0.4,100), _V(-86.6,-0.4,-50), _V(86.6,-0.4,-50));
	SetTouchdownPoints(tdvtx_geardown,ntdvtx_geardown);
	SetRotDrag (_V(2,2,2));

	SetMeshVisibilityMode ((AddMesh ("FalconHeavy/landingpad")), MESHVIS_ALWAYS);
	SetCameraOffset (_V(0,10,-100));


	COLOUR4 col_d = {1,1,0.9,0};
	COLOUR4 col_s = {1,1,0.9,0};
	COLOUR4 col_a = {0,0,0,0};
	
	spotlight1 = (SpotLight*)AddSpotLight(_V(0,20,-100), _V(0,0,1),400, 1, 0, 5e-5, 45*RAD, (90*RAD), col_d, col_s, col_a);
	spotlight1->Activate(false);

	// beacons
	beac.shape = BEACONSHAPE_STAR;
	beac.pos = &bspos;
	beac.col = &bscol;
	beac.size = 1;
	beac.falloff = 0.5;
	beac.period = 2;
	beac.duration = 0.2;
	beac.tofs = 0;
	beac.active = false;
	AddBeacon (&beac);

	beac1.shape = BEACONSHAPE_STAR;
	beac1.pos = &bspos1;
	beac1.col = &bscol;
	beac1.size = 1;
	beac1.falloff = 0.5;
	beac1.period = 2;
	beac1.duration = 0.2;
	beac1.tofs = 0;
	beac1.active = false;
	AddBeacon (&beac1);

	beac2.shape = BEACONSHAPE_STAR;
	beac2.pos = &bspos2;
	beac2.col = &bscol;
	beac2.size = 1;
	beac2.falloff = 0.5;
	beac2.period = 2;
	beac2.duration = 0.2;
	beac2.tofs = 0;
	beac2.active = false;
	AddBeacon (&beac2);

	beac3.shape = BEACONSHAPE_STAR;
	beac3.pos = &bspos3;
	beac3.col = &bscol;
	beac3.size = 1;
	beac3.falloff = 0.5;
	beac3.period = 2;
	beac3.duration = 0.2;
	beac3.tofs = 0;
	beac3.active = false;
	AddBeacon (&beac3);

	PROPELLANT_HANDLE ph_vernier = CreatePropellantResource (PB_FUELMASS);

	th_hover = CreateThruster (_V(0,0,0), _V(0,0,1), PB_MAXHOVERTH, ph_vernier, PB_ISP);

	CreateThrusterGroup(&th_hover,1,THGROUP_MAIN);
}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new landpad (hvessel, flightmodel);
}
// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (landpad*)vessel;
}
// ==============================================================
// Overloaded clbkPostCreation callback functions
// ==============================================================

//void landpad::clbkPostCreation ()
//{


//}

void landpad::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState(scn);

	sprintf (cbuf, "%d",pad_vis);
	oapiWriteScenario_string (scn, "PADVIS", cbuf);
}

void landpad::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
  char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "PADVIS", 6)) {
		sscanf (line+6, "%d", &pad_vis);
		}else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }
	

	if (pad_vis == 1)SetMeshVisibilityMode(0,MESHVIS_NEVER);
}

//int landpad::clbkConsumeDirectKey(char *kstate)
//{

//}


int landpad::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) 
{
	if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) 
  {	 
	switch (key)
	{
	

	} 
  }

  else if (KEYMOD_CONTROL (kstate)) 
  { 
	switch (key)
	{
		
	} 
  }

else { // unmodified keys
    switch (key) {

	case OAPI_KEY_N:
			if (pad_vis == 0)
			{
				pad_vis = 1;
				SetMeshVisibilityMode(0,MESHVIS_NEVER);
			}
			else if (pad_vis == 1)
			{
				pad_vis = 0;
				SetMeshVisibilityMode(0,MESHVIS_ALWAYS);
			}
		return 1;
		}
	}
	return 0;
}


void landpad::clbkPreStep(double SimT, double SimDT, double MJD) 
{
	//shadowcheck
	VECTOR3 rsuninv;
	VECTOR3 rsunloc;
	VECTOR3 rsun;
	GetGlobalPos(rsun);
	rsuninv = _V(-rsun.x, -rsun.y, -rsun.z);
	Global2Local(rsuninv,rsunloc);

	if (spotlight1->IsActive()== false && rsunloc.y < 0)
			{
			spotlight1->Activate(true);
			beac.active = true;
			beac1.active = true;
			beac2.active = true;
			beac3.active = true;
			}
	else if (spotlight1->IsActive()== true && rsunloc.y > 0)
			{
			spotlight1->Activate(false);
			beac.active = false;
			beac1.active = false;
			beac2.active = false;
			beac3.active = false;
			}

if (spotlight1->IsActive() == true)
{
	VECTOR3 gpos = _V(0,0,0);
	VECTOR3 lpos = _V(0,0,0);
	VECTOR3 gpad = _V(0,0,0);
	//Local2Global(_V(0,40,0),gpad);

	for (DWORD i = 0; i < oapiGetVesselCount(); i++)
	{
	OBJHANDLE hV = oapiGetVesselByIndex (i);

	if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
	

	oapiGetGlobalPos (hV, &gpos);
	Global2Local(gpos, lpos);

	if (length(lpos) > 5000)
		{	
		spotlight1->Activate(false);
		continue;
		}

	double lpos_range = sqrt((lpos.x*lpos.x)+(lpos.z*lpos.z));

	if (lpos.y < 5000 && lpos.y > 50)
		{ // in range
		spotlight1->Activate(true);
		break;
		}
	else if (lpos.y < 50 && lpos_range < 300)
		{
		spotlight1->Activate(true);
		break;
		}
	else spotlight1->Activate(false);
	}
}


}


//void landpad::clbkPostStep (double simt, double simdt, double mjd)
//{
	
//}



//bool landpad::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
//{
	// draw the default HUD

//}