// ==============================================================
//                 ORBITER MODULE: slc40h
//
// slc40h.cpp
// Control module for slc40h vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include "gcAPI.h"

// ==============================================================
// Some vessel parameters
// ==============================================================

//const double ANTENNA_OPERATING_SPEED = 0.8;
const double CHUTE_OPERATING_SPEED = 0.03;
const double PB_FUELMASS = 750.0;
const double PB_ISP = 5e4;
const double PB_MAXHOVERTH = 0;
const double TUBE_COL = 1.0;

VECTOR3 bspos = _V(-41.89, 105.5, 52);
VECTOR3 bspos1 = _V(36.87, 105.5, 53.18);
VECTOR3 bspos2 = _V(37.57, 105.5, -64.79);
VECTOR3 bspos3 = _V(-37.4, 105.5, -66.39);
VECTOR3 bscol = _V(1,1,1);


static const DWORD ntdvtx_geardown = 3;
static TOUCHDOWNVTX tdvtx_geardown[ntdvtx_geardown] = {
	{_V(    0,-0.01,50), 1e6, 1e6, 0.7, 0.7},
	{_V(-100,-0.01,-50), 1e6, 1e6, 0.7, 0.7},
	{_V( 100,-0.01,-50), 1e6, 1e6, 0.7, 0.7}
};

//const VECTOR3 pist_trans = _V(0,-11.557,-11.877);

// Calculate lift coefficient [Cl] as a function of aoa (angle of attack) over -Pi ... Pi
// Implemented here as a piecewise linear function
class slc40h: public VESSEL3 {
public:
	slc40h (OBJHANDLE hVessel, int flightmodel)
		: VESSEL3 (hVessel, flightmodel) {}
	void DefineAnimations (void);
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	void clbkPostStep (double simt, double simdt, double mjd);
	void clbkPostCreation();
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);

	THRUSTER_HANDLE th_hover, th_steam[4];
	ANIMATIONCOMPONENT_HANDLE Section0, Section1, Section2, Section3, Section4, Section5;
	ATTACHMENTHANDLE fh_attach;


private:
	UINT anim_chute, anim_pist, anim_trans;
	UINT anim_umb1_rot, anim_umb1_scale,anim_umb2_rot, anim_umb2_scale,anim_umb3_rot, anim_umb3_scale, anim_umb_col;
	double chute_proc, pist_proc, trans_proc, umb_proc;
	int light_status, lox_status, launch_status, chute_status;
	SpotLight *slca1_light;
	SpotLight *slca2_light;
	BEACONLIGHTSPEC beac, beac1, beac2, beac3;
};
// Define animation sequences for moving parts
// --------------------------------------------------------------
void slc40h::DefineAnimations (void)
{
	// 1. UMBILICALS
	//anim_ant = CreateAnimation (0);
	//static UINT HiGainAnt1Grp[3] = {7,8,9};

	//static MGROUP_ROTATE  HiGainAnt1 (0, HiGainAnt1Grp, 3, _V(3.125,0,3.915), _V(0,1,0), (float)(PI*0.611));
	//AddAnimationComponent (anim_ant, 0.2, 1,   &HiGainAnt1);



	// 2. TOWER 
	anim_chute = CreateAnimation (0);
	static UINT ChuteGrp1[4] = {1,3,15,16};

	static MGROUP_ROTATE  ChuteFold1 (0, ChuteGrp1, 4, _V(0,12.745,-4.411),_V(1,0,0), -30*RAD);
	Section0 = AddAnimationComponent (anim_chute, 0, 1,   &ChuteFold1);

	// claws
	static UINT ChuteGrp2[1] = {18};

	static MGROUP_ROTATE  ChuteFold2 (0, ChuteGrp2, 1, _V(-1.361,0,-1.968),_V(0,1,0), -50*RAD);
	AddAnimationComponent (anim_chute, 0, 0.1,   &ChuteFold2, Section0);

	static UINT ChuteGrp3[1] = {17};

	static MGROUP_ROTATE  ChuteFold3 (0, ChuteGrp3, 1, _V(1.361,0,-1.968),_V(0,1,0), 50*RAD);
	AddAnimationComponent (anim_chute, 0, 0.1,   &ChuteFold3, Section0);


	// pistons
	anim_pist = CreateAnimation (0);
	static UINT ChuteGrp4[1] = {5};

	static MGROUP_ROTATE  ChuteFold4 (0, ChuteGrp4, 1, _V(0,17.856,-18.536),_V(-1,0,0), 90*RAD);
	Section1 = AddAnimationComponent (anim_pist, 0, 1,   &ChuteFold4);

	anim_trans = CreateAnimation (0);
	static UINT ChuteGrp5[1] = {6};

	VECTOR3 pist_trans = _V(0,-14.139,-14.145);

	static MGROUP_TRANSLATE  ChuteFold5 (0, ChuteGrp5, 1, pist_trans);
	AddAnimationComponent (anim_trans, 0, 1,   &ChuteFold5, Section1);


	///// umb1
	
	static UINT UmbGrp1[1] = {0};

	anim_umb1_rot = CreateAnimation (0);
	static MGROUP_ROTATE  Umb1 (0, UmbGrp1, 0, _V(0,71.67,-4.26),_V(1,0,0), -15*RAD);
	Section1 = AddAnimationComponent (anim_umb1_rot, 0, 1,   &Umb1, Section0);

	anim_umb1_scale = CreateAnimation (0);
	static MGROUP_SCALE  Umb1s (0, UmbGrp1, 0, _V(0,71.67,-4.26),_V(1,1,0.15));
	Section2 = AddAnimationComponent (anim_umb1_scale, 0, 1, &Umb1s, Section1);

	static MGROUP_ROTATE  Umb2 (0, UmbGrp1, 0, _V(0,71.67,-4.26),_V(1,0,0), 6*RAD);
	Section3 = AddAnimationComponent (anim_umb1_rot, 0, 1,   &Umb2, Section2);

	/////////////// umb2

	static UINT UmbGrp2[1] = {14};

	anim_umb2_rot = CreateAnimation (0);
	static MGROUP_ROTATE  Umb12 (0, UmbGrp2, 0, _V(0,58.49,-3.74),_V(1,0,0), -15*RAD);
	Section1 = AddAnimationComponent (anim_umb2_rot, 0, 1,   &Umb12, Section0);

	anim_umb2_scale = CreateAnimation (0);
	static MGROUP_SCALE  Umb1s2 (0, UmbGrp2, 0, _V(0,58.49,-3.74),_V(1,1,0.15));
	Section2 = AddAnimationComponent (anim_umb2_scale, 0, 1, &Umb1s2, Section1);

	static MGROUP_ROTATE  Umb22 (0, UmbGrp2, 0, _V(0,58.49,-3.74),_V(1,0,0), 6*RAD);
	Section4 = AddAnimationComponent (anim_umb2_rot, 0, 1,   &Umb22, Section2);

	///////////// umb3

	static UINT UmbGrp3[1] = {13};

	anim_umb3_rot = CreateAnimation (0);
	static MGROUP_ROTATE  Umb13 (0, UmbGrp3, 0, _V(0,49.45,-2.96),_V(1,0,0), -15*RAD);
	Section1 = AddAnimationComponent (anim_umb3_rot, 0, 1,   &Umb13, Section0);

	anim_umb3_scale = CreateAnimation (0);
	static MGROUP_SCALE  Umb1s3 (0, UmbGrp3, 0, _V(0,49.45,-2.90),_V(1,1,0.15));
	Section2 = AddAnimationComponent (anim_umb3_scale, 0, 1, &Umb1s3, Section1);

	static MGROUP_ROTATE  Umb23 (0, UmbGrp3, 0, _V(0,49.45,-2.96),_V(1,0,0), 1*RAD);
	Section5 = AddAnimationComponent (anim_umb3_rot, 0, 1,   &Umb23, Section2);

	/// collapse umb

	anim_umb_col = CreateAnimation (0);

	static UINT UmbGrp4[1] = {0};
	static MGROUP_ROTATE  Umb14 (0, UmbGrp4, 1, _V(0,71.67,-4.26),_V(1,0,0), 45*RAD);
	AddAnimationComponent (anim_umb_col, 0, 1,   &Umb14, Section3);

	static UINT UmbGrp5[1] = {14};
	static MGROUP_ROTATE  Umb15 (0, UmbGrp5, 1, _V(0,58.49,-3.74),_V(1,0,0), 45*RAD);
	AddAnimationComponent (anim_umb_col, 0, 1,   &Umb15, Section4);

	static UINT UmbGrp6[1] = {13};
	static MGROUP_ROTATE  Umb16 (0, UmbGrp6, 1, _V(0,49.45,-2.96),_V(1,0,0), 60*RAD);
	AddAnimationComponent (anim_umb_col, 0, 1,   &Umb16, Section5);



}


// ==============================================================
// slc40h class interface
// ==============================================================

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void slc40h::clbkSetClassCaps (FILEHANDLE cfg)
{
	pist_proc = 0.0;
	trans_proc = 0.0;
	chute_proc = 0.0;
	umb_proc = 0.0;
	chute_status = 0;
	DefineAnimations ();
	gcInitialize();
	light_status = 0;
	lox_status = 0;
	launch_status = 0;
	// physical specs
	SetEmptyMass (1000000);
	SetCW (1, 1, 1, 1);
	SetSize (200);
	SetCrossSections (_V(1000,1000,10000));
	SetPMI (_V(100,100,100));
	//SetTouchdownPoints (_V(0,-0.01,50), _V(-100,-0.01,-50), _V(100,-0.01,-50));
	SetTouchdownPoints (tdvtx_geardown,ntdvtx_geardown);
	SetRotDrag (_V(1,1,1));

	SetMeshVisibilityMode ((AddMesh ("FalconHeavy/slc40h")), MESHVIS_ALWAYS);
	SetCameraOffset (_V(0,10,-20));

	PROPELLANT_HANDLE ph_vernier = CreatePropellantResource (PB_FUELMASS);

	PARTICLESTREAMSPEC contrail_hover = {
		0, 0.2, 15, 0.5, 10, 1.5, 1, 0.5, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC contrail_hover2 = {
		0, 1, 15, 1.5, 10, 2.5, 1.5, 3, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	// big launch smoke
	SURFHANDLE boom1 = oapiRegisterParticleTexture ("Contrail4");

	PARTICLESTREAMSPEC contrail_steam = {
		0, 40, 10, 250, 1.3, 1.5, 16, 3, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC contrail_steam3 = {
		0, 30, 6, 250, 0.8, 0.8, 20, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 4,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1,
		boom1
	};

	if (gcEnabled() == true)
	{
		contrail_steam.atmslowdown = 2.5;
		contrail_steam.lifetime = 2;
		contrail_steam3.atmslowdown = 2.5;
		contrail_steam3.lifetime = 1.1;
	}

// small launch smoke
	PARTICLESTREAMSPEC contrail_steam2 = {
		0, 10, 30, 100, 0.75, 0.5, 6, 3, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	th_hover = CreateThruster (_V(0,0,0), _V(0,0,1), PB_MAXHOVERTH, ph_vernier, PB_ISP);

	//th_steam = CreateThruster (_V(0,0,0), _V(0,0,-1), PB_MAXHOVERTH, ph_vernier, PB_ISP);

	th_steam[0] = CreateThruster (_V(50, 0, 0), _V(-0.966,-0.259,0), PB_MAXHOVERTH, ph_vernier, PB_ISP);
	AddExhaustStream (th_steam[0], &contrail_steam);
	AddExhaustStream (th_steam[0], &contrail_steam3);


	th_steam[1] = CreateThruster (_V(0, 8, 0), _V(0, -1, 0), PB_MAXHOVERTH, ph_vernier, PB_ISP);
	AddExhaustStream (th_steam[1], &contrail_steam2);

	PARTICLESTREAMSPEC contrail_water = {
		0, 0.3, 10, 6, 0.1, 0.75, 2, 0.125, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	if (gcEnabled() == true)
	{
		AddExhaustStream (th_hover, _V(0,40,-1.8), &contrail_hover2);
		AddExhaustStream (th_hover, _V(0,62.2,-1.8), &contrail_hover2);

		contrail_water.srcsize = 0.5;
		contrail_water.srcrate = 12;
		contrail_water.lifetime = 1.25;
	}
	else
	{
		AddExhaustStream (th_hover, _V(0,40,-1.8), &contrail_hover);
		AddExhaustStream (th_hover, _V(0,62.2,-1.8), &contrail_hover);
	}
	

	th_steam[2] = CreateThruster (_V(0, 0, 0), _V(0.966,0.259,0), PB_MAXHOVERTH, ph_vernier, PB_ISP);
	th_steam[3] = CreateThruster (_V(0, 0, 0), _V(-0.966,0.259,0), PB_MAXHOVERTH, ph_vernier, PB_ISP);

	

	AddExhaustStream (th_steam[2], _V( 10.65,16.75,-6.85), &contrail_water);
	AddExhaustStream (th_steam[2], _V( 10.65,16.75,-2.12), &contrail_water);
	AddExhaustStream (th_steam[2], _V( 10.65,16.75, 2.5), &contrail_water);
	AddExhaustStream (th_steam[2], _V( 10.65,16.75, 7.3), &contrail_water);
	AddExhaustStream (th_steam[3], _V(-10.65,16.75,-6.85), &contrail_water);
	AddExhaustStream (th_steam[3], _V(-10.65,16.75,-2.12), &contrail_water);
	AddExhaustStream (th_steam[3], _V(-10.65,16.75, 2.5), &contrail_water);
	AddExhaustStream (th_steam[3], _V(-10.65,16.75, 7.3), &contrail_water);


	// ***************** thruster definitions *******************

	COLOUR4 col_d = {1,1,0.9,0};
	COLOUR4 col_s = {1,1,0.9,0};
	COLOUR4 col_a = {0,0,0,0};
	COLOUR4 col_white = {1,1,1,0};
	slca1_light = (SpotLight*)AddSpotLight(_V(50,25,50), _V(-0.612,0.5,-0.612), 5000, 1.8e-4, 0, 1.8e-4, RAD*90, RAD*120, col_d, col_s, col_a);
	slca1_light->Activate(false);

	slca2_light = (SpotLight*)AddSpotLight(_V(-60,30,-60), _V(0.612,0.5,0.612), 5000, 1.8e-4, 0, 1.8e-4, RAD*90, RAD*120, col_d, col_s, col_a);
	slca2_light->Activate(false);

	fh_attach = CreateAttachment (false, _V( 0, 13.6, 0), _V( 0,1,0), _V(0,0,-1), "FH");

	// beacons
	beac.shape = BEACONSHAPE_STAR;
	beac.pos = &bspos;
	beac.col = &bscol;
	beac.size = 0.5;
	beac.falloff = 0.5;
	beac.period = 4;
	beac.duration = 0.1;
	beac.tofs = 0;
	beac.active = false;
	AddBeacon (&beac);

	beac1.shape = BEACONSHAPE_STAR;
	beac1.pos = &bspos1;
	beac1.col = &bscol;
	beac1.size = 0.5;
	beac1.falloff = 0.5;
	beac1.period = 4;
	beac1.duration = 0.1;
	beac1.tofs = 0;
	beac1.active = false;
	AddBeacon (&beac1);

	beac2.shape = BEACONSHAPE_STAR;
	beac2.pos = &bspos2;
	beac2.col = &bscol;
	beac2.size = 0.5;
	beac2.falloff = 0.5;
	beac2.period = 4;
	beac2.duration = 0.1;
	beac2.tofs = 0;
	beac2.active = false;
	AddBeacon (&beac2);

	beac3.shape = BEACONSHAPE_STAR;
	beac3.pos = &bspos3;
	beac3.col = &bscol;
	beac3.size = 0.5;
	beac3.falloff = 0.5;
	beac3.period = 4;
	beac3.duration = 0.1;
	beac3.tofs = 0;
	beac3.active = false;
	AddBeacon (&beac3);

}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new slc40h (hvessel, flightmodel);
}
// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (slc40h*)vessel;
}
// ==============================================================
// Overloaded clbkPostCreation callback functions
// ==============================================================

void slc40h::clbkPostCreation ()
{
	/*if (launch_status < 3)
	{
	umb_proc = 1-(chute_proc*2);
	if(umb_proc < 0)umb_proc = 0;
	SetAnimation (anim_umb1_rot, umb_proc);
	SetAnimation (anim_umb1_scale, umb_proc);
	SetAnimation (anim_umb2_rot, umb_proc);
	SetAnimation (anim_umb2_scale, umb_proc);
	SetAnimation (anim_umb3_rot, umb_proc);
	SetAnimation (anim_umb3_scale, umb_proc);
	}
	else 
	{
	SetAnimation (anim_umb1_rot, 0);
	SetAnimation (anim_umb1_scale, umb_proc);
	SetAnimation (anim_umb2_rot, 0);
	SetAnimation (anim_umb2_scale, umb_proc);
	SetAnimation (anim_umb3_rot, 0);
	SetAnimation (anim_umb3_scale, umb_proc);
	SetAnimation (anim_umb_col,umb_proc);
	}*/
}
void slc40h::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState (scn);

	// custom parameters
	sprintf (cbuf, "%0.4f %0.4f %d", pist_proc, trans_proc, launch_status);
	oapiWriteScenario_string (scn, "PST", cbuf);
	sprintf (cbuf, "%d %0.4f %0.4f", chute_status, chute_proc, umb_proc);
	oapiWriteScenario_string (scn, "TOWER", cbuf);
}
void slc40h::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "PST", 3)) {
			sscanf (line+3, "%lf%lf%d", &pist_proc, &trans_proc, &launch_status);
		} else if (!strnicmp (line, "TOWER", 5)) {
			sscanf (line+5, "%d%lf%lf", &chute_status, &chute_proc, &umb_proc);
		}else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }
	SetAnimation (anim_pist, pist_proc);
	SetAnimation (anim_trans, trans_proc);
	SetAnimation (anim_chute, chute_proc);
	if (launch_status < 3)
	{
	SetAnimation (anim_umb1_rot, umb_proc);
	SetAnimation (anim_umb1_scale, umb_proc);
	SetAnimation (anim_umb2_rot, umb_proc);
	SetAnimation (anim_umb2_scale, umb_proc);
	SetAnimation (anim_umb3_rot, umb_proc);
	SetAnimation (anim_umb3_scale, umb_proc);
	}
	else
	{
	SetAnimation (anim_umb1_rot, 0);
	SetAnimation (anim_umb1_scale, umb_proc);
	SetAnimation (anim_umb2_rot, 0);
	SetAnimation (anim_umb2_scale, umb_proc);
	SetAnimation (anim_umb3_rot, 0);
	SetAnimation (anim_umb3_scale, umb_proc);
	SetAnimation (anim_umb_col,umb_proc);
	}
}


bool AddPayload(void *id, char *str, void *data)
{
	char cbuf[256];
	sscanf(str, "%s", &cbuf);
	OBJHANDLE hPld = oapiGetObjectByName(cbuf);
	if (hPld == NULL)return false;
	else
	{
	VESSEL2* vme = (VESSEL2*)oapiGetVesselInterface(oapiGetFocusObject ());
	//DWORD ad = vme->AttachmentCount(false);
	//if (ad == 0) return false;

	ATTACHMENTHANDLE hPldAtt;
	hPldAtt = vme->GetAttachmentHandle(false, 0);
	if(vme->GetAttachmentStatus(hPldAtt) != NULL)return false;

	VESSEL2* v = (VESSEL2*)oapiGetVesselInterface(hPld);
	DWORD ac = v->AttachmentCount(true);
	if (ac == 0) return false;

	ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle(true, 0);
	if(v->GetAttachmentStatus(hAtt) != NULL)return false;

	
	if (vme->AttachChild(hPld, hPldAtt, hAtt) == true) return true;
	}

	return false;
}


int slc40h::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) {
  if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) {

  } else { // unmodified keys
    switch (key) {
	case OAPI_KEY_G: // move mss
		if(chute_status == 0)chute_status = 1;
		else if(chute_status == 1)chute_status = 3;
		else if(chute_status == 2)chute_status = 3;
		else if(chute_status == 3)chute_status = 1;
		else if(chute_status == 5)chute_status = 3;
		return 1;
	case OAPI_KEY_M: // noise sup water
		SetThrusterLevel (th_steam[2], 1);
		SetThrusterLevel (th_steam[3], 1);
		return 1;
	case OAPI_KEY_K: // lights on/off
		if (light_status == 0) light_status = 1;
		else if (light_status == 1) light_status = 0;
		return 1;
	case OAPI_KEY_V: // LOX vent on/off
		if (lox_status == 0)lox_status = 1;
		else lox_status = 0;
		return 1;

	case OAPI_KEY_P:
		oapiOpenInputBox("Enter Launcher by Name",AddPayload, 0, 50, 0);
		return 1;

	}
  }
  return 0;
}
void slc40h::clbkPreStep(double SimT, double SimDT, double MJD) {
	// next block of code
	ENGINESTATUS es;
	VECTOR3 gpos = _V(0,0,0);
	VECTOR3 gpad = _V(0,0,0);
	Local2Global(_V(0,40,0),gpad);

	es.main = 0;
	
	if (lox_status == 0)
{
	for (DWORD i = 0; i < oapiGetVesselCount(); i++)
	{
	OBJHANDLE hV = oapiGetVesselByIndex (i);

	if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
	oapiGetGlobalPos (hV, &gpos);

	if (dist (gpos, gpad) < 50)
	{ // in range
		SetThrusterLevel (th_hover, 1);
		VESSEL *v = oapiGetVesselInterface (hV);
		oapiGetEngineStatus(hV, &es);
		if (es.main > 0) 
		{
			SetThrusterLevel (th_hover, 0);
			break;
		}
	}

	}
}
	else SetThrusterLevel (th_hover, 0);
	if(oapiGetTimeAcceleration() > 10)SetThrusterLevel (th_hover, 0);
	
	if (es.main > 0 ) 
	{
		SetThrusterLevel (th_steam[0], 1);
		SetThrusterLevel (th_steam[1], 1);
		SetThrusterLevel (th_steam[2], 1);
		SetThrusterLevel (th_steam[3], 1);
		launch_status = 1;
		if (es.main > 0.9 )launch_status = 2;
	}
	else 
	{
		SetThrusterLevel (th_steam[0], 0);
		SetThrusterLevel (th_steam[1], 0);
	}

	if(oapiGetTimeAcceleration() > 10)
	{
		SetThrusterLevel (th_hover, 0);
		SetThrusterLevel (th_steam[0], 0);
		SetThrusterLevel (th_steam[1], 0);
		SetThrusterLevel (th_steam[2], 0);
		SetThrusterLevel (th_steam[3], 0);
	}
	if (es.main > 0.95 )
	{
		DetachChild(fh_attach, 0.0);
		launch_status = 3;
		chute_status = 4;
		SetThrusterLevel (th_steam[2], 0);
		SetThrusterLevel (th_steam[3], 0);
	}

/////////////////////shadowcheck
	VECTOR3 rsuninv;
	VECTOR3 rsunloc;
	VECTOR3 rsun;
	GetGlobalPos(rsun);
	rsuninv = _V(-rsun.x, -rsun.y, -rsun.z);
	Global2Local(rsuninv,rsunloc);

	if (slca1_light->IsActive()== false && rsunloc.y < 0)
			{
			slca1_light->Activate(true);
			slca2_light->Activate(true);
			beac.active = true;
			beac1.active = true;
			beac2.active = true;
			beac3.active = true;
			}
	else if (slca1_light->IsActive()== true && rsunloc.y > 0)
			{
			slca1_light->Activate(false);
			slca2_light->Activate(false);
			beac.active = false;
			beac1.active = false;
			beac2.active = false;
			beac3.active = false;
			}

	if (light_status == 1)
			{
			slca1_light->Activate(false);
			slca2_light->Activate(false);
			}
	if (launch_status == 1)
			{
			slca1_light->SetIntensity(1-(es.main));
			slca2_light->SetIntensity(1-(es.main));
			}
	else if (launch_status > 1)
			{
			slca1_light->Activate(false);
			slca2_light->Activate(false);
			}
	////////////////////////////////////////// ANIMATIONS
	
		// Animate chutes
	if (chute_status == 0) 
	{
		chute_proc = 0;
		SetAnimation (anim_chute, chute_proc);
	}
	else if (chute_status == 1)
	{
		double da = SimDT*CHUTE_OPERATING_SPEED*0.2;
		chute_proc = chute_proc+da;
		if (chute_proc > 0.1)
		{
			chute_proc = 0.1;
			chute_status = 2;
		}
		SetAnimation (anim_chute, chute_proc);
	}
	else if (chute_status == 2)
	{
		chute_proc = 0.1;
		SetAnimation (anim_chute, chute_proc);
	}
	else if (chute_status == 3)
	{
		double da = SimDT*CHUTE_OPERATING_SPEED*0.2;
		chute_proc = chute_proc-da;
		if (chute_proc < 0)
		{
			chute_proc = 0;
			chute_status = 0;
		}
		SetAnimation (anim_chute, chute_proc);
	}
	else if (chute_status == 4)
	{
		double da = SimDT*CHUTE_OPERATING_SPEED*10;
		chute_proc = chute_proc+da;
		if (chute_proc > 1)
		{
			chute_proc = 1;
			chute_status = 5;
		}
		SetAnimation (anim_chute, chute_proc);
	}

	if (launch_status < 3)
	{
	umb_proc = 1-(2*chute_proc);
	if(umb_proc < 0)umb_proc = 0;
	SetAnimation (anim_umb1_rot, umb_proc);
	SetAnimation (anim_umb1_scale, umb_proc);
	SetAnimation (anim_umb2_rot, umb_proc);
	SetAnimation (anim_umb2_scale, umb_proc);
	SetAnimation (anim_umb3_rot, umb_proc);
	SetAnimation (anim_umb3_scale, umb_proc);
	}
	else if(chute_status == 3)
	{
	umb_proc = 1-(2*chute_proc);
	if(umb_proc < 0)umb_proc = 0;
	SetAnimation (anim_umb1_rot, umb_proc);
	SetAnimation (anim_umb1_scale, umb_proc);
	SetAnimation (anim_umb2_rot, umb_proc);
	SetAnimation (anim_umb2_scale, umb_proc);
	SetAnimation (anim_umb3_rot, umb_proc);
	SetAnimation (anim_umb3_scale, umb_proc);
	}
	else 
	{
	double dc = SimDT * TUBE_COL * 0.5;
	umb_proc = umb_proc+dc;
	if (umb_proc > 1)umb_proc = 1;
	SetAnimation (anim_umb1_rot, 0);
	SetAnimation (anim_umb1_scale, umb_proc);
	SetAnimation (anim_umb2_rot, 0);
	SetAnimation (anim_umb2_scale, umb_proc);
	SetAnimation (anim_umb3_rot, 0);
	SetAnimation (anim_umb3_scale, umb_proc);
	SetAnimation (anim_umb_col,umb_proc);
	}

	double theta = chute_proc*30*RAD;
	double sin_theta = sin(theta);
	double cos_theta = cos(theta);

	VECTOR3 pist_low = _V(0,17.856,-18.536);
	VECTOR3 pist_top = _V(0,29.556,-6.831);
	VECTOR3 twr_low = _V(0,12.745,-4.411);
	pist_low = pist_low - twr_low;
	pist_top = pist_top - twr_low;

	VECTOR3 pist_vect_orig = pist_top-pist_low;
	double pist_angle_orig = atan2(pist_vect_orig.y,pist_vect_orig.z);

	VECTOR3 pist_top_curr = _V(0,0,0);

	pist_top_curr.z = (pist_top.z*cos_theta) - (pist_top.y*sin_theta);
	pist_top_curr.y = (pist_top.z*sin_theta) + (pist_top.y*cos_theta);

	VECTOR3 pist_vect_curr = pist_top_curr-pist_low;
	double pist_angle_curr = atan2(pist_vect_curr.y,pist_vect_curr.z);

	double pist_angle_diff = DEG*(pist_angle_curr - pist_angle_orig);
	pist_proc = pist_angle_diff/90;

	double length_diff = length(pist_vect_orig)-length(pist_vect_curr);
	trans_proc = length_diff/length(_V(0,-14.139,-14.145));
	SetAnimation (anim_pist, pist_proc);
	SetAnimation (anim_trans, trans_proc);

	}

void slc40h::clbkPostStep (double simt, double simdt, double mjd)
{
	
	//	// Animate chutes
	//if (chute_status >= DOOR_CLOSING) {
	//	double da = simdt * CHUTE_OPERATING_SPEED;
	//	if (chute_status == DOOR_CLOSING) {
	//		if (chute_proc > 0.0) chute_proc = max (0.0, chute_proc-da);
	//		else                chute_status = DOOR_CLOSED;
	//	} else {
	//		if (chute_proc < 1.0) chute_proc = min (1.0, chute_proc+da);
	//		else                chute_status = DOOR_OPEN;
	//	}
	//	SetAnimation (anim_chute, chute_proc);
	//}

	//if (launch_status < 3)
	//{
	//umb_proc = 1-chute_proc;
	//SetAnimation (anim_umb1_rot, umb_proc);
	//SetAnimation (anim_umb1_scale, umb_proc);
	//SetAnimation (anim_umb2_rot, umb_proc);
	//SetAnimation (anim_umb2_scale, umb_proc);
	//SetAnimation (anim_umb3_rot, umb_proc);
	//SetAnimation (anim_umb3_scale, umb_proc);
	//}
	//else 
	//{
	//double dc = simdt * TUBE_COL;
	//umb_proc = umb_proc+dc;
	//if (umb_proc > 1)umb_proc = 1;
	//SetAnimation (anim_umb1_rot, 0);
	//SetAnimation (anim_umb1_scale, umb_proc);
	//SetAnimation (anim_umb2_rot, 0);
	//SetAnimation (anim_umb2_scale, umb_proc);
	//SetAnimation (anim_umb3_rot, 0);
	//SetAnimation (anim_umb3_scale, umb_proc);
	//SetAnimation (anim_umb_col,umb_proc);
	//}

	//double theta = chute_proc*15*RAD;
	//double sin_theta = sin(theta);
	//double cos_theta = cos(theta);

	//VECTOR3 pist_low = _V(0,17.856,-18.536);
	//VECTOR3 pist_top = _V(0,29.556,-6.831);
	//VECTOR3 twr_low = _V(0,12.745,-4.411);
	//pist_low = pist_low - twr_low;
	//pist_top = pist_top - twr_low;

	//VECTOR3 pist_vect_orig = pist_top-pist_low;
	//double pist_angle_orig = atan2(pist_vect_orig.y,pist_vect_orig.z);

	//VECTOR3 pist_top_curr = _V(0,0,0);

	//pist_top_curr.z = (pist_top.z*cos_theta) - (pist_top.y*sin_theta);
	//pist_top_curr.y = (pist_top.z*sin_theta) + (pist_top.y*cos_theta);

	//VECTOR3 pist_vect_curr = pist_top_curr-pist_low;
	//double pist_angle_curr = atan2(pist_vect_curr.y,pist_vect_curr.z);

	//double pist_angle_diff = DEG*(pist_angle_curr - pist_angle_orig);
	//pist_proc = pist_angle_diff/90;

	//double length_diff = length(pist_vect_orig)-length(pist_vect_curr);
	//trans_proc = length_diff/length(_V(0,-14.139,-14.145));
	//SetAnimation (anim_pist, pist_proc);
	//SetAnimation (anim_trans, trans_proc);
}